@@ -63,7 +63,7 @@ glfs_iatt_from_stat (struct stat *stat, int valid, struct iatt *iatt,
 
 struct glfs_object *
 pub_glfs_h_lookupat (struct glfs *fs, struct glfs_object *parent,
-                     const char *path, struct stat *stat)
+                     const char *path, struct stat *stat, int follow)
 {
         int                      ret = 0;
         xlator_t                *subvol = NULL;
@@ -100,7 +100,7 @@ pub_glfs_h_lookupat (struct glfs *fs, struct glfs_object *parent,
 
         /* fop/op */
         ret = glfs_resolve_at (fs, subvol, inode, path, &loc, &iatt,
-                                    0 /*TODO: links? */, 0);
+                                    follow, 0);
 
         /* populate out args */
         if (!ret) {
@@ -124,7 +124,16 @@ invalid_fs:
         return object;
 }
 
-GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_h_lookupat, 3.4.2);
+GFAPI_SYMVER_PUBLIC_DEFAULT(glfs_h_lookupat, 3.7.4);
+
+struct glfs_object *
+pub_glfs_h_lookupat34 (struct glfs *fs, struct glfs_object *parent,
+                       const char *path, struct stat *stat)
+{
+        return pub_glfs_h_lookupat (fs, parent, path, stat, 0);
+}
+
+GFAPI_SYMVER_PUBLIC(glfs_h_lookupat34, glfs_h_lookupat, 3.4.2);
 
 int
 pub_glfs_h_statfs (struct glfs *fs, struct glfs_object *object,
@@ -1929,10 +1938,10 @@ pub_glfs_h_poll_upcall (struct glfs *fs, struct callback_arg *up_arg)
                          *
                          * Applications will ignore this notification
                          * as up_arg->object will be NULL */
-                        gf_log (subvol->name, GF_LOG_WARNING,
-                                "handle creation of %s failed: %s",
-                                uuid_utoa (upcall_data->gfid),
-                                strerror (errno));
+                        gf_msg (subvol->name, GF_LOG_WARNING, errno,
+                                API_MSG_CREATE_HANDLE_FAILED,
+                                "handle creation of %s failed",
+                                uuid_utoa (upcall_data->gfid));
 
                         reason = GFAPI_CBK_EVENT_NULL;
                         break;
