@@ -25,10 +25,11 @@
 
 #include "glfs-internal.h"
 #include "glfs-mem-types.h"
+#include "gfapi-messages.h"
 
 
 int
-glfs_graph_setup (struct glfs *fs, glusterfs_graph_t *graph)
+graph_setup (struct glfs *fs, glusterfs_graph_t *graph)
 {
 	xlator_t      *new_subvol = NULL;
 	xlator_t      *old_subvol = NULL;
@@ -88,23 +89,37 @@ notify (xlator_t *this, int event, void *data, ...)
 
 	switch (event) {
 	case GF_EVENT_GRAPH_NEW:
-		gf_log (this->name, GF_LOG_INFO, "New graph %s (%d) coming up",
+		gf_msg (this->name, GF_LOG_INFO, 0, API_MSG_NEW_GRAPH,
+                        "New graph %s (%d) coming up",
 			uuid_utoa ((unsigned char *)graph->graph_uuid),
 			graph->id);
 		break;
 	case GF_EVENT_CHILD_UP:
-		glfs_graph_setup (fs, graph);
+                pthread_mutex_lock (&fs->mutex);
+                {
+                        graph->used = 1;
+                }
+                pthread_mutex_unlock (&fs->mutex);
+		graph_setup (fs, graph);
 		glfs_init_done (fs, 0);
 		break;
 	case GF_EVENT_CHILD_DOWN:
-		glfs_graph_setup (fs, graph);
+                pthread_mutex_lock (&fs->mutex);
+                {
+                        graph->used = 0;
+                        pthread_cond_broadcast (&fs->child_down_cond);
+                }
+                pthread_mutex_unlock (&fs->mutex);
+		graph_setup (fs, graph);
 		glfs_init_done (fs, 1);
 		break;
 	case GF_EVENT_CHILD_CONNECTING:
 		break;
+        case GF_EVENT_UPCALL:
+                glfs_process_upcall_event (fs, data);
+                break;
 	default:
-		gf_log (this->name, GF_LOG_DEBUG,
-			"got notify event %d", event);
+		gf_msg_debug (this->name, 0, "got notify event %d", event);
 		break;
 	}
 
@@ -122,7 +137,8 @@ mem_acct_init (xlator_t *this)
 
 	ret = xlator_mem_acct_init (this, glfs_mt_end + 1);
 	if (ret) {
-		gf_log (this->name, GF_LOG_ERROR, "Failed to initialise "
+		gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                        API_MSG_MEM_ACCT_INIT_FAILED, "Failed to initialise "
                         "memory accounting");
 		return ret;
 	}
