@@ -29,6 +29,7 @@
 #include "common-utils.h"
 #include "syncop.h"
 #include "call-stub.h"
+#include "gfapi-messages.h"
 
 #include "glfs-internal.h"
 
@@ -48,8 +49,9 @@ glfs_first_lookup_safe (xlator_t *subvol)
 	loc.name = "";
 
 	ret = syncop_lookup (subvol, &loc, 0, 0, 0, 0);
+        DECODE_SYNCOP_ERR (ret);
 
-	gf_log (subvol->name, GF_LOG_DEBUG, "first lookup complete %d", ret);
+	gf_msg_debug (subvol->name, 0, "first lookup complete %d", ret);
 
 	return ret;
 }
@@ -73,34 +75,53 @@ __glfs_first_lookup (struct glfs *fs, xlator_t *subvol)
 }
 
 
+/**
+ * We have to check if need_lookup flag is set in both old and the new inodes.
+ * If its set in oldinode, then directly go ahead and do an explicit lookup.
+ * But if its not set in the oldinode, then check if the newinode is linked
+ * via readdirp. If so an explicit lookup is needed on the new inode, so that
+ * below xlators can set their respective contexts.
+ */
 inode_t *
-glfs_refresh_inode_safe (xlator_t *subvol, inode_t *oldinode)
+glfs_refresh_inode_safe (xlator_t *subvol, inode_t *oldinode,
+                         gf_boolean_t need_lookup)
 {
 	loc_t        loc = {0, };
 	int          ret = -1;
 	struct iatt  iatt = {0, };
 	inode_t     *newinode = NULL;
+        gf_boolean_t lookup_needed = _gf_false;
 
 
 	if (!oldinode)
 		return NULL;
 
-	if (oldinode->table->xl == subvol)
+	if (!need_lookup && oldinode->table->xl == subvol)
 		return inode_ref (oldinode);
 
 	newinode = inode_find (subvol->itable, oldinode->gfid);
-	if (newinode)
-		return newinode;
+	if (!need_lookup && newinode) {
+
+                lookup_needed = inode_needs_lookup (newinode, THIS);
+                if (!lookup_needed)
+                        return newinode;
+        }
+
+	gf_uuid_copy (loc.gfid, oldinode->gfid);
+        if (!newinode)
+                loc.inode = inode_new (subvol->itable);
+        else
+                loc.inode = newinode;
 
-	uuid_copy (loc.gfid, oldinode->gfid);
-	loc.inode = inode_new (subvol->itable);
 	if (!loc.inode)
 		return NULL;
 
-	ret = syncop_lookup (subvol, &loc, 0, &iatt, 0, 0);
+	ret = syncop_lookup (subvol, &loc, &iatt, 0, 0, 0);
+        DECODE_SYNCOP_ERR (ret);
 
 	if (ret) {
-		gf_log (subvol->name, GF_LOG_WARNING,
+		gf_msg (subvol->name, GF_LOG_WARNING, errno,
+                        API_MSG_INODE_REFRESH_FAILED,
 			"inode refresh of %s failed: %s",
 			uuid_utoa (oldinode->gfid), strerror (errno));
 		loc_wipe (&loc);
@@ -118,14 +139,15 @@ glfs_refresh_inode_safe (xlator_t *subvol, inode_t *oldinode)
 
 
 inode_t *
-__glfs_refresh_inode (struct glfs *fs, xlator_t *subvol, inode_t *inode)
+__glfs_refresh_inode (struct glfs *fs, xlator_t *subvol, inode_t *inode,
+                      gf_boolean_t need_lookup)
 {
 	inode_t *newinode = NULL;
 
 	fs->migration_in_progress = 1;
 	pthread_mutex_unlock (&fs->mutex);
 	{
-		newinode = glfs_refresh_inode_safe (subvol, inode);
+		newinode = glfs_refresh_inode_safe (subvol, inode, need_lookup);
 	}
 	pthread_mutex_lock (&fs->mutex);
 	fs->migration_in_progress = 0;
@@ -135,34 +157,20 @@ __glfs_refresh_inode (struct glfs *fs, xlator_t *subvol, inode_t *inode)
 }
 
 int
-glfs_loc_touchup (loc_t *loc)
+priv_glfs_loc_touchup (loc_t *loc)
 {
-	char *path = NULL;
-	int   ret = -1;
-	char *bn = NULL;
+        int     ret = 0;
 
-	if (loc->parent)
-		ret = inode_path (loc->parent, loc->name, &path);
-	else
-		ret = inode_path (loc->inode, 0, &path);
+        ret = loc_touchup (loc, loc->name);
+        if (ret < 0) {
+                errno = -ret;
+                ret = -1;
+        }
 
-	loc->path = path;
-
-	if (ret < 0 || !path) {
-		ret = -1;
-		errno = ENOMEM;
-		goto out;
-	}
-
-	bn = strrchr (path, '/');
-	if (bn)
-		bn++;
-	loc->name = bn;
-	ret = 0;
-out:
-	return ret;
+        return ret;
 }
 
+GFAPI_SYMVER_PRIVATE_DEFAULT(glfs_loc_touchup, 3.4.0);
 
 int
 glfs_resolve_symlink (struct glfs *fs, xlator_t *subvol, inode_t *inode,
@@ -174,13 +182,14 @@ glfs_resolve_symlink (struct glfs *fs, xlator_t *subvol, inode_t *inode,
 	int    ret = -1;
 
 	loc.inode = inode_ref (inode);
-	uuid_copy (loc.gfid, inode->gfid);
+	gf_uuid_copy (loc.gfid, inode->gfid);
 	ret = inode_path (inode, NULL, &rpath);
 	if (ret < 0)
 		goto out;
 	loc.path = rpath;
 
-	ret = syncop_readlink (subvol, &loc, &path, 4096);
+	ret = syncop_readlink (subvol, &loc, &path, 4096, NULL, NULL);
+        DECODE_SYNCOP_ERR (ret);
 
 	if (ret < 0)
 		goto out;
@@ -202,14 +211,15 @@ glfs_resolve_base (struct glfs *fs, xlator_t *subvol, inode_t *inode,
 	char       *path = NULL;
 
 	loc.inode = inode_ref (inode);
-	uuid_copy (loc.gfid, inode->gfid);
+	gf_uuid_copy (loc.gfid, inode->gfid);
 
 	ret = inode_path (loc.inode, NULL, &path);
 	loc.path = path;
 	if (ret < 0)
 		goto out;
 
-	ret = syncop_lookup (subvol, &loc, NULL, iatt, NULL, NULL);
+	ret = syncop_lookup (subvol, &loc, iatt, NULL, NULL, NULL);
+        DECODE_SYNCOP_ERR (ret);
 out:
 	loc_wipe (&loc);
 
@@ -234,18 +244,30 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 	loc.name = component;
 
 	loc.parent = inode_ref (parent);
-	uuid_copy (loc.pargfid, parent->gfid);
+	gf_uuid_copy (loc.pargfid, parent->gfid);
 
+        /* /.. and /. should point back to /
+           we lookup using inode and gfid of root
+           Fill loc.name so that we make use md-cache.
+           md-cache is not valid for nameless lookups.
+        */
+        if (__is_root_gfid (parent->gfid) &&
+            (strcmp (component, "..") == 0)) {
+                loc.inode = inode_ref (parent);
+                loc.name = ".";
+        } else {
+                if (strcmp (component, ".") == 0)
+                        loc.inode = inode_ref (parent);
+                else if (strcmp (component, "..") == 0)
+                        loc.inode = inode_parent (parent, 0, 0);
+                else
+                        loc.inode = inode_grep (parent->table, parent,
+                                                component);
+        }
 
-	if (strcmp (component, ".") == 0)
-		loc.inode = inode_ref (parent);
-	else if (strcmp (component, "..") == 0)
-		loc.inode = inode_parent (parent, 0, 0);
-	else
-		loc.inode = inode_grep (parent->table, parent, component);
 
 	if (loc.inode) {
-		uuid_copy (loc.gfid, loc.inode->gfid);
+		gf_uuid_copy (loc.gfid, loc.inode->gfid);
 		reval = 1;
 
 		if (!force_lookup) {
@@ -254,21 +276,42 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 			goto found;
 		}
 	} else {
-		uuid_generate (gfid);
+		gf_uuid_generate (gfid);
 		loc.inode = inode_new (parent->table);
+                if (!loc.inode) {
+                        errno = ENOMEM;
+                        goto out;
+                }
+
+                xattr_req = dict_new ();
+                if (!xattr_req) {
+                        errno = ENOMEM;
+                        goto out;
+                }
+
+                ret = dict_set_static_bin (xattr_req, "gfid-req", gfid, 16);
+                if (ret) {
+                        errno = ENOMEM;
+                        goto out;
+                }
+
 	}
 
-	if (!loc.inode)
-		goto out;
-
-	glret = glfs_loc_touchup (&loc);
+	glret = priv_glfs_loc_touchup (&loc);
 	if (glret < 0) {
 		ret = -1;
 		goto out;
 	}
 
-	ret = syncop_lookup (subvol, &loc, NULL, &ciatt, NULL, NULL);
-	if (ret && reval) {
+        ret = syncop_lookup (subvol, &loc, &ciatt, NULL, xattr_req, NULL);
+        if (ret && reval) {
+                /*
+                 * A stale mapping might exist for a dentry/inode that has been
+                 * removed from another client.
+                 */
+                if (-ret == ENOENT)
+                        inode_unlink(loc.inode, loc.parent,
+                                     loc.name);
 		inode_unref (loc.inode);
 		loc.inode = inode_new (parent->table);
 		if (!loc.inode) {
@@ -282,7 +325,7 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 			goto out;
 		}
 
-		uuid_generate (gfid);
+		gf_uuid_generate (gfid);
 
 		ret = dict_set_static_bin (xattr_req, "gfid-req", gfid, 16);
 		if (ret) {
@@ -290,9 +333,10 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 			goto out;
 		}
 
-		ret = syncop_lookup (subvol, &loc, xattr_req, &ciatt,
-				     NULL, NULL);
+		ret = syncop_lookup (subvol, &loc, &ciatt, NULL,
+				     xattr_req, NULL);
 	}
+        DECODE_SYNCOP_ERR (ret);
 	if (ret)
 		goto out;
 
@@ -313,7 +357,7 @@ out:
 
 
 int
-glfs_resolve_at (struct glfs *fs, xlator_t *subvol, inode_t *at,
+priv_glfs_resolve_at (struct glfs *fs, xlator_t *subvol, inode_t *at,
 		 const char *origpath, loc_t *loc, struct iatt *iatt,
 		 int follow, int reval)
 {
@@ -363,8 +407,10 @@ glfs_resolve_at (struct glfs *fs, xlator_t *subvol, inode_t *at,
 						*/
 						(reval || (!next_component &&
 						iatt)));
-		if (!inode)
+		if (!inode) {
+                        ret = -1;
 			break;
+                }
 
 		if (IA_ISLNK (ciatt.ia_type) && (next_component || follow)) {
 			/* If the component is not the last piece,
@@ -390,7 +436,7 @@ glfs_resolve_at (struct glfs *fs, xlator_t *subvol, inode_t *at,
 			if (ret < 0)
 				break;
 
-			ret = glfs_resolve_at (fs, subvol, parent, lpath,
+			ret = priv_glfs_resolve_at (fs, subvol, parent, lpath,
 					       &sym_loc,
 					       /* followed iatt becomes the
 						  component iatt
@@ -432,19 +478,21 @@ glfs_resolve_at (struct glfs *fs, xlator_t *subvol, inode_t *at,
 
 	loc->parent = parent;
 	if (parent) {
-		uuid_copy (loc->pargfid, parent->gfid);
+		gf_uuid_copy (loc->pargfid, parent->gfid);
 		loc->name = component;
 	}
 
 	loc->inode = inode;
 	if (inode) {
-		uuid_copy (loc->gfid, inode->gfid);
+		gf_uuid_copy (loc->gfid, inode->gfid);
 		if (iatt)
 			*iatt = ciatt;
 		ret = 0;
 	}
 
-	glfs_loc_touchup (loc);
+        if (priv_glfs_loc_touchup (loc) < 0) {
+                ret = -1;
+        }
 out:
 	GF_FREE (path);
 
@@ -453,6 +501,8 @@ out:
 	return ret;
 }
 
+GFAPI_SYMVER_PRIVATE_DEFAULT(glfs_resolve_at, 3.4.0);
+
 
 int
 glfs_resolve_path (struct glfs *fs, xlator_t *subvol, const char *origpath,
@@ -462,22 +512,29 @@ glfs_resolve_path (struct glfs *fs, xlator_t *subvol, const char *origpath,
 	inode_t *cwd = NULL;
 
 	if (origpath[0] == '/')
-		return glfs_resolve_at (fs, subvol, NULL, origpath, loc, iatt,
-					follow, reval);
+		return priv_glfs_resolve_at (fs, subvol, NULL, origpath, loc,
+                                             iatt, follow, reval);
 
 	cwd = glfs_cwd_get (fs);
+        if (NULL == cwd) {
+                gf_msg (subvol->name, GF_LOG_WARNING, EIO,
+                        API_MSG_GET_CWD_FAILED, "Failed to get cwd");
+                errno = EIO;
+                goto out;
+        }
 
-	ret = glfs_resolve_at (fs, subvol, cwd, origpath, loc, iatt,
-			       follow, reval);
+	ret = priv_glfs_resolve_at (fs, subvol, cwd, origpath, loc, iatt,
+                                    follow, reval);
 	if (cwd)
 		inode_unref (cwd);
 
+out:
 	return ret;
 }
 
 
 int
-glfs_resolve (struct glfs *fs, xlator_t *subvol, const char *origpath,
+priv_glfs_resolve (struct glfs *fs, xlator_t *subvol, const char *origpath,
 	      loc_t *loc, struct iatt *iatt, int reval)
 {
 	int ret = -1;
@@ -486,7 +543,7 @@ glfs_resolve (struct glfs *fs, xlator_t *subvol, const char *origpath,
 
 	return ret;
 }
-
+GFAPI_SYMVER_PRIVATE_DEFAULT(glfs_resolve, 3.7.0);
 
 int
 glfs_lresolve (struct glfs *fs, xlator_t *subvol, const char *origpath,
@@ -514,9 +571,11 @@ glfs_migrate_fd_locks_safe (struct glfs *fs, xlator_t *oldsubvol, fd_t *oldfd,
 	newfd->lk_ctx = fd_lk_ctx_ref (oldfd->lk_ctx);
 
 	ret = syncop_fgetxattr (oldsubvol, oldfd, &lockinfo,
-				GF_XATTR_LOCKINFO_KEY);
+				GF_XATTR_LOCKINFO_KEY, NULL, NULL);
+        DECODE_SYNCOP_ERR (ret);
 	if (ret < 0) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                        API_MSG_FGETXATTR_FAILED,
 			"fgetxattr (%s) failed (%s) on graph %s (%d)",
 			uuid_utoa_r (oldfd->inode->gfid, uuid1),
 			strerror (errno),
@@ -525,16 +584,19 @@ glfs_migrate_fd_locks_safe (struct glfs *fs, xlator_t *oldsubvol, fd_t *oldfd,
 	}
 
 	if (!dict_get (lockinfo, GF_XATTR_LOCKINFO_KEY)) {
-		gf_log (fs->volname, GF_LOG_WARNING,
-			"missing lokinfo key (%s) on graph %s (%d)",
+		gf_msg (fs->volname, GF_LOG_WARNING, 0,
+                        API_MSG_LOCKINFO_KEY_MISSING,
+			"missing lockinfo key (%s) on graph %s (%d)",
 			uuid_utoa_r (oldfd->inode->gfid, uuid1),
 			graphid_str (oldsubvol), oldsubvol->graph->id);
 		goto out;
 	}
 
-	ret = syncop_fsetxattr (newsubvol, newfd, lockinfo, 0);
+	ret = syncop_fsetxattr (newsubvol, newfd, lockinfo, 0, NULL, NULL);
+        DECODE_SYNCOP_ERR (ret);
 	if (ret < 0) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, 0,
+                        API_MSG_FSETXATTR_FAILED,
 			"fsetxattr (%s) failed (%s) on graph %s (%d)",
 			uuid_utoa_r (newfd->inode->gfid, uuid1),
 			strerror (errno),
@@ -567,19 +629,21 @@ glfs_migrate_fd_safe (struct glfs *fs, xlator_t *newsubvol, fd_t *oldfd)
 		return fd_ref (oldfd);
 
 	if (!oldsubvol->switched) {
-		ret = syncop_fsync (oldsubvol, oldfd, 0);
+		ret = syncop_fsync (oldsubvol, oldfd, 0, NULL, NULL);
+                DECODE_SYNCOP_ERR (ret);
 		if (ret) {
-			gf_log (fs->volname, GF_LOG_WARNING,
-				"fsync() failed (%s) on %s graph %s (%d)",
-				strerror (errno),
+			gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                                API_MSG_FSYNC_FAILED, "fsync() failed "
+                                "(%s) on %s graph %s (%d)", strerror (errno),
 				uuid_utoa_r (oldfd->inode->gfid, uuid1),
 				graphid_str (oldsubvol), oldsubvol->graph->id);
 		}
 	}
 
-	newinode = glfs_refresh_inode_safe (newsubvol, oldinode);
+	newinode = glfs_refresh_inode_safe (newsubvol, oldinode, _gf_false);
 	if (!newinode) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                        API_MSG_INODE_REFRESH_FAILED,
 			"inode (%s) refresh failed (%s) on graph %s (%d)",
 			uuid_utoa_r (oldinode->gfid, uuid1),
 			strerror (errno),
@@ -589,7 +653,8 @@ glfs_migrate_fd_safe (struct glfs *fs, xlator_t *newsubvol, fd_t *oldfd)
 
 	newfd = fd_create (newinode, getpid());
 	if (!newfd) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                        API_MSG_FDCREATE_FAILED,
 			"fd_create (%s) failed (%s) on graph %s (%d)",
 			uuid_utoa_r (newinode->gfid, uuid1),
 			strerror (errno),
@@ -601,23 +666,26 @@ glfs_migrate_fd_safe (struct glfs *fs, xlator_t *newsubvol, fd_t *oldfd)
 
         ret = inode_path (oldfd->inode, NULL, (char **)&loc.path);
         if (ret < 0) {
-                gf_log (fs->volname, GF_LOG_INFO, "inode_path failed");
+                gf_msg (fs->volname, GF_LOG_INFO, 0, API_MSG_INODE_PATH_FAILED,
+                        "inode_path failed");
                 goto out;
         }
 
-        uuid_copy (loc.gfid, oldinode->gfid);
+        gf_uuid_copy (loc.gfid, oldinode->gfid);
 
 
 	if (IA_ISDIR (oldinode->ia_type))
-		ret = syncop_opendir (newsubvol, &loc, newfd);
+		ret = syncop_opendir (newsubvol, &loc, newfd, NULL, NULL);
 	else
 		ret = syncop_open (newsubvol, &loc,
 				   oldfd->flags & ~(O_TRUNC|O_EXCL|O_CREAT),
-				   newfd);
+				   newfd, NULL, NULL);
+        DECODE_SYNCOP_ERR (ret);
 	loc_wipe (&loc);
 
 	if (ret) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                        API_MSG_SYNCOP_OPEN_FAILED,
 			"syncop_open%s (%s) failed (%s) on graph %s (%d)",
 			IA_ISDIR (oldinode->ia_type) ? "dir" : "",
 			uuid_utoa_r (newinode->gfid, uuid1),
@@ -630,7 +698,8 @@ glfs_migrate_fd_safe (struct glfs *fs, xlator_t *newsubvol, fd_t *oldfd)
 					  newfd);
 
 	if (ret) {
-		gf_log (fs->volname, GF_LOG_WARNING,
+		gf_msg (fs->volname, GF_LOG_WARNING, errno,
+                        API_MSG_LOCK_MIGRATE_FAILED,
 			"lock migration (%s) failed (%s) on graph %s (%d)",
 			uuid_utoa_r (newinode->gfid, uuid1),
 			strerror (errno),
@@ -717,8 +786,9 @@ __glfs_migrate_openfds (struct glfs *fs, xlator_t *subvol)
 	fd_t *fd = NULL;
 
 	list_for_each_entry (glfd, &fs->openfds, openfds) {
-		if (uuid_is_null (glfd->fd->inode->gfid)) {
-			gf_log (fs->volname, GF_LOG_INFO,
+		if (gf_uuid_is_null (glfd->fd->inode->gfid)) {
+			gf_msg (fs->volname, GF_LOG_INFO, 0,
+                                API_MSG_OPENFD_SKIPPED,
 				"skipping openfd %p/%p in graph %s (%d)",
 				glfd, glfd->fd,	graphid_str(subvol),
 				subvol->graph->id);
@@ -749,7 +819,8 @@ __glfs_active_subvol (struct glfs *fs)
 
 	ret = __glfs_first_lookup (fs, new_subvol);
 	if (ret) {
-		gf_log (fs->volname, GF_LOG_INFO,
+		gf_msg (fs->volname, GF_LOG_INFO, errno,
+                        API_MSG_FIRST_LOOKUP_GRAPH_FAILED,
 			"first lookup on graph %s (%d) failed (%s)",
 			graphid_str (new_subvol), new_subvol->graph->id,
 			strerror (errno));
@@ -757,11 +828,13 @@ __glfs_active_subvol (struct glfs *fs)
 	}
 
 	if (fs->cwd) {
-		new_cwd = __glfs_refresh_inode (fs, new_subvol, fs->cwd);
+		new_cwd = __glfs_refresh_inode (fs, new_subvol, fs->cwd,
+                                                _gf_false);
 
 		if (!new_cwd) {
 			char buf1[64];
-			gf_log (fs->volname, GF_LOG_INFO,
+			gf_msg (fs->volname, GF_LOG_INFO, errno,
+                                API_MSG_CWD_GRAPH_REF_FAILED,
 				"cwd refresh of %s graph %s (%d) failed (%s)",
 				uuid_utoa_r (fs->cwd->gfid, buf1),
 				graphid_str (new_subvol),
@@ -784,42 +857,16 @@ __glfs_active_subvol (struct glfs *fs)
 		inode_unref (new_cwd);
 	}
 
-	gf_log (fs->volname, GF_LOG_INFO, "switched to graph %s (%d)",
+	gf_msg (fs->volname, GF_LOG_INFO, 0, API_MSG_SWITCHED_GRAPH,
+                "switched to graph %s (%d)",
 		graphid_str (new_subvol), new_subvol->graph->id);
 
 	return new_subvol;
 }
 
-xlator_t *
-glfs_active_subvol (struct glfs *fs)
-{
-	xlator_t      *subvol = NULL;
-	xlator_t      *old_subvol = NULL;
-
-	glfs_lock (fs);
-	{
-		subvol = __glfs_active_subvol (fs);
-
-		if (subvol)
-			subvol->winds++;
-
-		if (fs->old_subvol) {
-			old_subvol = fs->old_subvol;
-			fs->old_subvol = NULL;
-			old_subvol->switched = 1;
-		}
-	}
-	glfs_unlock (fs);
-
-	if (old_subvol)
-		glfs_subvol_done (fs, old_subvol);
-
-	return subvol;
-}
-
 
 void
-glfs_subvol_done (struct glfs *fs, xlator_t *subvol)
+priv_glfs_subvol_done (struct glfs *fs, xlator_t *subvol)
 {
 	int ref = 0;
 	xlator_t *active_subvol = NULL;
@@ -840,12 +887,44 @@ glfs_subvol_done (struct glfs *fs, xlator_t *subvol)
 	}
 }
 
+GFAPI_SYMVER_PRIVATE_DEFAULT(glfs_subvol_done, 3.4.0);
+
+
+xlator_t *
+priv_glfs_active_subvol (struct glfs *fs)
+{
+	xlator_t      *subvol = NULL;
+	xlator_t      *old_subvol = NULL;
+
+	glfs_lock (fs);
+	{
+		subvol = __glfs_active_subvol (fs);
+
+		if (subvol)
+			subvol->winds++;
+
+		if (fs->old_subvol) {
+			old_subvol = fs->old_subvol;
+			fs->old_subvol = NULL;
+			old_subvol->switched = 1;
+		}
+	}
+	glfs_unlock (fs);
+
+	if (old_subvol)
+		priv_glfs_subvol_done (fs, old_subvol);
+
+	return subvol;
+}
+
+GFAPI_SYMVER_PRIVATE_DEFAULT(glfs_active_subvol, 3.4.0);
 
 int
 __glfs_cwd_set (struct glfs *fs, inode_t *inode)
 {
 	if (inode->table->xl != fs->active_subvol) {
-		inode = __glfs_refresh_inode (fs, fs->active_subvol, inode);
+		inode = __glfs_refresh_inode (fs, fs->active_subvol, inode,
+                                              _gf_false);
 		if (!inode)
 			return -1;
 	} else {
@@ -889,7 +968,7 @@ __glfs_cwd_get (struct glfs *fs)
 		return cwd;
 	}
 
-	cwd = __glfs_refresh_inode (fs, fs->active_subvol, fs->cwd);
+	cwd = __glfs_refresh_inode (fs, fs->active_subvol, fs->cwd, _gf_false);
 
 	return cwd;
 }
@@ -913,12 +992,15 @@ __glfs_resolve_inode (struct glfs *fs, xlator_t *subvol,
 		    struct glfs_object *object)
 {
 	inode_t *inode = NULL;
+        gf_boolean_t lookup_needed = _gf_false;
 
-	if (object->inode->table->xl == subvol)
+        lookup_needed = inode_needs_lookup (object->inode, THIS);
+
+	if (!lookup_needed && object->inode->table->xl == subvol)
 		return inode_ref (object->inode);
 
 	inode = __glfs_refresh_inode (fs, fs->active_subvol,
-					object->inode);
+                                      object->inode, lookup_needed);
 	if (!inode)
 		return NULL;
 
@@ -958,7 +1040,7 @@ glfs_create_object (loc_t *loc, struct glfs_object **retobject)
 	}
 
 	object->inode = loc->inode;
-	uuid_copy (object->gfid, object->inode->gfid);
+	gf_uuid_copy (object->gfid, object->inode->gfid);
 
 	/* we hold the reference */
 	loc->inode = NULL;
@@ -967,3 +1049,40 @@ glfs_create_object (loc_t *loc, struct glfs_object **retobject)
 
 	return 0;
 }
+
+struct glfs_object *
+glfs_h_resolve_symlink (struct glfs *fs, struct glfs_object *object)
+{
+
+        xlator_t                *subvol         = NULL;
+        loc_t                   sym_loc         = {0,};
+        struct iatt             iatt            = {0,};
+        char                    *lpath          = NULL;
+        int                     ret             = 0;
+        struct glfs_object      *target_object  = NULL;
+
+        subvol = glfs_active_subvol (fs);
+        if (!subvol) {
+                ret = -1;
+                errno = EIO;
+                goto out;
+        }
+
+        ret = glfs_resolve_symlink (fs, subvol, object->inode, &lpath);
+        if (ret < 0)
+                goto out;
+
+        ret = glfs_resolve_at (fs, subvol, NULL, lpath,
+                               &sym_loc, &iatt,
+                              /* always recurisvely follow while
+                                following symlink
+                              */
+                               1, 0);
+        if (ret == 0)
+                ret = glfs_create_object (&sym_loc, &target_object);
+
+out:
+        loc_wipe (&sym_loc);
+        GF_FREE (lpath);
+        return target_object;
+}
