@@ -231,6 +231,12 @@ cli_cmds_register (struct cli_state *state)
         if (ret)
                 goto out;
 
+        ret = cli_cmd_snapshot_register (state);
+        if (ret)
+                goto out;
+        ret = cli_cmd_global_register (state);
+        if (ret)
+                goto out;
 out:
         return ret;
 }
@@ -287,14 +293,21 @@ cli_cmd_await_response (unsigned time)
                                         &ts);
         }
 
+        if (!cmd_done) {
+                if (ret == ETIMEDOUT)
+                        cli_out ("Error : Request timed out");
+                else
+                        cli_out ("Error : Command returned with error code:%d",
+                                 ret);
+        }
         cmd_done = 0;
 
-        if (ret)
-                return ret;
-
         return cli_op_ret;
 }
 
+/* This function must be called _only_ after all actions associated with
+ * command processing is complete. Otherwise, gluster process may exit before
+ * reporting results to stdout/stderr. */
 int
 cli_cmd_broadcast_response (int32_t status)
 {
@@ -361,7 +374,8 @@ cli_cmd_submit (struct rpc_clnt* rpc, void *req, call_frame_t *frame,
         unsigned        timeout = 0;
 
         if ((GLUSTER_CLI_PROFILE_VOLUME == procnum) ||
-            (GLUSTER_CLI_HEAL_VOLUME == procnum))
+            (GLUSTER_CLI_HEAL_VOLUME == procnum) ||
+            (GLUSTER_CLI_GANESHA == procnum))
                 timeout = CLI_TEN_MINUTES_TIMEOUT;
         else
                 timeout = CLI_DEFAULT_CMD_TIMEOUT;
@@ -381,3 +395,28 @@ cli_cmd_submit (struct rpc_clnt* rpc, void *req, call_frame_t *frame,
         gf_log ("cli", GF_LOG_DEBUG, "Returning %d", ret);
         return ret;
 }
+
+int
+cli_cmd_pattern_cmp (void *a, void *b)
+{
+        struct cli_cmd *ia = NULL;
+        struct cli_cmd *ib = NULL;
+        int            ret = 0;
+
+        ia = a;
+        ib = b;
+        if (strcmp (ia->pattern, ib->pattern) > 0)
+                ret = 1;
+        else if (strcmp (ia->pattern, ib->pattern) < 0)
+                ret = -1;
+        else
+                ret = 0;
+        return ret;
+}
+
+void
+cli_cmd_sort (struct cli_cmd *cmd, int count)
+{
+        gf_array_insertionsort (cmd, 1, count - 2, sizeof(struct cli_cmd),
+                                cli_cmd_pattern_cmp);
+}
