@@ -631,7 +631,7 @@ cli_rpc_init (struct cli_state *state)
                         goto out;
         }
 
-        rpc = rpc_clnt_new (options, this->ctx, this->name, 16);
+        rpc = rpc_clnt_new (options, this, this->name, 16);
         if (!rpc)
                 goto out;
 
@@ -657,6 +657,7 @@ cli_local_get ()
         cli_local_t     *local = NULL;
 
         local = GF_CALLOC (1, sizeof (*local), cli_mt_cli_local_t);
+        LOCK_INIT (&local->lock);
 
         return local;
 }
@@ -763,18 +764,18 @@ print_quota_list_header (int type)
 {
         if (type == GF_QUOTA_OPTION_TYPE_LIST) {
                 cli_out ("                  Path                   Hard-limit "
-                         "Soft-limit   Used  Available  Soft-limit exceeded?"
-                         " Hard-limit exceeded?");
+                         " Soft-limit      Used  Available  Soft-limit "
+                         "exceeded? Hard-limit exceeded?");
                 cli_out ("-----------------------------------------------------"
                          "-----------------------------------------------------"
-                         "-----------------");
+                         "---------------------");
         } else {
                 cli_out ("                  Path                   Hard-limit  "
-                         "Soft-limit     Files       Dirs     Available  "
+                         " Soft-limit      Files       Dirs     Available  "
                          "Soft-limit exceeded? Hard-limit exceeded?");
                 cli_out ("-----------------------------------------------------"
                          "-----------------------------------------------------"
-                         "-----------------------------------");
+                         "-------------------------------------");
         }
 }
 
