@@ -30,8 +30,22 @@ SERVICE_MAN="DISTRO_NOT_FOUND"
 RHEL6_PCS_CNAME_OPTION="--name"
 SECRET_PEM="/var/lib/glusterd/nfs/secret.pem"
 
+# Try loading the config from any of the distro
+# specific configuration locations
+if [ -f /etc/sysconfig/ganesha ]
+        then
+        . /etc/sysconfig/ganesha
+fi
+if [ -f /etc/conf.d/ganesha ]
+        then
+        . /etc/conf.d/ganesha
+fi
+if [ -f /etc/default/ganesha ]
+        then
+        . /etc/default/ganesha
+fi
+
 GANESHA_CONF=
-CONFFILE=
 
 function find_rhel7_conf
 {
@@ -50,14 +64,9 @@ function find_rhel7_conf
          done
 }
 
-cfgline=$(grep ^CONFFILE= /etc/sysconfig/ganesha)
-eval $(echo ${cfgline} | grep -F ^CONFFILE=)
-
 if [ -z $CONFFILE ]
         then
-        cfgline=$(grep ^OPTIONS= /etc/sysconfig/ganesha)
-        eval $(echo ${cfgline} | grep -F ^OPTIONS=)
-        find_rhel7_conf $cfgline
+        find_rhel7_conf $OPTIONS
 
 fi
 
@@ -97,7 +106,7 @@ manage_service ()
 {
         local action=${1}
         local new_node=${2}
-        if [ "$SERVICE_MAN" == "/usr/sbin/systemctl" ]
+        if [ "$SERVICE_MAN" == "/usr/bin/systemctl" ]
         then
                 ssh -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} root@${new_node} "$SERVICE_MAN  ${action} nfs-ganesha"
@@ -107,6 +116,7 @@ ${SECRET_PEM} root@${new_node} "$SERVICE_MAN nfs-ganesha ${action}"
         fi
 }
 
+
 check_cluster_exists()
 {
     local name=${1}
@@ -245,38 +255,68 @@ grep Export_Id | cut -d " " -f8`
         while [[ ${3} ]]; do
             current_host=`echo ${3} | cut -d "." -f 1`
             if [ ${short_host} != ${current_host} ]; then
-                scp -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
+                scp -q -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} ${HA_CONFDIR}/exports/export.$VOL.conf \
 ${current_host}:${HA_CONFDIR}/exports/
-                 ssh -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
-${SECRET_PEM} root@${current_host} "dbus-send --print-reply --system \
---dest=org.ganesha.nfsd /org/ganesha/nfsd/ExportMgr \
-org.ganesha.nfsd.exportmgr.RemoveExport uint16:$removed_id"
+                output=$(ssh -oPasswordAuthentication=no \
+-oStrictHostKeyChecking=no -i ${SECRET_PEM} root@${current_host} \
+"dbus-send --print-reply --system --dest=org.ganesha.nfsd \
+/org/ganesha/nfsd/ExportMgr org.ganesha.nfsd.exportmgr.RemoveExport \
+uint16:$removed_id 2>&1")
+                 ret=$?
+                 logger <<< "${output}"
+                 if [ ${ret} -ne 0 ]; then
+                        echo "Error: refresh-config failed on ${current_host}."
+                        exit 1
+                 fi
                  sleep 1
-                 ssh -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
-${SECRET_PEM} root@${current_host} "dbus-send  --system \
---dest=org.ganesha.nfsd  /org/ganesha/nfsd/ExportMgr \
-org.ganesha.nfsd.exportmgr.AddExport  string:$HA_CONFDIR/exports/export.$VOL.conf \
-string:\"EXPORT(Path=/$VOL)\""
-               if [ $? -ne 0 ]; then
-                    echo "warning: refresh-config failed on ${current_host}"
+                 output=$(ssh -oPasswordAuthentication=no \
+-oStrictHostKeyChecking=no -i ${SECRET_PEM} root@${current_host} \
+"dbus-send --print-reply --system --dest=org.ganesha.nfsd \
+/org/ganesha/nfsd/ExportMgr org.ganesha.nfsd.exportmgr.AddExport \
+string:$HA_CONFDIR/exports/export.$VOL.conf \
+string:\"EXPORT(Path=/$VOL)\" 2>&1")
+                 ret=$?
+                 logger <<< "${output}"
+               if [ ${ret} -ne 0 ]; then
+                    echo "Error: refresh-config failed on ${current_host}."
+                    exit 1
+               else
+                    echo "Refresh-config completed on ${current_host}."
                fi
+
             fi
             shift
         done
     else
-        echo "warning: refresh-config failed on ${1}"
+        echo "Error: refresh-config failed. Passwordless ssh is not enabled."
+        exit 1
     fi
 
 #Run the same command on the localhost,
-        dbus-send --print-reply --system \
---dest=org.ganesha.nfsd /org/ganesha/nfsd/ExportMgr \
-org.ganesha.nfsd.exportmgr.RemoveExport uint16:$removed_id
+        output=$(dbus-send --print-reply --system --dest=org.ganesha.nfsd \
+/org/ganesha/nfsd/ExportMgr org.ganesha.nfsd.exportmgr.RemoveExport \
+uint16:$removed_id 2>&1)
+        ret=$?
+        logger <<< "${output}"
+        if [ ${ret} -ne 0 ]; then
+                echo "Error: refresh-config failed on localhost."
+                exit 1
+        fi
         sleep 1
-        dbus-send  --system \
---dest=org.ganesha.nfsd  /org/ganesha/nfsd/ExportMgr \
-org.ganesha.nfsd.exportmgr.AddExport  string:$HA_CONFDIR/exports/export.$VOL.conf \
-string:"EXPORT(Path=/$VOL)"
+        output=$(dbus-send --print-reply --system --dest=org.ganesha.nfsd \
+/org/ganesha/nfsd/ExportMgr org.ganesha.nfsd.exportmgr.AddExport \
+string:$HA_CONFDIR/exports/export.$VOL.conf \
+string:"EXPORT(Path=/$VOL)" 2>&1)
+        ret=$?
+        logger <<< "${output}"
+        if [ ${ret} -ne 0 ] ; then
+                echo "Error: refresh-config failed on localhost."
+                exit 1
+        else
+                echo "Success: refresh-config completed."
+        fi
+
 }
 
 copy_export_config ()
@@ -287,14 +327,27 @@ copy_export_config ()
     local short_host=$(hostname -s)
     # avoid prompting for password, even with password-less scp
     # scp $host1:$file $host2:$file prompts for the password
-    scp -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
+    # Ideally all the existing nodes in the cluster should have same
+    # copy of the configuration files. Maybe for sanity check, copy
+    # the state from HA_VOL_SERVER?
+    if [ "${HA_VOL_SERVER}" == $(hostname) ]
+    then
+        cp ${GANESHA_CONF} ${tganesha_conf}
+    else
+        scp -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} ${HA_VOL_SERVER}:${GANESHA_CONF} $short_host:${tganesha_conf}
+    fi
     scp -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} ${tganesha_conf} ${new_node}:${GANESHA_CONF}
     rm -f ${tganesha_conf}
 
-    scp -r -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
+    if [ "${HA_VOL_SERVER}" == $(hostname) ]
+    then
+        cp -r ${HA_CONFDIR}/exports ${tganesha_exports}
+    else
+        scp -r -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} ${HA_VOL_SERVER}:${HA_CONFDIR}/exports/ $short_host:${tganesha_exports}
+    fi
     scp -r -oPasswordAuthentication=no -oStrictHostKeyChecking=no -i \
 ${SECRET_PEM} ${tganesha_exports}/exports ${new_node}:${HA_CONFDIR}/
     rm -rf ${tganesha_exports}
@@ -781,6 +834,16 @@ deletenode_delete_resources()
 }
 
 
+deletenode_update_haconfig()
+{
+    local name="VIP_${1}"
+    local clean_name=${name//[-.]/_}
+
+    ha_servers=$(echo ${HA_SERVERS} | sed -e "s/ /,/")
+    sed -i -e "s/^HA_CLUSTER_NODES=.*$/HA_CLUSTER_NODES=\"${ha_servers// /,}\"/" -e "s/^${clean_name}=.*$//" -e "/^$/d" ${HA_CONFDIR}/ganesha-ha.conf
+}
+
+
 setup_state_volume()
 {
     local mnt=${HA_VOL_MNT}
@@ -843,6 +906,29 @@ setup_state_volume()
 }
 
 
+status()
+{
+    local regex_str="^ ${1}"; shift
+    local status_file=$(mktemp)
+
+    while [[ ${1} ]]; do
+
+        regex_str="${regex_str}|^ ${1}"
+
+        shift
+    done
+
+    pcs status | egrep "^Online:" > ${status_file}
+
+    echo >> ${status_file}
+
+    pcs status | egrep "${regex_str}" | sed -e "s/\t/ /" | cut -d ' ' -f 2,4 >> ${status_file}
+
+    cat ${status_file}
+
+    rm -f ${status_file}
+}
+
 
 main()
 {
@@ -938,7 +1024,8 @@ main()
         #from the entries in the ganesha-ha.conf file. Adding the
         #newly added node to the file so that the resources specfic
         #to this node is correctly recreated in the future.
-        echo "VIP_$node=\"$vip\"" >> ${HA_CONFDIR}/ganesha-ha.conf
+        clean_node=${node//[-.]/_}
+        echo "VIP_$clean_node=\"${vip}\"" >> ${HA_CONFDIR}/ganesha-ha.conf
 
         NEW_NODES="$HA_CLUSTER_NODES,$node"
 
@@ -963,8 +1050,7 @@ $HA_CONFDIR/ganesha-ha.conf
             logger "warning: pcs cluster node remove ${node} failed"
         fi
 
-        ha_servers=$(echo ${HA_SERVERS} | sed -e "s/ /,/")
-        sed -i "s/^HA_CLUSTER_NODES=.*$/HA_CLUSTER_NODES=\"${ha_servers// /,}\"/" ${HA_CONFDIR}/ganesha-ha.conf
+        deletenode_update_haconfig ${node}
 
         setup_copy_config ${HA_SERVERS}
 
@@ -976,7 +1062,9 @@ $HA_CONFDIR/ganesha-ha.conf
         ;;
 
     status | --status)
-        exec pcs status
+        determine_servers "status"
+
+        status ${HA_SERVERS}
         ;;
 
     refresh-config | --refresh-config)
