@@ -38,6 +38,7 @@
 #include "xlator.h"
 
 static gf_boolean_t is_mgmt_rpc_reconnect = _gf_false;
+int need_emancipate = 0;
 
 int glusterfs_mgmt_pmap_signin (glusterfs_ctx_t *ctx);
 int glusterfs_volfile_fetch (glusterfs_ctx_t *ctx);
@@ -365,7 +366,7 @@ glusterfs_volume_top_write_perf (uint32_t blk_size, uint32_t blk_count,
 {
         int32_t                 fd = -1;
         int32_t                 input_fd = -1;
-        char                    export_path[PATH_MAX];
+        char                    export_path[PATH_MAX] = {0,};
         char                    *buf = NULL;
         int32_t                 iter = 0;
         int32_t                 ret = -1;
@@ -448,7 +449,7 @@ glusterfs_volume_top_read_perf (uint32_t blk_size, uint32_t blk_count,
         int32_t                 fd = -1;
         int32_t                 input_fd = -1;
         int32_t                 output_fd = -1;
-        char                    export_path[PATH_MAX];
+        char                    export_path[PATH_MAX] = {0,};
         char                    *buf = NULL;
         int32_t                 iter = 0;
         int32_t                 ret = -1;
@@ -914,6 +915,10 @@ glusterfs_handle_node_status (rpcsvc_request_t *req)
                 ret = gf_asprintf (&node_name, "%s", "glustershd");
         else if ((cmd & GF_CLI_STATUS_QUOTAD) != 0)
                 ret = gf_asprintf (&node_name, "%s", "quotad");
+        else if ((cmd & GF_CLI_STATUS_BITD) != 0)
+                ret = gf_asprintf (&node_name, "%s", "bitd");
+        else if ((cmd & GF_CLI_STATUS_SCRUB) != 0)
+                ret = gf_asprintf (&node_name, "%s", "scrubber");
 
         else {
                 ret = -1;
@@ -939,6 +944,10 @@ glusterfs_handle_node_status (rpcsvc_request_t *req)
                 ret = gf_asprintf (&subvol_name, "%s-replicate-0", volname);
         else if ((cmd & GF_CLI_STATUS_QUOTAD) != 0)
                 ret = gf_asprintf (&subvol_name, "%s", volname);
+        else if ((cmd & GF_CLI_STATUS_BITD) != 0)
+                ret = gf_asprintf (&subvol_name, "%s", volname);
+        else if ((cmd & GF_CLI_STATUS_SCRUB) != 0)
+                ret = gf_asprintf (&subvol_name, "%s", volname);
         else {
                 ret = -1;
                 goto out;
@@ -1128,6 +1137,201 @@ out:
         return ret;
 }
 
+int
+glusterfs_handle_volume_barrier_op (rpcsvc_request_t *req)
+{
+        int32_t                  ret     = -1;
+        gd1_mgmt_brick_op_req    xlator_req = {0,};
+        dict_t                   *dict    = NULL;
+        xlator_t                 *xlator = NULL;
+        xlator_t                 *any = NULL;
+        dict_t                   *output = NULL;
+        char                     msg[2048] = {0};
+        glusterfs_ctx_t          *ctx = NULL;
+        glusterfs_graph_t        *active = NULL;
+        xlator_t                 *this = NULL;
+
+        GF_ASSERT (req);
+        this = THIS;
+        GF_ASSERT (this);
+
+        ctx = glusterfsd_ctx;
+        GF_ASSERT (ctx);
+
+        active = ctx->active;
+        if (!active) {
+                req->rpc_err = GARBAGE_ARGS;
+                goto out;
+        }
+
+        any = active->first;
+        ret = xdr_to_generic (req->msg[0], &xlator_req,
+                              (xdrproc_t)xdr_gd1_mgmt_brick_op_req);
+        if (ret < 0) {
+                //failed to decode msg;
+                req->rpc_err = GARBAGE_ARGS;
+                goto out;
+        }
+        dict = dict_new ();
+        if (!dict)
+                goto out;
+
+        ret = dict_unserialize (xlator_req.input.input_val,
+                                xlator_req.input.input_len,
+                                &dict);
+        if (ret < 0) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "failed to "
+                        "unserialize req-buffer to dictionary");
+                goto out;
+        }
+        xlator = xlator_search_by_name (any, xlator_req.name);
+        if (!xlator) {
+                snprintf (msg, sizeof (msg), "xlator %s is not loaded",
+                          xlator_req.name);
+                goto out;
+        }
+
+        output = dict_new ();
+        if (!output) {
+                ret = -1;
+                goto out;
+        }
+
+        ret = xlator->notify (xlator, GF_EVENT_VOLUME_BARRIER_OP,
+                              dict, output);
+
+        ret = glusterfs_translator_info_response_send (req, ret,
+                                                       msg, output);
+out:
+        if (dict)
+                dict_unref (dict);
+        free (xlator_req.input.input_val); // malloced by xdr
+        if (output)
+                dict_unref (output);
+        free (xlator_req.name); //malloced by xdr
+
+        return ret;
+
+}
+
+int
+glusterfs_handle_barrier (rpcsvc_request_t *req)
+{
+        int                     ret         = -1;
+        gd1_mgmt_brick_op_req   brick_req   = {0,};
+        gd1_mgmt_brick_op_rsp   brick_rsp   = {0,};
+        glusterfs_ctx_t         *ctx        = NULL;
+        glusterfs_graph_t       *active     = NULL;
+        xlator_t                *any        = NULL;
+        xlator_t                *xlator     = NULL;
+        xlator_t                *old_THIS   = NULL;
+        dict_t                  *dict       = NULL;
+        char                    name[1024]  = {0,};
+        gf_boolean_t            barrier     = _gf_true;
+        gf_boolean_t            barrier_err = _gf_false;
+
+        GF_ASSERT (req);
+
+        ret = xdr_to_generic(req->msg[0], &brick_req,
+                             (xdrproc_t)xdr_gd1_mgmt_brick_op_req);
+        if (ret < 0) {
+                req->rpc_err = GARBAGE_ARGS;
+                goto out;
+        }
+
+        ctx = glusterfsd_ctx;
+        GF_ASSERT (ctx);
+        active = ctx->active;
+        any = active->first;
+
+        dict = dict_new();
+        if (!dict) {
+                ret = -1;
+                goto out;
+        }
+
+        ret = dict_unserialize(brick_req.input.input_val,
+                               brick_req.input.input_len, &dict);
+        if (ret < 0) {
+                gf_log (THIS->name, GF_LOG_ERROR, "Failed to unserialize "
+                        "request dictionary");
+                goto out;
+        }
+
+        brick_rsp.op_ret = 0;
+        brick_rsp.op_errstr = ""; // initing to prevent serilaztion failures
+        old_THIS = THIS;
+
+        /* Send barrier request to the barrier xlator */
+        snprintf (name, sizeof (name), "%s-barrier", brick_req.name);
+        xlator = xlator_search_by_name(any, name);
+        if (!xlator) {
+                ret = -1;
+                gf_log (THIS->name, GF_LOG_ERROR, "%s xlator is not loaded",
+                        name);
+                goto out;
+        }
+
+        THIS = xlator;
+        // TODO: Extend this to accept return of errnos
+        ret = xlator->notify (xlator, GF_EVENT_TRANSLATOR_OP, dict);
+        if (ret) {
+                brick_rsp.op_ret = ret;
+                brick_rsp.op_errstr = gf_strdup ("Failed to reconfigure "
+                                                 "barrier.");
+                /* This is to invoke changelog-barrier disable if barrier
+                 * disable fails and don't invoke if barrier enable fails.
+                 */
+                barrier = dict_get_str_boolean (dict, "barrier", _gf_true);
+                if (barrier)
+                        goto submit_reply;
+                else
+                        barrier_err = _gf_true;
+        }
+
+        /* Reset THIS so that we have it correct in case of an error below
+         */
+        THIS = old_THIS;
+
+        /* Send barrier request to changelog as well */
+
+        memset (name, 0, sizeof (name));
+        snprintf (name, sizeof (name), "%s-changelog", brick_req.name);
+        xlator = xlator_search_by_name(any, name);
+        if (!xlator) {
+                ret = -1;
+                gf_log (THIS->name, GF_LOG_ERROR, "%s xlator is not loaded",
+                        name);
+                goto out;
+        }
+
+        THIS = xlator;
+        ret = xlator->notify (xlator, GF_EVENT_TRANSLATOR_OP, dict);
+        if (ret) {
+                brick_rsp.op_ret = ret;
+                brick_rsp.op_errstr = gf_strdup ("changelog notify failed");
+                goto submit_reply;
+        }
+
+        if (barrier_err)
+                ret = -1;
+
+submit_reply:
+        THIS = old_THIS;
+
+        ret = glusterfs_submit_reply (req, &brick_rsp, NULL, 0, NULL,
+                                      (xdrproc_t)xdr_gd1_mgmt_brick_op_rsp);
+
+out:
+        if (dict)
+                dict_unref (dict);
+        free (brick_req.input.input_val);
+
+        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        return ret;
+}
+
 int
 glusterfs_handle_rpc_msg (rpcsvc_request_t *req)
 {
@@ -1192,6 +1396,8 @@ rpcsvc_actor_t glusterfs_actors[GLUSTERD_BRICK_MAXVALUE] = {
         [GLUSTERD_BRICK_XLATOR_DEFRAG] = {"TRANSLATOR DEFRAG", GLUSTERD_BRICK_XLATOR_DEFRAG, glusterfs_handle_defrag,              NULL, 0, DRC_NA},
         [GLUSTERD_NODE_PROFILE]        = {"NFS PROFILE",       GLUSTERD_NODE_PROFILE,        glusterfs_handle_nfs_profile,         NULL, 0, DRC_NA},
         [GLUSTERD_NODE_STATUS]         = {"NFS STATUS",        GLUSTERD_NODE_STATUS,         glusterfs_handle_node_status,         NULL, 0, DRC_NA},
+        [GLUSTERD_VOLUME_BARRIER_OP]   = {"VOLUME BARRIER OP", GLUSTERD_VOLUME_BARRIER_OP,   glusterfs_handle_volume_barrier_op,   NULL, 0, DRC_NA},
+        [GLUSTERD_BRICK_BARRIER]       = {"BARRIER",           GLUSTERD_BRICK_BARRIER,       glusterfs_handle_barrier,             NULL, 0, DRC_NA},
 };
 
 struct rpcsvc_program glusterfs_mop_prog = {
@@ -1371,6 +1577,7 @@ mgmt_getspec_cbk (struct rpc_req *req, struct iovec *iov, int count,
         oldvollen = size;
         memcpy (oldvolfile, rsp.spec, size);
         if (!is_mgmt_rpc_reconnect) {
+                need_emancipate = 1;
                 glusterfs_mgmt_pmap_signin (ctx);
                 is_mgmt_rpc_reconnect =  _gf_true;
         }
@@ -1380,8 +1587,6 @@ out:
 
         free (rsp.spec);
 
-        emancipate (ctx, ret);
-
         // Stop if server is running at an unsupported op-version
         if (ENOTSUP == ret) {
                 gf_log ("mgmt", GF_LOG_ERROR, "Server is operating at an "
@@ -1445,6 +1650,16 @@ glusterfs_volfile_fetch (glusterfs_ctx_t *ctx)
                 goto out;
         }
 
+        if (cmd_args->brick_name) {
+                ret = dict_set_dynstr_with_alloc (dict, "brick_name",
+                                                  cmd_args->brick_name);
+                if (ret) {
+                        gf_log (THIS->name, GF_LOG_ERROR,
+                                "Failed to set brick_name in request dict");
+                        goto out;
+                }
+        }
+
         ret = dict_allocate_and_serialize (dict, &req.xdata.xdata_val,
                                            &req.xdata.xdata_len);
         if (ret < 0) {
@@ -1572,7 +1787,9 @@ glusterfs_rebalance_event_notify (dict_t *dict)
 
         GF_FREE (req.dict.dict_val);
 
-        STACK_DESTROY (frame->root);
+        if (frame) {
+              STACK_DESTROY (frame->root);
+        }
         return ret;
 }
 
@@ -1820,7 +2037,7 @@ glusterfs_mgmt_init (glusterfs_ctx_t *ctx)
         if (ret)
                 goto out;
 
-        rpc = rpc_clnt_new (options, THIS->ctx, THIS->name, 8);
+        rpc = rpc_clnt_new (options, THIS, THIS->name, 8);
         if (!rpc) {
                 ret = -1;
                 gf_log (THIS->name, GF_LOG_WARNING, "failed to create rpc clnt");
@@ -1857,12 +2074,15 @@ mgmt_pmap_signin2_cbk (struct rpc_req *req, struct iovec *iov, int count,
                        void *myframe)
 {
         pmap_signin_rsp  rsp   = {0,};
+        glusterfs_ctx_t *ctx   = NULL;
         call_frame_t    *frame = NULL;
         int              ret   = 0;
 
+        ctx = glusterfsd_ctx;
         frame = myframe;
 
         if (-1 == req->rpc_status) {
+                ret = -1;
                 rsp.op_ret   = -1;
                 rsp.op_errno = EINVAL;
                 goto out;
@@ -1879,9 +2099,13 @@ mgmt_pmap_signin2_cbk (struct rpc_req *req, struct iovec *iov, int count,
         if (-1 == rsp.op_ret) {
                 gf_log (frame->this->name, GF_LOG_ERROR,
                         "failed to register the port with glusterd");
+                ret = -1;
                 goto out;
         }
 out:
+        if (need_emancipate)
+                emancipate (ctx, ret);
+
         STACK_DESTROY (frame->root);
         return 0;
 
@@ -1894,6 +2118,7 @@ mgmt_pmap_signin_cbk (struct rpc_req *req, struct iovec *iov, int count,
         pmap_signin_rsp  rsp   = {0,};
         call_frame_t    *frame = NULL;
         int              ret   = 0;
+        int              emancipate_ret  = -1;
         pmap_signin_req  pmap_req = {0, };
         cmd_args_t      *cmd_args = NULL;
         glusterfs_ctx_t *ctx      = NULL;
@@ -1902,6 +2127,7 @@ mgmt_pmap_signin_cbk (struct rpc_req *req, struct iovec *iov, int count,
         frame = myframe;
 
         if (-1 == req->rpc_status) {
+                ret = -1;
                 rsp.op_ret   = -1;
                 rsp.op_errno = EINVAL;
                 goto out;
@@ -1918,6 +2144,7 @@ mgmt_pmap_signin_cbk (struct rpc_req *req, struct iovec *iov, int count,
         if (-1 == rsp.op_ret) {
                 gf_log (frame->this->name, GF_LOG_ERROR,
                         "failed to register the port with glusterd");
+                ret = -1;
                 goto out;
         }
 
@@ -1926,6 +2153,7 @@ mgmt_pmap_signin_cbk (struct rpc_req *req, struct iovec *iov, int count,
 
         if (!cmd_args->brick_port2) {
                 /* We are done with signin process */
+                emancipate_ret = 0;
                 goto out;
         }
 
@@ -1942,6 +2170,8 @@ mgmt_pmap_signin_cbk (struct rpc_req *req, struct iovec *iov, int count,
         return 0;
 
 out:
+        if (need_emancipate && (ret < 0 || !cmd_args->brick_port2))
+                emancipate (ctx, emancipate_ret);
 
         STACK_DESTROY (frame->root);
         return 0;
@@ -1950,10 +2180,12 @@ out:
 int
 glusterfs_mgmt_pmap_signin (glusterfs_ctx_t *ctx)
 {
-        call_frame_t     *frame = NULL;
-        pmap_signin_req   req = {0, };
-        int               ret = -1;
-        cmd_args_t       *cmd_args = NULL;
+        call_frame_t     *frame                 = NULL;
+        pmap_signin_req   req                   = {0, };
+        int               ret                   = -1;
+        int               emancipate_ret        = -1;
+        cmd_args_t       *cmd_args              = NULL;
+        char              brick_name[PATH_MAX]  = {0,};
 
         frame = create_frame (THIS, ctx->pool);
         cmd_args = &ctx->cmd_args;
@@ -1961,17 +2193,27 @@ glusterfs_mgmt_pmap_signin (glusterfs_ctx_t *ctx)
         if (!cmd_args->brick_port || !cmd_args->brick_name) {
                 gf_log ("fsd-mgmt", GF_LOG_DEBUG,
                         "portmapper signin arguments not given");
+                emancipate_ret = 0;
                 goto out;
         }
 
+        if (cmd_args->volfile_server_transport &&
+                      !strcmp(cmd_args->volfile_server_transport, "rdma")) {
+                snprintf (brick_name, sizeof(brick_name), "%s.rdma",
+                          cmd_args->brick_name);
+                req.brick = brick_name;
+        } else
+                req.brick = cmd_args->brick_name;
+
         req.port  = cmd_args->brick_port;
-        req.brick = cmd_args->brick_name;
 
         ret = mgmt_submit_request (&req, frame, ctx, &clnt_pmap_prog,
                                    GF_PMAP_SIGNIN, mgmt_pmap_signin_cbk,
                                    (xdrproc_t)xdr_pmap_signin_req);
 
 out:
+        if (need_emancipate && ret < 0)
+                emancipate (ctx, emancipate_ret);
         return ret;
 }
 
@@ -2016,6 +2258,7 @@ glusterfs_mgmt_pmap_signout (glusterfs_ctx_t *ctx)
         pmap_signout_req  req = {0, };
         call_frame_t     *frame = NULL;
         cmd_args_t       *cmd_args = NULL;
+        char              brick_name[PATH_MAX]  = {0,};
 
         frame = create_frame (THIS, ctx->pool);
         cmd_args = &ctx->cmd_args;
@@ -2026,9 +2269,16 @@ glusterfs_mgmt_pmap_signout (glusterfs_ctx_t *ctx)
                 goto out;
         }
 
-        req.port  = cmd_args->brick_port;
-        req.brick = cmd_args->brick_name;
+        if (cmd_args->volfile_server_transport &&
+                      !strcmp(cmd_args->volfile_server_transport, "rdma")) {
+                snprintf (brick_name, sizeof(brick_name), "%s.rdma",
+                          cmd_args->brick_name);
+                req.brick = brick_name;
+        } else
+                req.brick = cmd_args->brick_name;
 
+        req.port  = cmd_args->brick_port;
+        req.rdma_port = cmd_args->brick_port2;
         ret = mgmt_submit_request (&req, frame, ctx, &clnt_pmap_prog,
                                    GF_PMAP_SIGNOUT, mgmt_pmap_signout_cbk,
                                    (xdrproc_t)xdr_pmap_signout_req);
