@@ -39,6 +39,8 @@
 #define GF_MEMPOOL_COUNT_OF_DATA_T        (GF_MEMPOOL_COUNT_OF_DICT_T * 4)
 #define GF_MEMPOOL_COUNT_OF_DATA_PAIR_T   (GF_MEMPOOL_COUNT_OF_DICT_T * 4)
 
+#define GF_MEMPOOL_COUNT_OF_LRU_BUF_T     256
+
 enum argp_option_keys {
         ARGP_VOLFILE_SERVER_KEY           = 's',
         ARGP_VOLUME_FILE_KEY              = 'f',
@@ -50,6 +52,8 @@ enum argp_option_keys {
         ARGP_SOCK_FILE_KEY                = 'S',
         ARGP_NO_DAEMON_KEY                = 'N',
         ARGP_RUN_ID_KEY                   = 'r',
+        ARGP_PRINT_NETGROUPS              = 'n',
+        ARGP_PRINT_EXPORTS                = 'e',
         ARGP_DEBUG_KEY                    = 133,
         ARGP_NEGATIVE_TIMEOUT_KEY         = 134,
         ARGP_ENTRY_TIMEOUT_KEY            = 135,
@@ -84,6 +88,14 @@ enum argp_option_keys {
 	ARGP_FUSE_MOUNTOPTS_KEY		  = 164,
         ARGP_FUSE_USE_READDIRP_KEY        = 165,
 	ARGP_AUX_GFID_MOUNT_KEY		  = 166,
+        ARGP_FUSE_NO_ROOT_SQUASH_KEY      = 167,
+        ARGP_LOGGER                       = 168,
+        ARGP_LOG_FORMAT                   = 169,
+        ARGP_LOG_BUF_SIZE                 = 170,
+        ARGP_LOG_FLUSH_TIMEOUT            = 171,
+        ARGP_SECURE_MGMT_KEY              = 172,
+        ARGP_GLOBAL_TIMER_WHEEL           = 173,
+        ARGP_RESOLVE_GIDS_KEY             = 174,
 };
 
 struct _gfd_vol_top_priv_t {
