@@ -339,35 +339,6 @@ out:
 }
 
 static int
-glfsh_print_brick (xlator_t *xl, loc_t *rootloc)
-{
-        int     ret = 0;
-        dict_t  *xattr = NULL;
-        char    *pathinfo = NULL;
-        char    *brick_start = NULL;
-        char    *brick_end = NULL;
-
-        ret = syncop_getxattr (xl, rootloc, &xattr, GF_XATTR_PATHINFO_KEY,
-                               NULL, NULL);
-        if (ret < 0)
-                goto out;
-
-        ret = dict_get_str (xattr, GF_XATTR_PATHINFO_KEY, &pathinfo);
-        if (ret < 0)
-                goto out;
-
-        brick_start = strchr (pathinfo, ':') + 1;
-        brick_end = pathinfo + strlen (pathinfo) - 1;
-        *brick_end = 0;
-        printf ("Brick %s\n", brick_start);
-
-out:
-        if (xattr)
-                dict_unref (xattr);
-        return ret;
-}
-
-void
 glfsh_print_brick_from_xl (xlator_t *xl)
 {
         char    *remote_host = NULL;
@@ -386,6 +357,7 @@ out:
                 printf ("Brick - Not able to get brick information\n");
         else
                 printf ("Brick %s:%s\n", remote_host, remote_subvol);
+        return ret;
 }
 
 void
@@ -405,12 +377,9 @@ glfsh_print_pending_heals (glfs_t *fs, xlator_t *top_subvol, loc_t *rootloc,
         ret = dict_set_int32 (xattr_req, "heal-op", heal_op);
         if (ret)
                 goto out;
-        ret = glfsh_print_brick (xl, rootloc);
-        if (ret < 0) {
-                glfsh_print_brick_from_xl (xl);
-                printf ("Status: %s\n", strerror (-ret));
+        ret = glfsh_print_brick_from_xl (xl);
+        if (ret < 0)
                 goto out;
-        }
 
         ret = glfsh_get_index_dir_loc (rootloc, xl, &dirloc, &op_errno);
         if (ret < 0) {
@@ -534,6 +503,27 @@ glfsh_gather_heal_info (glfs_t *fs, xlator_t *top_subvol, loc_t *rootloc,
         return 0;
 }
 
+int
+_validate_directory (dict_t *xattr_req, char *file)
+{
+        int heal_op = -1;
+        int ret = 0;
+
+        ret = dict_get_int32 (xattr_req, "heal-op", &heal_op);
+        if (ret)
+                return ret;
+
+        if (heal_op == GF_SHD_OP_SBRAIN_HEAL_FROM_BIGGER_FILE) {
+                printf ("'bigger-file' not a valid option for directories.\n");
+                ret = -1;
+        } else if (heal_op == GF_SHD_OP_SBRAIN_HEAL_FROM_BRICK) {
+                printf ("'source-brick' option used on a directory (%s). "
+                        "Performing conservative merge.\n", file);
+        }
+
+        return ret;
+}
+
 int
 glfsh_heal_splitbrain_file (glfs_t *fs, xlator_t *top_subvol, loc_t *rootloc,
                            char *file, dict_t *xattr_req)
@@ -556,7 +546,7 @@ glfsh_heal_splitbrain_file (glfs_t *fs, xlator_t *top_subvol, loc_t *rootloc,
                 gf_uuid_parse (path, loc.gfid);
                 loc.path = gf_strdup (uuid_utoa (loc.gfid));
                 loc.inode = inode_new (rootloc->inode->table);
-                ret = syncop_lookup (xl, &loc, 0, 0, xattr_req, &xattr_rsp);
+                ret = syncop_lookup (xl, &loc, &iatt, 0, xattr_req, &xattr_rsp);
                 if (ret) {
                         op_errno = -ret;
                         printf ("Lookup failed on %s:%s.\n", file,
@@ -580,6 +570,11 @@ retry:
                 }
         }
 
+        if (iatt.ia_type == IA_IFDIR) {
+                ret = _validate_directory (xattr_req, file);
+                if (ret)
+                        goto out;
+        }
         ret = syncop_getxattr (xl, &loc, &xattr_rsp, GF_AFR_HEAL_SBRAIN,
                                xattr_req, NULL);
         if (ret) {
