@@ -131,6 +131,7 @@ enum _gf_client_pid
 
 enum _gf_xlator_ipc_targets {
         GF_IPC_TARGET_CHANGELOG = 0,
+        GF_IPC_TARGET_CTR = 1
 };
 
 typedef enum _gf_boolean gf_boolean_t;
@@ -300,8 +301,7 @@ int  gf_set_log_ident (cmd_args_t *cmd_args);
                         gf_msg (this->name, GF_LOG_ERROR, op_errno,     \
                                 LG_MSG_NO_PERM,                         \
                                 "attempt to remove internal "           \
-                                "xattr: %s: %s", key,                   \
-                                strerror (op_errno));                   \
+                                "xattr: %s", key);                      \
                         goto label;                                     \
                 }                                                       \
         } while (0)
