@@ -31,11 +31,25 @@
 #include <sys/un.h>
 #include <linux/limits.h>
 #include <sys/xattr.h>
+#include <linux/xattr.h>
 #include <endian.h>
 #ifdef HAVE_LINUX_FALLOC_H
 #include <linux/falloc.h>
 #endif
 
+#ifdef HAVE_ENDIAN_H
+#include <endian.h>
+#endif
+
+#ifndef _PATH_UMOUNT
+#define _PATH_UMOUNT "/bin/umount"
+#endif
+#endif /* GF_LINUX_HOST_OS */
+
+#ifdef HAVE_XATTR_H
+#include <sys/xattr.h>
+#endif
+
 /*
  * Define the fallocate flags in case we do not have the header. This also
  * accounts for older systems that do not define FALLOC_FL_PUNCH_HOLE.
@@ -61,7 +75,29 @@
 #define lsetxattr(path,key,value,size,flags) setxattr(path,key,value,size,flags)
 
 #endif /* HAVE_LLISTXATTR */
-#endif /* GF_LINUX_HOST_OS */
+
+
+#ifdef GF_DARWIN_HOST_OS
+#include <machine/endian.h>
+#include <libkern/OSByteOrder.h>
+
+#define htobe16(x) OSSwapHostToBigInt16(x)
+#define htole16(x) OSSwapHostToLittleInt16(x)
+#define be16toh(x) OSSwapBigToHostInt16(x)
+#define le16toh(x) OSSwapLittleToHostInt16(x)
+
+#define htobe32(x) OSSwapHostToBigInt32(x)
+#define htole32(x) OSSwapHostToLittleInt32(x)
+#define be32toh(x) OSSwapBigToHostInt32(x)
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+
+#define htobe64(x) OSSwapHostToBigInt64(x)
+#define htole64(x) OSSwapHostToLittleInt64(x)
+#define be64toh(x) OSSwapBigToHostInt64(x)
+#define le64toh(x) OSSwapLittleToHostInt64(x)
+
+#endif
+
 
 #ifdef GF_BSD_HOST_OS
 /* In case of FreeBSD and NetBSD */
@@ -93,6 +129,11 @@ enum {
 #define sighandler_t sig_t
 #endif
 
+#ifdef __FreeBSD__
+#undef ino_t
+#define ino_t uint64_t
+#endif /* __FreeBSD__ */
+
 #ifndef ino64_t
 #define ino64_t ino_t
 #endif
@@ -121,16 +162,41 @@ enum {
 #define F_GETLK64       F_GETLK
 #define F_SETLK64       F_SETLK
 #define F_SETLKW64      F_SETLKW
+#define FALLOC_FL_KEEP_SIZE     0x01 /* default is extend size */
+#define FALLOC_FL_PUNCH_HOLE    0x02 /* de-allocates range */
 
+#ifndef _PATH_UMOUNT
+  #define _PATH_UMOUNT "/sbin/umount"
+#endif
 #endif /* GF_BSD_HOST_OS */
 
 #ifdef GF_DARWIN_HOST_OS
+#include <machine/endian.h>
+#include <libkern/OSByteOrder.h>
+
+#define htobe16(x) OSSwapHostToBigInt16(x)
+#define htole16(x) OSSwapHostToLittleInt16(x)
+#define be16toh(x) OSSwapBigToHostInt16(x)
+#define le16toh(x) OSSwapLittleToHostInt16(x)
+
+#define htobe32(x) OSSwapHostToBigInt32(x)
+#define htole32(x) OSSwapHostToLittleInt32(x)
+#define be32toh(x) OSSwapBigToHostInt32(x)
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+
+#define htobe64(x) OSSwapHostToBigInt64(x)
+#define htole64(x) OSSwapHostToLittleInt64(x)
+#define be64toh(x) OSSwapBigToHostInt64(x)
+#define le64toh(x) OSSwapLittleToHostInt64(x)
 
 #define UNIX_PATH_MAX 104
+/* OSX Yosemite now has this defined */
+#ifndef AT_SYMLINK_NOFOLLOW
+#define AT_SYMLINK_NOFOLLOW 0x100
+#endif
 #include <sys/types.h>
 
 #include <sys/un.h>
-#include <machine/endian.h>
 #include <sys/xattr.h>
 #include <limits.h>
 
@@ -183,6 +249,9 @@ int32_t gf_darwin_compat_listxattr (int len, dict_t *dict, int size);
 int32_t gf_darwin_compat_getxattr (const char *key, dict_t *dict);
 int32_t gf_darwin_compat_setxattr (dict_t *dict);
 
+#ifndef _PATH_UMOUNT
+  #define _PATH_UMOUNT "/sbin/umount"
+#endif
 #endif /* GF_DARWIN_HOST_OS */
 
 #ifdef GF_SOLARIS_HOST_OS
@@ -262,6 +331,9 @@ enum {
 #ifndef _PATH_MOUNTED
  #define _PATH_MOUNTED "/etc/mtab"
 #endif
+#ifndef _PATH_UMOUNT
+ #define _PATH_UMOUNT "/sbin/umount"
+#endif
 
 #ifndef O_ASYNC
   #ifdef FASYNC
@@ -408,4 +480,6 @@ int gf_mkostemp (char *tmpl, int suffixlen, int flags);
 #pragma GCC poison system popen
 #endif
 
+int gf_umount_lazy(char *xlname, char *path, int rmdir);
+
 #endif /* __COMPAT_H__ */
