@@ -10,21 +10,7 @@
 #ifndef __GFDB_DATA_STORE_TYPE_H
 #define __GFDB_DATA_STORE_TYPE_H
 
-
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
-#include <time.h>
-#include <sys/time.h>
-#include <string.h>
-
-#include "common-utils.h"
-#include "compat-uuid.h"
-#include "gfdb_mem-types.h"
-#include "dict.h"
-#include "libglusterfs-messages.h"
+#include "gfdb_data_store_helper.h"
 
 /*
  * Helps in dynamically choosing log level
@@ -89,11 +75,11 @@ typedef enum gf_db_operation {
 typedef  struct timeval gfdb_time_t;
 
 /*Convert time into  seconds*/
-static inline long int
+static inline uint64_t
 gfdb_time_2_usec(gfdb_time_t *gfdb_time)
 {
         GF_ASSERT(gfdb_time);
-        return gfdb_time->tv_sec * GFDB_MICROSEC + gfdb_time->tv_usec;
+        return ((uint64_t) gfdb_time->tv_sec * GFDB_MICROSEC) + gfdb_time->tv_usec;
 }
 
 
@@ -154,7 +140,6 @@ typedef enum gfdb_db_type {
 } gfdb_db_type_t;
 
 /*String related to the db types*/
-#define GFDB_DATA_STORE               "gfdbdatastore"
 #define GFDB_STR_HASH_FILE_STORE      "hashfile"
 #define GFDB_STR_ROCKS_DB             "rocksdb"
 #define GFDB_STR_SQLITE3              "sqlite3"
@@ -319,168 +304,6 @@ typedef struct gfdb_db_record {
 } gfdb_db_record_t;
 
 
-
-/*******************************************************************************
- *
- *                     Query related data structure and functions
- *
- * ****************************************************************************/
-
-
-
-/*Structure used for querying purpose*/
-typedef struct gfdb_query_record {
-        /*Inode info*/
-        uuid_t                          gfid;
-        /*All the hard link of the inode
-         * All the hard links will be queried as
-         * "GF_PID,FNAME,FPATH,W_DEL_FLAG,LINK_UPDATE"
-         * and multiple hardlinks will be seperated by "::"*/
-        /*Do only shallow copy. The gf_query_callback_t */
-        /* function should do the deep copy.*/
-        char                            *_link_info_str;
-        ssize_t                         link_info_size;
-} gfdb_query_record_t;
-
-/*Function to create the query_record*/
-static inline gfdb_query_record_t *
-gfdb_query_record_init()
-{
-        int ret = -1;
-        gfdb_query_record_t *gfdb_query_record = NULL;
-
-        gfdb_query_record = GF_CALLOC (1, sizeof(gfdb_query_record_t),
-                                        gf_mt_gfdb_query_record_t);
-        if (!gfdb_query_record) {
-                gf_msg (GFDB_DATA_STORE, GF_LOG_ERROR, ENOMEM,
-                        LG_MSG_NO_MEMORY, "Error allocating memory to "
-                        "gfdb_query_record ");
-                goto out;
-        }
-        ret = 0;
-out:
-        if (ret == -1) {
-                GF_FREE (gfdb_query_record);
-        }
-        return gfdb_query_record;
-}
-
-/*Function to destroy query record*/
-static inline void
-gfdb_query_record_fini(gfdb_query_record_t
-                                                **gfdb_query_record) {
-        GF_FREE (*gfdb_query_record);
-}
-
-
-
-
-
-
-
-
-/*Structure to hold the link information*/
-typedef struct gfdb_link_info {
-        uuid_t                          pargfid;
-        char                            file_name[PATH_MAX];
-        char                            file_path[PATH_MAX];
-        gf_boolean_t                    is_link_updated;
-        gf_boolean_t                    is_del_flag_set;
-} gfdb_link_info_t;
-
-/*Create a single link info structure*/
-static inline gfdb_link_info_t *
-gfdb_link_info_init ()
-{
-        gfdb_link_info_t *gfdb_link_info = NULL;
-
-        gfdb_link_info = GF_CALLOC (1, sizeof(gfdb_link_info_t),
-                                        gf_mt_gfdb_link_info_t);
-        if (!gfdb_link_info) {
-                gf_msg (GFDB_DATA_STORE, GF_LOG_ERROR, ENOMEM,
-                        LG_MSG_NO_MEMORY, "Error allocating memory to "
-                        "gfdb_link_info ");
-        }
-
-        return gfdb_link_info;
-}
-
-/*Destroy a link info structure*/
-static inline void
-gfdb_link_info_fini(gfdb_link_info_t **gfdb_link_info)
-{
-        if (gfdb_link_info)
-                GF_FREE (*gfdb_link_info);
-}
-
-
-/*Length of each hard link string */
-#define DEFAULT_LINK_INFO_STR_LEN       1024
-
-/* Parse a single link string into link_info structure
- * Input format of str_link
- *      "GF_PID,FNAME,FPATH,W_DEL_FLAG,LINK_UPDATE"
- *
- * */
-static inline int
-str_to_link_info (char *str_link,
-                             gfdb_link_info_t *link_info)
-{
-        int     ret = -1;
-        const char *delimiter = ",";
-        char *token_str = NULL;
-        char *saveptr = NULL;
-        char gfid[200] = "";
-
-        GF_ASSERT (str_link);
-        GF_ASSERT (link_info);
-
-        /*Parent GFID*/
-        token_str = strtok_r(str_link, delimiter, &saveptr);
-        if (token_str != NULL) {
-                strcpy (gfid, token_str);
-                ret = gf_uuid_parse (gfid, link_info->pargfid);
-                if (ret == -1)
-                        goto out;
-        }
-
-        /*Filename*/
-        token_str = strtok_r(NULL, delimiter, &saveptr);
-        if (token_str != NULL) {
-                strcpy (link_info->file_name, token_str);
-        }
-
-        /*Filepath*/
-        token_str = strtok_r(NULL, delimiter, &saveptr);
-        if (token_str != NULL) {
-                strcpy (link_info->file_path, token_str);
-        }
-
-        /*is_link_updated*/
-        token_str = strtok_r(NULL, delimiter, &saveptr);
-        if (token_str != NULL) {
-                link_info->is_link_updated = atoi(token_str);
-                if (link_info->is_link_updated != 0 &&
-                                link_info->is_link_updated != 1) {
-                        goto out;
-                }
-        }
-
-        /*is_del_flag_set*/
-        token_str = strtok_r(NULL, delimiter, &saveptr);
-        if (token_str != NULL) {
-                link_info->is_del_flag_set = atoi (token_str);
-                if (link_info->is_del_flag_set != 0 &&
-                                link_info->is_del_flag_set != 1) {
-                        goto out;
-                }
-        }
-        ret = 0;
-out:
-        return ret;
-}
-
-
 /*******************************************************************************
  *
  *                           Signatures for the plugin functions
@@ -654,6 +477,14 @@ typedef int
                                         gf_boolean_t _clear_counters);
 
 
+typedef int (*gfdb_clear_files_heat_t)(void *db_conn);
+
+typedef int (*gfdb_get_db_version_t)(void *db_conn,
+                                        char **version);
+
+typedef int (*gfdb_get_db_setting_t)(void *db_conn,
+                                char *param_key,
+                                char **param_value);
 
 
 /*Data structure holding all the above plugin function pointers*/
@@ -669,10 +500,11 @@ typedef struct gfdb_db_operations {
                                         find_unchanged_for_time_freq_op;
         gfdb_find_recently_changed_files_freq_t
                                         find_recently_changed_files_freq_op;
+        gfdb_clear_files_heat_t clear_files_heat_op;
+        gfdb_get_db_version_t           get_db_version;
+        gfdb_get_db_setting_t           get_db_setting;
 } gfdb_db_operations_t;
 
-
-
 /*******************************************************************************
  *
  * Database connection object: This objected is maitained by libgfdb for each
@@ -714,6 +546,7 @@ typedef struct gfdb_connection {
                         gf_msg (comp_name, GF_LOG_ERROR, 0,\
                                 LG_MSG_SET_PARAM_FAILED, "Failed setting %s "\
                                 "to params dictionary", param_key);\
+                        data_destroy (data);\
                         goto error;\
                 };\
         } while (0)
