@@ -89,9 +89,6 @@
 #define GF_XATTR_LOCKINFO_KEY   "trusted.glusterfs.lockinfo"
 #define GF_XATTR_GET_REAL_FILENAME_KEY "glusterfs.get_real_filename:"
 #define GF_XATTR_USER_PATHINFO_KEY   "glusterfs.pathinfo"
-#define QUOTA_LIMIT_KEY "trusted.glusterfs.quota.limit-set"
-#define QUOTA_LIMIT_OBJECTS_KEY "trusted.glusterfs.quota.limit-objects"
-#define VIRTUAL_QUOTA_XATTR_CLEANUP_KEY "glusterfs.quota-xattr-cleanup"
 #define GF_INTERNAL_IGNORE_DEEM_STATFS "ignore-deem-statfs"
 
 #define GF_READDIR_SKIP_DIRS       "readdir-filter-directories"
@@ -161,7 +158,11 @@
 #define GF_XATTR_STIME_PATTERN "trusted.glusterfs.*.stime"
 #define GF_XATTR_TRIGGER_SYNC "glusterfs.geo-rep.trigger-sync"
 
+/* quota xattrs */
 #define QUOTA_SIZE_KEY "trusted.glusterfs.quota.size"
+#define QUOTA_LIMIT_KEY "trusted.glusterfs.quota.limit-set"
+#define QUOTA_LIMIT_OBJECTS_KEY "trusted.glusterfs.quota.limit-objects"
+#define VIRTUAL_QUOTA_XATTR_CLEANUP_KEY "glusterfs.quota-xattr-cleanup"
 
 /* Index xlator related */
 #define GF_XATTROP_INDEX_GFID "glusterfs.xattrop_index_gfid"
@@ -244,6 +245,8 @@
 #define CTR_RESPONSE_LINK_COUNT_XDATA "ctr_response_link_count"
 #define CTR_REQUEST_LINK_COUNT_XDATA  "ctr_request_link_count"
 
+#define CTR_ATTACH_TIER_LOOKUP    "ctr_attach_tier_lookup"
+
 #define GF_LOG_LRU_BUFSIZE_DEFAULT 5
 #define GF_LOG_LRU_BUFSIZE_MIN 0
 #define GF_LOG_LRU_BUFSIZE_MAX 20
@@ -377,7 +380,10 @@ typedef enum {
         GF_XATTROP_ADD_ARRAY,
         GF_XATTROP_ADD_ARRAY64,
         GF_XATTROP_OR_ARRAY,
-        GF_XATTROP_AND_ARRAY
+        GF_XATTROP_AND_ARRAY,
+        GF_XATTROP_GET_AND_SET,
+        GF_XATTROP_ADD_ARRAY_WITH_DEFAULT,
+        GF_XATTROP_ADD_ARRAY64_WITH_DEFAULT
 } gf_xattrop_flags_t;
 
 #define GF_SET_IF_NOT_PRESENT 0x1 /* default behaviour */
@@ -470,6 +476,7 @@ struct _cmd_args {
         int              congestion_threshold;
         char             *fuse_mountopts;
         int              mem_acct;
+        int              resolve_gids;
 
         /* key args */
         char            *mount_point;
