@@ -196,6 +196,21 @@ glusterfs_graph_worm (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
         return ret;
 }
 
+
+int
+glusterfs_graph_meta (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
+{
+        int ret = 0;
+
+	if (!ctx->master)
+		return 0;
+
+        ret = glusterfs_graph_insert (graph, ctx, "meta",
+                                      "meta-autoload", 1);
+        return ret;
+}
+
+
 int
 glusterfs_graph_mac_compat (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
 {
@@ -313,6 +328,19 @@ glusterfs_graph_init (glusterfs_graph_t *graph)
         return 0;
 }
 
+int
+glusterfs_graph_deactivate (glusterfs_graph_t *graph)
+{
+        xlator_t           *top = NULL;
+
+        if (graph == NULL)
+                goto out;
+
+        top = graph->top;
+        xlator_tree_fini (top);
+ out:
+        return 0;
+}
 
 static int
 _log_if_unknown_option (dict_t *dict, char *key, data_t *value, void *data)
@@ -367,7 +395,7 @@ fill_uuid (char *uuid, int size)
                         strerror (errno));
         }
 
-        gf_time_fmt (now_str, sizeof now_str, tv.tv_sec, gf_timefmt_Ymd_T);
+        gf_time_fmt (now_str, sizeof now_str, tv.tv_sec, gf_timefmt_dirent);
         snprintf (uuid, size, "%s-%d-%s:%"GF_PRI_SUSECONDS,
                   hostname, getpid(), now_str, tv.tv_usec);
 
@@ -464,6 +492,14 @@ glusterfs_graph_prepare (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
                 return -1;
         }
 
+	/* XXX: topmost xlator */
+	ret = glusterfs_graph_meta (graph, ctx);
+	if (ret) {
+		gf_log ("graph", GF_LOG_ERROR,
+			"glusterfs graph meta failed");
+		return -1;
+	}
+
         /* XXX: this->ctx setting */
         for (trav = graph->first; trav; trav = trav->next) {
                 trav->ctx = ctx;
@@ -479,15 +515,138 @@ glusterfs_graph_prepare (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
         /* XXX: --xlator-option additions */
         gf_add_cmdline_options (graph, &ctx->cmd_args);
 
-
         return 0;
 }
 
+static
+xlator_t *glusterfs_root(glusterfs_graph_t *graph)
+{
+        return graph->first;
+}
+
+static
+int glusterfs_is_leaf(xlator_t *xl)
+{
+        int ret = 0;
+
+        if (!xl->children)
+                ret = 1;
+
+        return ret;
+}
+
+static
+uint32_t glusterfs_count_leaves(xlator_t *xl)
+{
+        int n = 0;
+        xlator_list_t *list = NULL;
+
+        if (glusterfs_is_leaf(xl))
+                n = 1;
+        else
+                for (list = xl->children; list; list = list->next)
+                        n += glusterfs_count_leaves(list->xlator);
+
+        return n;
+}
+
+int glusterfs_get_leaf_count(glusterfs_graph_t *graph)
+{
+        return graph->leaf_count;
+}
+
+static
+int _glusterfs_leaf_position(xlator_t *tgt, int *id, xlator_t *xl)
+{
+        xlator_list_t *list = NULL;
+        int found = 0;
+
+        if (xl == tgt)
+                found = 1;
+        else if (glusterfs_is_leaf(xl))
+                *id += 1;
+        else
+                for (list = xl->children; !found && list; list = list->next)
+                        found = _glusterfs_leaf_position(tgt, id, list->xlator);
+
+        return found;
+}
+
+int glusterfs_leaf_position(xlator_t *tgt)
+{
+        xlator_t *root = NULL;
+        int pos = 0;
+
+        root = glusterfs_root(tgt->graph);
+
+        if (!_glusterfs_leaf_position(tgt, &pos, root))
+                pos = -1;
+
+        return pos;
+}
+
+static int
+_glusterfs_reachable_leaves(xlator_t *base, xlator_t *xl, dict_t *leaves)
+{
+        xlator_list_t *list = NULL;
+        int err = 1;
+        int pos = 0;
+        char strpos[6];
+
+        if (glusterfs_is_leaf(xl)) {
+                pos = glusterfs_leaf_position(xl);
+                if (pos < 0)
+                        goto out;
+                sprintf(strpos, "%d", pos);
+
+                err = dict_set_static_ptr(leaves, strpos, base);
+
+        } else {
+                for (err = 0, list = xl->children;
+                     !err && list;
+                     list = list->next)
+                        err = _glusterfs_reachable_leaves(base, list->xlator,
+                                                          leaves);
+        }
+
+out:
+        return err;
+}
+
+/*
+ * This function determines which leaves are children (or grandchildren)
+ * of the given base. The base may have multiple sub volumes. Each sub
+ * volumes in turn may have sub volumes.. until the leaves are reached.
+ * Each leaf is numbered 1,2,3,...etc.
+ *
+ * The base translator calls this function to see which of *its* subvolumes
+ * it would forward an FOP to, to *get to* a particular leaf.
+ * That information is built into the "leaves" dictionary.
+ * key:destination leaf# -> value:base subvolume xlator.
+ */
+
+int
+glusterfs_reachable_leaves(xlator_t *base, dict_t *leaves)
+{
+        xlator_list_t *list = NULL;
+        int err = 0;
+
+        for (list = base->children; !err && list; list = list->next)
+                err = _glusterfs_reachable_leaves(list->xlator,
+                                                  list->xlator, leaves);
+
+        return err;
+}
 
 int
 glusterfs_graph_activate (glusterfs_graph_t *graph, glusterfs_ctx_t *ctx)
 {
         int ret = 0;
+        xlator_t *root = NULL;
+
+        root = glusterfs_root(graph);
+
+        graph->leaf_count = glusterfs_count_leaves(root);
 
         /* XXX: all xlator options validation */
         ret = glusterfs_graph_validate_options (graph);
@@ -671,6 +830,8 @@ glusterfs_volfile_reconfigure (int oldvollen, FILE *newvolfile_fp,
                 goto out;
         }
 
+	glusterfs_graph_prepare (newvolfile_graph, ctx);
+
         if (!is_graph_topology_equal (oldvolfile_graph,
                                       newvolfile_graph)) {
 
@@ -737,15 +898,54 @@ glusterfs_graph_reconfigure (glusterfs_graph_t *oldgraph,
         return xlator_tree_reconfigure (old_xl, new_xl);
 }
 
+int
+glusterfs_graph_destroy_residual (glusterfs_graph_t *graph)
+{
+        int ret = -1;
+
+        if (graph == NULL)
+                return ret;
+
+        ret = xlator_tree_free_memacct (graph->first);
+
+        list_del_init (&graph->list);
+        GF_FREE (graph);
+
+        return ret;
+}
+
+/* This function destroys all the xlator members except for the
+ * xlator strcuture and its mem accounting field.
+ *
+ * If otherwise, it would destroy the master xlator object as well
+ * its mem accounting, which would mean after calling glusterfs_graph_destroy()
+ * there cannot be any reference to GF_FREE() from the master xlator, this is
+ * not possible because of the following dependencies:
+ * - glusterfs_ctx_t will have mem pools allocated by the master xlators
+ * - xlator objects will have references to those mem pools(g: dict)
+ *
+ * Ordering the freeing in any of the order will also not solve the dependency:
+ * - Freeing xlator objects(including memory accounting) before mem pools
+ *   destruction will mean not use GF_FREE while destroying mem pools.
+ * - Freeing mem pools and then destroying xlator objects would lead to crashes
+ *   when xlator tries to unref dict or other mem pool objects.
+ *
+ * Hence the way chosen out of this interdependency is to split xlator object
+ * free into two stages:
+ * - Free all the xlator members excpet for its mem accounting structure
+ * - Free all the mem accouting structures of xlator along with the xlator
+ *   object itself.
+ */
 int
 glusterfs_graph_destroy (glusterfs_graph_t *graph)
 {
-        xlator_tree_free (graph->first);
+        int ret = 0;
 
-        if (graph) {
-                list_del_init (&graph->list);
-                GF_FREE (graph);
-        }
+        GF_VALIDATE_OR_GOTO ("graph", graph, out);
 
-        return 0;
+        ret = xlator_tree_free_members (graph->first);
+
+        ret = glusterfs_graph_destroy_residual (graph);
+out:
+        return ret;
 }
