@@ -22,78 +22,84 @@
 #define is_mem_chunk_in_use(ptr)         (*ptr == 1)
 #define mem_pool_from_ptr(ptr)           ((ptr) + GF_MEM_POOL_LIST_BOUNDARY)
 
-#define GF_MEM_HEADER_SIZE  (4 + sizeof (size_t) + sizeof (xlator_t *) + 4 + 8)
-#define GF_MEM_TRAILER_SIZE 8
-
-#define GF_MEM_HEADER_MAGIC  0xCAFEBABE
-#define GF_MEM_TRAILER_MAGIC 0xBAADF00D
-
 #define GLUSTERFS_ENV_MEM_ACCT_STR  "GLUSTERFS_DISABLE_MEM_ACCT"
 
+#include "unittest/unittest.h"
+#include "libglusterfs-messages.h"
+
 void
 gf_mem_acct_enable_set (void *data)
 {
         glusterfs_ctx_t *ctx = NULL;
 
+        REQUIRE(data != NULL);
+
         ctx = data;
 
-        GF_ASSERT (ctx);
+        GF_ASSERT (ctx != NULL);
 
         ctx->mem_acct_enable = 1;
 
+        ENSURE(1 == ctx->mem_acct_enable);
+
         return;
 }
 
-void
-gf_mem_set_acct_info (xlator_t *xl, char **alloc_ptr,
-                      size_t size, uint32_t type)
+int
+gf_mem_set_acct_info (xlator_t *xl, char **alloc_ptr, size_t size,
+		      uint32_t type, const char *typestr)
 {
 
-        char    *ptr = NULL;
+        void              *ptr    = NULL;
+        struct mem_header *header = NULL;
 
         if (!alloc_ptr)
-                return;
+                return -1;
 
-        ptr = (char *) (*alloc_ptr);
+        ptr = *alloc_ptr;
 
         GF_ASSERT (xl != NULL);
 
-        GF_ASSERT (xl->mem_acct.rec != NULL);
+        GF_ASSERT (xl->mem_acct != NULL);
 
-        GF_ASSERT (type <= xl->mem_acct.num_types);
+        GF_ASSERT (type <= xl->mem_acct->num_types);
 
-        LOCK(&xl->mem_acct.rec[type].lock);
+        LOCK(&xl->mem_acct->rec[type].lock);
         {
-                xl->mem_acct.rec[type].size += size;
-                xl->mem_acct.rec[type].num_allocs++;
-                xl->mem_acct.rec[type].total_allocs++;
-                xl->mem_acct.rec[type].max_size =
-                        max (xl->mem_acct.rec[type].max_size,
-                             xl->mem_acct.rec[type].size);
-                xl->mem_acct.rec[type].max_num_allocs =
-                        max (xl->mem_acct.rec[type].max_num_allocs,
-                             xl->mem_acct.rec[type].num_allocs);
+		if (!xl->mem_acct->rec[type].typestr)
+			xl->mem_acct->rec[type].typestr = typestr;
+                xl->mem_acct->rec[type].size += size;
+                xl->mem_acct->rec[type].num_allocs++;
+                xl->mem_acct->rec[type].total_allocs++;
+                xl->mem_acct->rec[type].max_size =
+                        max (xl->mem_acct->rec[type].max_size,
+                             xl->mem_acct->rec[type].size);
+                xl->mem_acct->rec[type].max_num_allocs =
+                        max (xl->mem_acct->rec[type].max_num_allocs,
+                             xl->mem_acct->rec[type].num_allocs);
         }
-        UNLOCK(&xl->mem_acct.rec[type].lock);
+        UNLOCK(&xl->mem_acct->rec[type].lock);
 
-        *(uint32_t *)(ptr) = type;
-        ptr = ptr + 4;
-        memcpy (ptr, &size, sizeof(size_t));
-        ptr += sizeof (size_t);
-        memcpy (ptr, &xl, sizeof(xlator_t *));
-        ptr += sizeof (xlator_t *);
-        *(uint32_t *)(ptr) = GF_MEM_HEADER_MAGIC;
-        ptr = ptr + 4;
-        ptr = ptr + 8; //padding
+        INCREMENT_ATOMIC (xl->mem_acct->lock, xl->mem_acct->refcnt);
+
+        header = (struct mem_header *) ptr;
+        header->type = type;
+        header->size = size;
+        header->mem_acct = xl->mem_acct;
+        header->magic = GF_MEM_HEADER_MAGIC;
+
+        ptr += sizeof (struct mem_header);
+
+        /* data follows in this gap of 'size' bytes */
         *(uint32_t *) (ptr + size) = GF_MEM_TRAILER_MAGIC;
 
-        *alloc_ptr = (void *)ptr;
-        return;
+        *alloc_ptr = ptr;
+        return 0;
 }
 
 
 void *
-__gf_calloc (size_t nmemb, size_t size, uint32_t type)
+__gf_calloc (size_t nmemb, size_t size, uint32_t type, const char *typestr)
 {
         size_t          tot_size = 0;
         size_t          req_size = 0;
@@ -111,16 +117,16 @@ __gf_calloc (size_t nmemb, size_t size, uint32_t type)
         ptr = calloc (1, tot_size);
 
         if (!ptr) {
-                gf_log_nomem ("", GF_LOG_ALERT, tot_size);
+                gf_msg_nomem ("", GF_LOG_ALERT, tot_size);
                 return NULL;
         }
-        gf_mem_set_acct_info (xl, &ptr, req_size, type);
+        gf_mem_set_acct_info (xl, &ptr, req_size, type, typestr);
 
         return (void *)ptr;
 }
 
 void *
-__gf_malloc (size_t size, uint32_t type)
+__gf_malloc (size_t size, uint32_t type, const char *typestr)
 {
         size_t          tot_size = 0;
         char            *ptr = NULL;
@@ -135,10 +141,10 @@ __gf_malloc (size_t size, uint32_t type)
 
         ptr = malloc (tot_size);
         if (!ptr) {
-                gf_log_nomem ("", GF_LOG_ALERT, tot_size);
+                gf_msg_nomem ("", GF_LOG_ALERT, tot_size);
                 return NULL;
         }
-        gf_mem_set_acct_info (xl, &ptr, size, type);
+        gf_mem_set_acct_info (xl, &ptr, size, type, typestr);
 
         return (void *)ptr;
 }
@@ -146,35 +152,55 @@ __gf_malloc (size_t size, uint32_t type)
 void *
 __gf_realloc (void *ptr, size_t size)
 {
-        size_t          tot_size = 0;
-        char            *orig_ptr = NULL;
-        xlator_t        *xl = NULL;
-        uint32_t        type = 0;
+        size_t             tot_size = 0;
+        char              *new_ptr;
+        struct mem_header *old_header = NULL;
+        struct mem_header *new_header = NULL;
+        struct mem_header  tmp_header;
 
         if (!THIS->ctx->mem_acct_enable)
                 return REALLOC (ptr, size);
 
+        REQUIRE(NULL != ptr);
+
+        old_header = (struct mem_header *) (ptr - GF_MEM_HEADER_SIZE);
+        GF_ASSERT (old_header->magic == GF_MEM_HEADER_MAGIC);
+        tmp_header = *old_header;
+
         tot_size = size + GF_MEM_HEADER_SIZE + GF_MEM_TRAILER_SIZE;
-
-        orig_ptr = (char *)ptr - 8 - 4;
-
-        GF_ASSERT (*(uint32_t *)orig_ptr == GF_MEM_HEADER_MAGIC);
-
-        orig_ptr = orig_ptr - sizeof(xlator_t *);
-        xl = *((xlator_t **)orig_ptr);
-
-        orig_ptr = (char *)ptr - GF_MEM_HEADER_SIZE;
-        type = *(uint32_t *)orig_ptr;
-
-        ptr = realloc (orig_ptr, tot_size);
-        if (!ptr) {
-                gf_log_nomem ("", GF_LOG_ALERT, tot_size);
+        new_ptr = realloc (old_header, tot_size);
+        if (!new_ptr) {
+                gf_msg_nomem ("", GF_LOG_ALERT, tot_size);
                 return NULL;
         }
 
-        gf_mem_set_acct_info (xl, (char **)&ptr, size, type);
+        /*
+         * We used to pass (char **)&ptr as the second
+         * argument after the value of realloc was saved
+         * in ptr, but the compiler warnings complained
+         * about the casting to and forth from void ** to
+         * char **.
+         * TBD: it would be nice to adjust the memory accounting info here,
+         * but calling gf_mem_set_acct_info here is wrong because it bumps
+         * up counts as though this is a new allocation - which it's not.
+         * The consequence of doing nothing here is only that the sizes will be
+         * wrong, but at least the counts won't be.
+        uint32_t           type = 0;
+        xlator_t          *xl = NULL;
+        type = header->type;
+        xl = (xlator_t *) header->xlator;
+        gf_mem_set_acct_info (xl, &new_ptr, size, type, NULL);
+         */
 
-        return (void *)ptr;
+        new_header = (struct mem_header *) new_ptr;
+        *new_header = tmp_header;
+        new_header->size = size;
+
+        new_ptr += sizeof (struct mem_header);
+        /* data follows in this gap of 'size' bytes */
+        *(uint32_t *) (new_ptr + size) = GF_MEM_TRAILER_MAGIC;
+
+        return (void *)new_ptr;
 }
 
 int
@@ -216,13 +242,53 @@ gf_asprintf (char **string_ptr, const char *format, ...)
         return rv;
 }
 
+#ifdef DEBUG
+void
+__gf_mem_invalidate (void *ptr)
+{
+        struct mem_header *header = ptr;
+        void              *end    = NULL;
+
+        struct mem_invalid inval = {
+                .magic = GF_MEM_INVALID_MAGIC,
+                .mem_acct = header->mem_acct,
+                .type = header->type,
+                .size = header->size,
+                .baseaddr = ptr + GF_MEM_HEADER_SIZE,
+        };
+
+        /* calculate the last byte of the allocated area */
+        end = ptr + GF_MEM_HEADER_SIZE + inval.size + GF_MEM_TRAILER_SIZE;
+
+        /* overwrite the old mem_header */
+        memcpy (ptr, &inval, sizeof (inval));
+        ptr += sizeof (inval);
+
+        /* zero out remaining (old) mem_header bytes) */
+        memset (ptr, 0x00, sizeof (*header) - sizeof (inval));
+        ptr += sizeof (*header) - sizeof (inval);
+
+        /* zero out the first byte of data */
+        *(uint32_t *)(ptr) = 0x00;
+        ptr += 1;
+
+        /* repeated writes of invalid structurein data area */
+        while ((ptr + (sizeof (inval))) < (end - 1)) {
+                memcpy (ptr, &inval, sizeof (inval));
+                ptr += sizeof (inval);
+        }
+
+        /* fill out remaining data area with 0xff */
+        memset (ptr, 0xff, end - ptr);
+}
+#endif /* DEBUG */
+
 void
 __gf_free (void *free_ptr)
 {
-        size_t          req_size = 0;
-        char            *ptr = NULL;
-        uint32_t        type = 0;
-        xlator_t        *xl = NULL;
+        void              *ptr = NULL;
+        struct mem_acct   *mem_acct;
+        struct mem_header *header = NULL;
 
         if (!THIS->ctx->mem_acct_enable) {
                 FREE (free_ptr);
@@ -232,43 +298,41 @@ __gf_free (void *free_ptr)
         if (!free_ptr)
                 return;
 
-        ptr = (char *)free_ptr - 8 - 4;
+        ptr = free_ptr - GF_MEM_HEADER_SIZE;
+        header = (struct mem_header *) ptr;
 
         //Possible corruption, assert here
-        GF_ASSERT (GF_MEM_HEADER_MAGIC == *(uint32_t *)ptr);
+        GF_ASSERT (GF_MEM_HEADER_MAGIC == header->magic);
 
-        *(uint32_t *)ptr = 0;
-
-        ptr = ptr - sizeof(xlator_t *);
-        memcpy (&xl, ptr, sizeof(xlator_t *));
-
-        //gf_free expects xl to be available
-        GF_ASSERT (xl != NULL);
-
-        if (!xl->mem_acct.rec) {
-                ptr = (char *)free_ptr - GF_MEM_HEADER_SIZE;
+        mem_acct = header->mem_acct;
+        if (!mem_acct) {
                 goto free;
         }
 
-
-        ptr = ptr - sizeof(size_t);
-        memcpy (&req_size, ptr, sizeof (size_t));
-        ptr = ptr - 4;
-        type = *(uint32_t *)ptr;
-
         // This points to a memory overrun
         GF_ASSERT (GF_MEM_TRAILER_MAGIC ==
-                *(uint32_t *)((char *)free_ptr + req_size));
+                *(uint32_t *)((char *)free_ptr + header->size));
 
-        *(uint32_t *) ((char *)free_ptr + req_size) = 0;
-
-        LOCK (&xl->mem_acct.rec[type].lock);
+        LOCK (&mem_acct->rec[header->type].lock);
         {
-                xl->mem_acct.rec[type].size -= req_size;
-                xl->mem_acct.rec[type].num_allocs--;
+                mem_acct->rec[header->type].size -= header->size;
+                mem_acct->rec[header->type].num_allocs--;
+                /* If all the instaces are freed up then ensure typestr is
+                 * set to NULL */
+                if (!mem_acct->rec[header->type].num_allocs)
+                        mem_acct->rec[header->type].typestr = NULL;
         }
-        UNLOCK (&xl->mem_acct.rec[type].lock);
+        UNLOCK (&mem_acct->rec[header->type].lock);
+
+        if (DECREMENT_ATOMIC (mem_acct->lock, mem_acct->refcnt) == 0) {
+                FREE (mem_acct);
+        }
+
 free:
+#ifdef DEBUG
+        __gf_mem_invalidate (ptr);
+#endif
+
         FREE (ptr);
 }
 
@@ -280,14 +344,15 @@ mem_pool_new_fn (unsigned long sizeof_type,
 {
         struct mem_pool  *mem_pool = NULL;
         unsigned long     padded_sizeof_type = 0;
-        void             *pool = NULL;
-        int               i = 0;
+        GF_UNUSED void             *pool = NULL;
+        GF_UNUSED int               i = 0;
         int               ret = 0;
-        struct list_head *list = NULL;
+        GF_UNUSED struct list_head *list = NULL;
         glusterfs_ctx_t  *ctx = NULL;
 
         if (!sizeof_type || !count) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
         padded_sizeof_type = sizeof_type + GF_MEM_POOL_PAD_BOUNDARY;
@@ -310,9 +375,10 @@ mem_pool_new_fn (unsigned long sizeof_type,
         INIT_LIST_HEAD (&mem_pool->global_list);
 
         mem_pool->padded_sizeof_type = padded_sizeof_type;
-        mem_pool->cold_count = count;
         mem_pool->real_sizeof_type = sizeof_type;
 
+#ifndef DEBUG
+        mem_pool->cold_count = count;
         pool = GF_CALLOC (count, padded_sizeof_type, gf_common_mt_long);
         if (!pool) {
                 GF_FREE (mem_pool->name);
@@ -328,6 +394,7 @@ mem_pool_new_fn (unsigned long sizeof_type,
 
         mem_pool->pool = pool;
         mem_pool->pool_end = pool + (count * (padded_sizeof_type));
+#endif
 
         /* add this pool to the global list */
         ctx = THIS->ctx;
@@ -346,7 +413,8 @@ mem_get0 (struct mem_pool *mem_pool)
         void             *ptr = NULL;
 
         if (!mem_pool) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
 
@@ -367,7 +435,8 @@ mem_get (struct mem_pool *mem_pool)
         struct mem_pool **pool_ptr = NULL;
 
         if (!mem_pool) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
 
@@ -438,7 +507,8 @@ static int
 __is_member (struct mem_pool *pool, void *ptr)
 {
         if (!pool || !ptr) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return -1;
         }
 
@@ -463,21 +533,24 @@ mem_put (void *ptr)
         struct mem_pool *pool = NULL;
 
         if (!ptr) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return;
         }
 
         list = head = mem_pool_ptr2chunkhead (ptr);
         tmp = mem_pool_from_ptr (head);
         if (!tmp) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR,
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, 0,
+                                  LG_MSG_PTR_HEADER_CORRUPTED,
                                   "ptr header is corrupted");
                 return;
         }
 
         pool = *tmp;
         if (!pool) {
-                gf_log_callingfn ("mem-pool", GF_LOG_ERROR,
+                gf_msg_callingfn ("mem-pool", GF_LOG_ERROR, 0,
+                                  LG_MSG_MEMPOOL_PTR_NULL,
                                   "mem-pool ptr is NULL");
                 return;
         }
@@ -490,9 +563,12 @@ mem_put (void *ptr)
                         in_use = (head + GF_MEM_POOL_LIST_BOUNDARY +
                                   GF_MEM_POOL_PTR);
                         if (!is_mem_chunk_in_use(in_use)) {
-                                gf_log_callingfn ("mem-pool", GF_LOG_CRITICAL,
-                                                  "mem_put called on freed ptr %p of mem "
-                                                  "pool %p", ptr, pool);
+                                gf_msg_callingfn ("mem-pool", GF_LOG_CRITICAL,
+                                                  0,
+                                                  LG_MSG_MEMPOOL_INVALID_FREE,
+                                                  "mem_put called on freed ptr"
+                                                  " %p of mem pool %p", ptr,
+                                                  pool);
                                 break;
                         }
                         pool->hot_count--;
@@ -535,8 +611,9 @@ mem_pool_destroy (struct mem_pool *pool)
         if (!pool)
                 return;
 
-        gf_log (THIS->name, GF_LOG_INFO, "size=%lu max=%d total=%"PRIu64,
-                pool->padded_sizeof_type, pool->max_alloc, pool->alloc_count);
+        gf_msg (THIS->name, GF_LOG_INFO, 0, LG_MSG_MEM_POOL_DESTROY, "size=%lu "
+                "max=%d total=%"PRIu64, pool->padded_sizeof_type,
+                pool->max_alloc, pool->alloc_count);
 
         list_del (&pool->global_list);
 
