@@ -17,6 +17,7 @@
 
 #include "xlator.h"
 #include "defaults.h"
+#include "libglusterfs-messages.h"
 
 #define GF_OPTION_LIST_EMPTY(_opt) (_opt->value[0] == NULL)
 
@@ -32,7 +33,8 @@ xlator_option_validate_path (xlator_t *xl, const char *key, const char *value,
                 snprintf (errstr, 256,
                           "invalid path given '%s'",
                           value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -42,7 +44,8 @@ xlator_option_validate_path (xlator_t *xl, const char *key, const char *value,
                           "option %s %s: '%s' is not an "
                           "absolute path name",
                           key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -58,6 +61,7 @@ xlator_option_validate_int (xlator_t *xl, const char *key, const char *value,
                             volume_option_t *opt, char **op_errstr)
 {
         long long inputll = 0;
+        unsigned long long uinputll = 0;
         int       ret = -1;
         char      errstr[256];
 
@@ -66,35 +70,47 @@ xlator_option_validate_int (xlator_t *xl, const char *key, const char *value,
                 snprintf (errstr, 256,
                           "invalid number format \"%s\" in option \"%s\"",
                           value, key);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
+                goto out;
+        }
+
+        /* Handle '-0' */
+        if ((inputll == 0) && (gf_string2ulonglong (value, &uinputll) != 0)) {
+                snprintf (errstr, 256,
+                          "invalid number format \"%s\" in option \"%s\"",
+                          value, key);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
         if ((opt->min == 0) && (opt->max == 0) &&
             (opt->validate == GF_OPT_VALIDATE_BOTH)) {
-                gf_log (xl->name, GF_LOG_TRACE,
-                        "no range check required for 'option %s %s'",
-                        key, value);
+                gf_msg_trace (xl->name, 0, "no range check required for "
+                              "'option %s %s'", key, value);
                 ret = 0;
                 goto out;
         }
 
-        if ((opt->validate == GF_OPT_VALIDATE_MIN)) {
+        if (opt->validate == GF_OPT_VALIDATE_MIN) {
                 if (inputll < opt->min) {
                         snprintf (errstr, 256,
                                   "'%lld' in 'option %s %s' is smaller than "
                                   "minimum value '%.0f'", inputll, key,
                                   value, opt->min);
-                        gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_INVALID_ENTRY, "%s", errstr);
                         goto out;
                 }
-        } else if ((opt->validate == GF_OPT_VALIDATE_MAX)) {
-                if ((inputll > opt->max)) {
+        } else if (opt->validate == GF_OPT_VALIDATE_MAX) {
+                if (inputll > opt->max) {
                         snprintf (errstr, 256,
                                   "'%lld' in 'option %s %s' is greater than "
                                   "maximum value '%.0f'", inputll, key,
                                   value, opt->max);
-                        gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_INVALID_ENTRY, "%s", errstr);
                         goto out;
                 }
         } else if ((inputll < opt->min) || (inputll > opt->max)) {
@@ -102,7 +118,8 @@ xlator_option_validate_int (xlator_t *xl, const char *key, const char *value,
                           "'%lld' in 'option %s %s' is out of range "
                           "[%.0f - %.0f]",
                           inputll, key, value, opt->min, opt->max);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -118,40 +135,42 @@ static int
 xlator_option_validate_sizet (xlator_t *xl, const char *key, const char *value,
                               volume_option_t *opt, char **op_errstr)
 {
-        uint64_t  size = 0;
+        size_t  size = 0;
         int       ret = 0;
         char      errstr[256];
 
         /* Check the range */
-        if (gf_string2bytesize (value, &size) != 0) {
+        if (gf_string2bytesize_size (value, &size) != 0) {
                 snprintf (errstr, 256,
                           "invalid number format \"%s\" in option \"%s\"",
                           value, key);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 ret = -1;
                 goto out;
         }
 
         if ((opt->min == 0) && (opt->max == 0)) {
-                gf_log (xl->name, GF_LOG_TRACE,
-                        "no range check required for 'option %s %s'",
-                        key, value);
+                gf_msg_trace (xl->name, 0, "no range check required for "
+                              "'option %s %s'", key, value);
                 goto out;
         }
 
         if ((size < opt->min) || (size > opt->max)) {
                 if ((strncmp (key, "cache-size", 10) == 0) &&
                     (size > opt->max)) {
-                       snprintf (errstr, 256, "Cache size %"PRId64" is out of "
+                       snprintf (errstr, 256, "Cache size %" GF_PRI_SIZET " is out of "
                                  "range [%.0f - %.0f]",
                                  size, opt->min, opt->max);
-                       gf_log (xl->name, GF_LOG_WARNING, "%s", errstr);
+                       gf_msg (xl->name, GF_LOG_WARNING, 0,
+                               LG_MSG_OUT_OF_RANGE, "%s", errstr);
                 } else {
                         snprintf (errstr, 256,
-                                  "'%"PRId64"' in 'option %s %s' "
+                                  "'%" GF_PRI_SIZET "' in 'option %s %s' "
                                   "is out of range [%.0f - %.0f]",
                                   size, key, value, opt->min, opt->max);
-                        gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_OUT_OF_RANGE, "%s", errstr);
                         ret = -1;
                 }
         }
@@ -179,7 +198,8 @@ xlator_option_validate_bool (xlator_t *xl, const char *key, const char *value,
                 snprintf (errstr, 256,
                           "option %s %s: '%s' is not a valid boolean value",
                           key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -217,7 +237,8 @@ xlator_option_validate_xlator (xlator_t *xl, const char *key, const char *value,
                 snprintf (errstr, 256,
                           "option %s %s: '%s' is not a valid volume name",
                           key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -316,7 +337,8 @@ xlator_option_validate_str (xlator_t *xl, const char *key, const char *value,
 out:
         if (ret) {
                 set_error_str (errstr, sizeof (errstr), opt, key, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 if (op_errstr)
                         *op_errstr = gf_strdup (errstr);
         }
@@ -337,7 +359,8 @@ xlator_option_validate_percent (xlator_t *xl, const char *key, const char *value
                 snprintf (errstr, 256,
                           "invalid percent format \"%s\" in \"option %s\"",
                           value, key);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -345,7 +368,8 @@ xlator_option_validate_percent (xlator_t *xl, const char *key, const char *value
                 snprintf (errstr, 256,
                           "'%lf' in 'option %s %s' is out of range [0 - 100]",
                           percent, key, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -356,36 +380,75 @@ out:
         return ret;
 }
 
+static int
+xlator_option_validate_fractional_value (const char *value)
+{
+        const char *s   = NULL;
+        int        ret  = 0;
+
+        s = strchr (value, '.');
+        if (s) {
+                for (s = s+1; *s != '\0'; s++) {
+                        if (*s != '0') {
+                                return -1;
+                        }
+                }
+        }
+
+        return ret;
+}
+
 static int
 xlator_option_validate_percent_or_sizet (xlator_t *xl, const char *key,
                                          const char *value,
                                          volume_option_t *opt, char **op_errstr)
 {
-        int          ret = -1;
-        char         errstr[256];
-        uint64_t     size = 0;
+        int               ret = -1;
+        char              errstr[256];
+        double            size = 0;
 	gf_boolean_t is_percent = _gf_false;
 
 	if (gf_string2percent_or_bytesize (value, &size, &is_percent) == 0) {
 		if (is_percent) {
+                        if ((size < 0.0) || (size > 100.0)) {
+                                snprintf (errstr, sizeof (errstr),
+                                          "'%lf' in 'option %s %s' is out"
+                                          " of range [0 - 100]", size, key,
+                                          value);
+                                gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                        LG_MSG_OUT_OF_RANGE, "%s", errstr);
+                                goto out;
+                        }
 			ret = 0;
 			goto out;
 		}
+
+                /*Input value of size(in byte) should not be fractional*/
+                ret = xlator_option_validate_fractional_value (value);
+                if (ret) {
+                        snprintf (errstr, sizeof (errstr), "'%lf' in 'option %s"
+                                  " %s' should not be fractional value. Use "
+                                  "valid unsigned integer value.", size, key,
+                                  value);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_INVALID_ENTRY, "%s", errstr);
+                        goto out;
+                }
+
 		/* Check the range */
 		if ((opt->min == 0) && (opt->max == 0)) {
-			gf_log (xl->name, GF_LOG_TRACE,
-				"no range check required for "
-				"'option %s %s'",
-				key, value);
+			gf_msg_trace (xl->name, 0, "no range check required "
+                                      "for 'option %s %s'", key, value);
 			ret = 0;
 			goto out;
 		}
 		if ((size < opt->min) || (size > opt->max)) {
 			snprintf (errstr, 256,
-				  "'%"PRId64"' in 'option %s %s'"
+				  "'%lf' in 'option %s %s'"
 				  " is out of range [%.0f - %.0f]",
 				  size, key, value, opt->min, opt->max);
-			gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+			gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE,
+                                "%s", errstr);
 			goto out;
 		}
 		ret = 0;
@@ -397,7 +460,7 @@ xlator_option_validate_percent_or_sizet (xlator_t *xl, const char *key,
 	snprintf (errstr, 256,
 		  "invalid number format \"%s\" in \"option %s\"",
 		  value, key);
-	gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+	gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
 
 
 out:
@@ -421,15 +484,14 @@ xlator_option_validate_time (xlator_t *xl, const char *key, const char *value,
                           "invalid time format \"%s\" in "
                           "\"option %s\"",
                           value, key);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
         if ((opt->min == 0) && (opt->max == 0)) {
-                gf_log (xl->name, GF_LOG_TRACE,
-                        "no range check required for "
-                        "'option %s %s'",
-                        key, value);
+                gf_msg_trace (xl->name, 0, "no range check required for "
+                              "'option %s %s'", key, value);
                 ret = 0;
                 goto out;
         }
@@ -440,7 +502,8 @@ xlator_option_validate_time (xlator_t *xl, const char *key, const char *value,
                           "out of range [%.0f - %.0f]",
                           input_time, key, value,
                           opt->min, opt->max);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -465,35 +528,37 @@ xlator_option_validate_double (xlator_t *xl, const char *key, const char *value,
                 snprintf (errstr, 256,
                           "invalid number format \"%s\" in option \"%s\"",
                           value, key);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 goto out;
         }
 
         if ((opt->min == 0) && (opt->max == 0) &&
             (opt->validate == GF_OPT_VALIDATE_BOTH)) {
-                gf_log (xl->name, GF_LOG_TRACE,
-                        "no range check required for 'option %s %s'",
-                        key, value);
+                gf_msg_trace (xl->name, 0, "no range check required for "
+                              "'option %s %s'", key, value);
                 ret = 0;
                 goto out;
         }
 
-        if ((opt->validate == GF_OPT_VALIDATE_MIN)) {
+        if (opt->validate == GF_OPT_VALIDATE_MIN) {
                 if (input < opt->min) {
                         snprintf (errstr, 256,
                                   "'%f' in 'option %s %s' is smaller than "
                                   "minimum value '%f'", input, key,
                                   value, opt->min);
-                        gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_INVALID_ENTRY, "%s", errstr);
                         goto out;
                 }
-        } else if ((opt->validate == GF_OPT_VALIDATE_MAX)) {
-                if ((input > opt->max)) {
+        } else if (opt->validate == GF_OPT_VALIDATE_MAX) {
+                if (input > opt->max) {
                         snprintf (errstr, 256,
                                   "'%f' in 'option %s %s' is greater than "
                                   "maximum value '%f'", input, key,
                                   value, opt->max);
-                        gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                        gf_msg (xl->name, GF_LOG_ERROR, 0,
+                                LG_MSG_INVALID_ENTRY, "%s", errstr);
                         goto out;
                 }
         } else if ((input < opt->min) || (input > opt->max)) {
@@ -501,7 +566,8 @@ xlator_option_validate_double (xlator_t *xl, const char *key, const char *value,
                           "'%f' in 'option %s %s' is out of range "
                           "[%f - %f]",
                           input, key, value, opt->min, opt->max);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
+                        errstr);
                 goto out;
         }
 
@@ -525,7 +591,8 @@ xlator_option_validate_addr (xlator_t *xl, const char *key, const char *value,
                           "option %s %s: '%s'  is not a valid internet-address,"
                           " it does not conform to standards.",
                           key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 if (op_errstr)
                         *op_errstr = gf_strdup (errstr);
         }
@@ -565,7 +632,8 @@ out:
         if (ret) {
                 snprintf (errstr, sizeof (errstr), "option %s %s: '%s' is not "
                 "a valid internet-address-list", key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 if (op_errstr)
                         *op_errstr = gf_strdup (errstr);
         }
@@ -604,7 +672,8 @@ out:
         if (ret) {
                 snprintf (errstr, sizeof (errstr), "option %s %s: '%s' is not "
                 "a valid mount-auth-address", key, value, value);
-                gf_log (xl->name, GF_LOG_ERROR, "%s", errstr);
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
+                        errstr);
                 if (op_errstr)
                         *op_errstr = gf_strdup (errstr);
         }
@@ -617,12 +686,12 @@ out:
 static int
 gf_validate_size (const char *sizestr, volume_option_t *opt)
 {
-        uint64_t                value = 0;
+        size_t                value = 0;
         int                     ret = 0;
 
         GF_ASSERT (opt);
 
-        if (gf_string2bytesize (sizestr, &value) != 0 ||
+        if (gf_string2bytesize_size (sizestr, &value) != 0 ||
             value < opt->min ||
             value % 512) {
                 ret = -1;
@@ -630,7 +699,7 @@ gf_validate_size (const char *sizestr, volume_option_t *opt)
         }
 
  out:
-        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug (THIS->name, 0, "Returning %d", ret);
         return ret;
 }
 
@@ -674,9 +743,9 @@ validate_list_elements (const char *string, volume_option_t *opt,
                 if (!key ||
                     (key_validator && key_validator(key))) {
                         ret = -1;
-                        gf_log (THIS->name, GF_LOG_WARNING,
-                                "invalid list '%s', key '%s' not valid.",
-                                string, key);
+                        gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                                LG_MSG_INVALID_ENTRY, "invalid list '%s', key "
+                                "'%s' not valid.", string, key);
                         goto out;
                 }
 
@@ -684,9 +753,9 @@ validate_list_elements (const char *string, volume_option_t *opt,
                 if (!value ||
                     (value_validator && value_validator(value, opt))) {
                         ret = -1;
-                        gf_log (THIS->name, GF_LOG_WARNING,
-                                "invalid list '%s', value '%s' not valid.",
-                                string, key);
+                        gf_msg (THIS->name, GF_LOG_WARNING, 0,
+                                LG_MSG_INVALID_ENTRY, "invalid list '%s', "
+                                "value '%s' not valid.", string, key);
                         goto out;
                 }
 
@@ -696,7 +765,7 @@ validate_list_elements (const char *string, volume_option_t *opt,
 
  out:
         GF_FREE (dup_string);
-        gf_log (THIS->name, GF_LOG_DEBUG, "Returning %d", ret);
+        gf_msg_debug (THIS->name, 0, "Returning %d", ret);
         return ret;
 }
 
@@ -787,8 +856,8 @@ xlator_option_validate (xlator_t *xl, char *key, char *value,
                 [GF_OPTION_TYPE_MAX]         = NULL,
         };
 
-        if (opt->type < 0 || opt->type >= GF_OPTION_TYPE_MAX) {
-                gf_log (xl->name, GF_LOG_ERROR,
+        if (opt->type > GF_OPTION_TYPE_MAX) {
+                gf_msg (xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY,
                         "unknown option type '%d'", opt->type);
                 goto out;
         }
@@ -875,17 +944,17 @@ xl_opt_validate (dict_t *dict, char *key, data_t *value, void *data)
 
         ret = xlator_option_validate (xl, key, value->data, opt, &errstr);
         if (ret)
-                gf_log (xl->name, GF_LOG_WARNING, "validate of %s returned %d",
-                        key, ret);
+                gf_msg (xl->name, GF_LOG_WARNING, 0, LG_MSG_VALIDATE_RETURNS,
+                        "validate of %s returned %d", key, ret);
 
         if (errstr)
                 /* possible small leak of previously set stub->errstr */
                 stub->errstr = errstr;
 
         if (fnmatch (opt->key[0], key, FNM_NOESCAPE) != 0) {
-                gf_log (xl->name, GF_LOG_WARNING, "option '%s' is deprecated, "
-                        "preferred is '%s', continuing with correction",
-                        key, opt->key[0]);
+                gf_msg (xl->name, GF_LOG_WARNING, 0, LG_MSG_INVALID_ENTRY,
+                        "option '%s' is deprecated, preferred is '%s', "
+                        "continuing with correction", key, opt->key[0]);
                 dict_set (dict, opt->key[0], value);
                 dict_del (dict, key);
         }
@@ -927,7 +996,7 @@ xlator_options_validate (xlator_t *xl, dict_t *options, char **op_errstr)
 
 
         if (!xl) {
-                gf_log (THIS->name, GF_LOG_DEBUG, "'this' not a valid ptr");
+                gf_msg_debug (THIS->name, 0, "'this' not a valid ptr");
                 ret = -1;
                 goto out;
         }
@@ -957,7 +1026,9 @@ xlator_validate_rec (xlator_t *xlator, char **op_errstr)
 
         while (trav) {
                 if (xlator_validate_rec (trav->xlator, op_errstr)) {
-                        gf_log ("xlator", GF_LOG_WARNING, "validate_rec failed");
+                        gf_msg ("xlator", GF_LOG_WARNING, 0,
+                                LG_MSG_VALIDATE_REC_FAILED, "validate_rec "
+                                "failed");
                         goto out;
                 }
 
@@ -965,13 +1036,13 @@ xlator_validate_rec (xlator_t *xlator, char **op_errstr)
         }
 
         if (xlator_dynload (xlator))
-                gf_log (xlator->name, GF_LOG_DEBUG, "Did not load the symbols");
+                gf_msg_debug (xlator->name, 0, "Did not load the symbols");
 
         old_THIS = THIS;
         THIS = xlator;
 
         /* Need this here, as this graph has not yet called init() */
-        if (!xlator->mem_acct.num_types) {
+        if (!xlator->mem_acct) {
                 if (!xlator->mem_acct_init)
                         xlator->mem_acct_init = default_mem_acct_init;
                 xlator->mem_acct_init (xlator);
@@ -981,11 +1052,12 @@ xlator_validate_rec (xlator_t *xlator, char **op_errstr)
         THIS = old_THIS;
 
         if (ret) {
-                gf_log (xlator->name, GF_LOG_INFO, "%s", *op_errstr);
+                gf_msg (xlator->name, GF_LOG_INFO, 0, LG_MSG_INVALID_ENTRY,
+                        "%s", *op_errstr);
                 goto out;
         }
 
-        gf_log (xlator->name, GF_LOG_DEBUG, "Validated options");
+        gf_msg_debug (xlator->name, 0, "Validated options");
 
         ret = 0;
 out:
@@ -1028,7 +1100,7 @@ xlator_reconfigure_rec (xlator_t *old_xl, xlator_t *new_xl)
                 if (ret)
                         goto out;
 
-                gf_log (trav1->xlator->name, GF_LOG_DEBUG, "reconfigured");
+                gf_msg_debug (trav1->xlator->name, 0, "reconfigured");
 
                 trav1 = trav1->next;
                 trav2 = trav2->next;
@@ -1045,7 +1117,7 @@ xlator_reconfigure_rec (xlator_t *old_xl, xlator_t *new_xl)
                 if (ret)
                         goto out;
         } else {
-                gf_log (old_xl->name, GF_LOG_DEBUG, "No reconfigure() found");
+                gf_msg_debug (old_xl->name, 0, "No reconfigure() found");
         }
 
         ret = 0;
@@ -1120,18 +1192,18 @@ pc_or_size (char *in, double *out)
 {
         double  pc = 0;
         int       ret = 0;
-        uint64_t  size = 0;
+        size_t  size = 0;
 
         if (gf_string2percent (in, &pc) == 0) {
                 if (pc > 100.0) {
-                        ret = gf_string2bytesize (in, &size);
+                        ret = gf_string2bytesize_size (in, &size);
                         if (!ret)
                                 *out = size;
                 } else {
                         *out = pc;
                 }
         } else {
-                ret = gf_string2bytesize (in, &size);
+                ret = gf_string2bytesize_size (in, &size);
                 if (!ret)
                         *out = size;
         }
@@ -1143,7 +1215,8 @@ DEFINE_INIT_OPT(uint64_t, uint64, gf_string2uint64);
 DEFINE_INIT_OPT(int64_t, int64, gf_string2int64);
 DEFINE_INIT_OPT(uint32_t, uint32, gf_string2uint32);
 DEFINE_INIT_OPT(int32_t, int32, gf_string2int32);
-DEFINE_INIT_OPT(uint64_t, size, gf_string2bytesize);
+DEFINE_INIT_OPT(size_t, size, gf_string2bytesize_size);
+DEFINE_INIT_OPT(uint64_t, size_uint64, gf_string2bytesize_uint64);
 DEFINE_INIT_OPT(double, percent, gf_string2percent);
 DEFINE_INIT_OPT(double, percent_or_size, pc_or_size);
 DEFINE_INIT_OPT(gf_boolean_t, bool, gf_string2boolean);
@@ -1158,7 +1231,8 @@ DEFINE_RECONF_OPT(uint64_t, uint64, gf_string2uint64);
 DEFINE_RECONF_OPT(int64_t, int64, gf_string2int64);
 DEFINE_RECONF_OPT(uint32_t, uint32, gf_string2uint32);
 DEFINE_RECONF_OPT(int32_t, int32, gf_string2int32);
-DEFINE_RECONF_OPT(uint64_t, size, gf_string2bytesize);
+DEFINE_RECONF_OPT(size_t, size, gf_string2bytesize_size);
+DEFINE_RECONF_OPT(uint64_t, size_uint64, gf_string2bytesize_uint64);
 DEFINE_RECONF_OPT(double, percent, gf_string2percent);
 DEFINE_RECONF_OPT(double, percent_or_size, pc_or_size);
 DEFINE_RECONF_OPT(gf_boolean_t, bool, gf_string2boolean);
