@@ -342,8 +342,7 @@ synctask_yield (struct synctask *task)
         }
         if (swapcontext (&task->ctx, &task->proc->sched) < 0) {
                 gf_msg ("syncop", GF_LOG_ERROR, errno,
-                        LG_MSG_SWAPCONTEXT_FAILED, "swapcontext failed (%s)",
-                        strerror (errno));
+                        LG_MSG_SWAPCONTEXT_FAILED, "swapcontext failed");
         }
 
         THIS = oldTHIS;
@@ -495,8 +494,7 @@ synctask_create (struct syncenv *env, size_t stacksize, synctask_fn_t fn,
 
         if (getcontext (&newtask->ctx) < 0) {
                 gf_msg ("syncop", GF_LOG_ERROR, errno,
-                        LG_MSG_GETCONTEXT_FAILED, "getcontext failed (%s)",
-                        strerror (errno));
+                        LG_MSG_GETCONTEXT_FAILED, "getcontext failed");
                 goto err;
         }
 
