@@ -19,6 +19,7 @@
 #include <sys/types.h>
 #include <utime.h>
 #include <sys/time.h>
+#include <fcntl.h>
 
 int
 sys_lstat (const char *path, struct stat *buf)
@@ -41,12 +42,58 @@ sys_fstat (int fd, struct stat *buf)
 }
 
 
+int
+sys_fstatat(int dirfd, const char *pathname, struct stat *buf, int flags)
+{
+#ifdef GF_DARWIN_HOST_OS
+        if (fchdir(dirfd) < 0)
+                return -1;
+        if(flags & AT_SYMLINK_NOFOLLOW)
+                return lstat(pathname, buf);
+        else
+                return stat(pathname, buf);
+#else
+        return fstatat (dirfd, pathname, buf, flags);
+#endif
+}
+
+
+int
+sys_openat(int dirfd, const char *pathname, int flags, ...)
+{
+        mode_t mode = 0;
+        if (flags & O_CREAT) {
+                va_list ap;
+                va_start(ap, flags);
+                mode = va_arg(ap, int);
+                va_end(ap);
+        }
+
+#ifdef GF_DARWIN_HOST_OS
+        if (fchdir(dirfd) < 0)
+                return -1;
+        return open (pathname, flags, mode);
+#else
+        return openat (dirfd, pathname, flags, mode);
+#endif
+}
+
 DIR *
 sys_opendir (const char *name)
 {
         return opendir (name);
 }
 
+int sys_mkdirat(int dirfd, const char *pathname, mode_t mode)
+{
+#ifdef GF_DARWIN_HOST_OS
+        if(fchdir(dirfd) < 0)
+                return -1;
+        return mkdir(pathname, mode);
+#else
+        return mkdirat (dirfd, pathname, mode);
+#endif
+}
 
 struct dirent *
 sys_readdir (DIR *dir)
@@ -121,7 +168,13 @@ int
 sys_link (const char *oldpath, const char *newpath)
 {
 #ifdef HAVE_LINKAT
-        /* see HAVE_LINKAT in xlators/storage/posix/src/posix.c */
+	/*
+	 * On most systems (Linux being the notable exception), link(2)
+	 * first resolves symlinks. If the target is a directory or
+	 * is nonexistent, it will fail. linkat(2) operates on the
+	 * symlink instead of its target when the AT_SYMLINK_FOLLOW
+	 * flag is not supplied.
+	 */
         return linkat (AT_FDCWD, oldpath, AT_FDCWD, newpath, 0);
 #else
         return link (oldpath, newpath);
@@ -256,13 +309,45 @@ sys_fsync (int fd)
 int
 sys_fdatasync (int fd)
 {
-#ifdef HAVE_FDATASYNC
-        return fdatasync (fd);
+#ifdef GF_DARWIN_HOST_OS
+        return fcntl (fd, F_FULLFSYNC);
+#elif __FreeBSD__
+	return fsync (fd);
 #else
-        return 0;
+        return fdatasync (fd);
 #endif
 }
 
+void
+gf_add_prefix(const char *ns, const char *key, char **newkey)
+{
+        /* if we dont have any namespace, append USER NS */
+        if (strncmp(key, XATTR_USER_PREFIX,     XATTR_USER_PREFIX_LEN) &&
+            strncmp(key, XATTR_TRUSTED_PREFIX,  XATTR_TRUSTED_PREFIX_LEN) &&
+            strncmp(key, XATTR_SECURITY_PREFIX, XATTR_TRUSTED_PREFIX_LEN) &&
+            strncmp(key, XATTR_SYSTEM_PREFIX,   XATTR_SYSTEM_PREFIX_LEN)) {
+                int ns_length =  strlen(ns);
+                *newkey = GF_MALLOC(ns_length + strlen(key) + 10,
+                                    gf_common_mt_char);
+                strcpy(*newkey, ns);
+                strcat(*newkey, key);
+        } else {
+                *newkey = gf_strdup(key);
+        }
+}
+
+void
+gf_remove_prefix(const char *ns, const char *key, char **newkey)
+{
+        int ns_length =  strlen(ns);
+        if (strncmp(key, ns, ns_length) == 0) {
+                *newkey = GF_MALLOC(-ns_length + strlen(key) + 10,
+                                    gf_common_mt_char);
+                strcpy(*newkey, key + ns_length);
+        } else {
+                *newkey = gf_strdup(key);
+        }
+}
 
 int
 sys_lsetxattr (const char *path, const char *name, const void *value,
@@ -283,8 +368,11 @@ sys_lsetxattr (const char *path, const char *name, const void *value,
 #endif
 
 #ifdef GF_DARWIN_HOST_OS
+        /* OS X clients will carry other flags, which will be used on a
+           OS X host, but masked out on others. GF assume NOFOLLOW on Linux,
+           enforcing  */
         return setxattr (path, name, value, size, 0,
-                         flags|XATTR_NOFOLLOW);
+                         (flags & ~XATTR_NOSECURITY) | XATTR_NOFOLLOW);
 #endif
 
 }
@@ -309,10 +397,8 @@ sys_llistxattr (const char *path, char *list, size_t size)
 #ifdef GF_DARWIN_HOST_OS
         return listxattr (path, list, size, XATTR_NOFOLLOW);
 #endif
-
 }
 
-
 ssize_t
 sys_lgetxattr (const char *path, const char *name, void *value, size_t size)
 {
@@ -331,7 +417,7 @@ sys_lgetxattr (const char *path, const char *name, void *value, size_t size)
 #endif
 
 #ifdef GF_DARWIN_HOST_OS
-        return getxattr (path, name, value, size, 0, XATTR_NOFOLLOW);
+         return getxattr (path, name, value, size, 0, XATTR_NOFOLLOW);
 #endif
 
 }
@@ -360,7 +446,6 @@ sys_fgetxattr (int filedes, const char *name, void *value, size_t size)
 
 }
 
-
 int
 sys_fremovexattr (int filedes, const char *name)
 {
@@ -369,11 +454,8 @@ sys_fremovexattr (int filedes, const char *name)
         return fremovexattr (filedes, name);
 #endif
 
-        errno = ENOSYS;
-        return -1;
-#if 0 /* TODO: to port to other OSes, fill in each of below */
 #ifdef GF_BSD_HOST_OS
-        return extattr_remove_fd (filedes, EXTATTR_NAMESPACE_USER, name);
+        return extattr_delete_fd (filedes, EXTATTR_NAMESPACE_USER, name);
 #endif
 
 #ifdef GF_SOLARIS_HOST_OS
@@ -383,7 +465,6 @@ sys_fremovexattr (int filedes, const char *name)
 #ifdef GF_DARWIN_HOST_OS
         return fremovexattr (filedes, name, 0);
 #endif
-#endif
 }
 
 
@@ -406,7 +487,8 @@ sys_fsetxattr (int filedes, const char *name, const void *value,
 #endif
 
 #ifdef GF_DARWIN_HOST_OS
-        return fsetxattr (filedes, name, value, size, 0, flags);
+        return fsetxattr (filedes, name, value, size, 0,
+                          flags & ~XATTR_NOSECURITY);
 #endif
 
 }
@@ -469,20 +551,59 @@ int
 sys_fallocate(int fd, int mode, off_t offset, off_t len)
 {
 #ifdef HAVE_FALLOCATE
-	return fallocate(fd, mode, offset, len);
+        return fallocate(fd, mode, offset, len);
 #endif
 
 #ifdef HAVE_POSIX_FALLOCATE
-	if (mode) {
-		/* keep size not supported */
-		errno = EOPNOTSUPP;
-		return -1;
-	}
+        if (mode) {
+                /* keep size not supported */
+                errno = EOPNOTSUPP;
+                return -1;
+        }
 
-	return posix_fallocate(fd, offset, len);
+        return posix_fallocate(fd, offset, len);
 #endif
 
-	errno = ENOSYS;
-	return -1;
-}
+#if defined(F_ALLOCATECONFIG) && defined(GF_DARWIN_HOST_OS)
+        /* C conversion from C++ implementation for OSX by Mozilla Foundation */
+        if (mode) {
+                /* keep size not supported */
+                errno = EOPNOTSUPP;
+                return -1;
+        }
+        /*
+         *   The F_PREALLOCATE command operates on the following structure:
+         *
+         *    typedef struct fstore {
+         *    u_int32_t fst_flags;      // IN: flags word
+         *    int       fst_posmode;    // IN: indicates offset field
+         *    off_t     fst_offset;     // IN: start of the region
+         *    off_t     fst_length;     // IN: size of the region
+         *    off_t     fst_bytesalloc; // OUT: number of bytes allocated
+         *    } fstore_t;
+         *
+         * The flags (fst_flags) for the F_PREALLOCATE command are as follows:
+         *    F_ALLOCATECONTIG   Allocate contiguous space.
+         *    F_ALLOCATEALL      Allocate all requested space or no space at all.
+         *
+         * The position modes (fst_posmode) for the F_PREALLOCATE command
+         * indicate how to use the offset field.  The modes are as follows:
+         *    F_PEOFPOSMODE   Allocate from the physical end of file.
+         *    F_VOLPOSMODE    Allocate from the volume offset.
+         *
+         */
 
+        int ret;
+        fstore_t store = {F_ALLOCATECONTIG, F_PEOFPOSMODE, offset, len, 0};
+        ret = fcntl (fd, F_PREALLOCATE, &store);
+        if (ret == -1) {
+                store.fst_flags = F_ALLOCATEALL;
+                ret = fcntl (fd, F_PREALLOCATE, &store);
+        }
+        if (ret == -1)
+                return ret;
+        return ftruncate (fd, offset + len);
+#endif
+        errno = ENOSYS;
+        return -1;
+}
