@@ -18,6 +18,7 @@
 #include "common-utils.h"
 #include "globals.h"
 #include "timespec.h"
+#include "libglusterfs-messages.h"
 
 gf_timer_t *
 gf_timer_call_after (glusterfs_ctx_t *ctx,
@@ -32,14 +33,27 @@ gf_timer_call_after (glusterfs_ctx_t *ctx,
 
         if (ctx == NULL)
         {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
+                return NULL;
+        }
+
+        /* ctx and its fields are not accessed inside mutex!?
+         * TODO: Even with this there is a possiblity of race
+         * when cleanup_started is set after checking for it
+         */
+        if (ctx->cleanup_started) {
+                gf_msg_callingfn ("timer", GF_LOG_INFO, 0,
+                                  LG_MSG_CTX_CLEANUP_STARTED, "ctx cleanup "
+                                  "started");
                 return NULL;
         }
 
         reg = gf_timer_registry_init (ctx);
 
         if (!reg) {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "!reg");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, 0,
+                                  LG_MSG_TIMER_REGISTER_ERROR, "!reg");
                 return NULL;
         }
 
@@ -76,7 +90,8 @@ gf_timer_call_stale (gf_timer_registry_t *reg,
 {
         if (reg == NULL || event == NULL)
         {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return 0;
         }
 
@@ -98,13 +113,15 @@ gf_timer_call_cancel (glusterfs_ctx_t *ctx,
 
         if (ctx == NULL || event == NULL)
         {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return 0;
         }
 
         reg = gf_timer_registry_init (ctx);
         if (!reg) {
-                gf_log ("timer", GF_LOG_ERROR, "!reg");
+                gf_msg ("timer", GF_LOG_ERROR, 0, LG_MSG_INIT_TIMER_FAILED,
+                        "!reg");
                 GF_FREE (event);
                 return 0;
         }
@@ -120,28 +137,37 @@ gf_timer_call_cancel (glusterfs_ctx_t *ctx,
         return 0;
 }
 
+static inline void __delete_entry (gf_timer_t *event) {
+        event->next->prev = event->prev;
+        event->prev->next = event->next;
+        GF_FREE (event);
+}
+
 void *
 gf_timer_proc (void *ctx)
 {
         gf_timer_registry_t *reg = NULL;
         const struct timespec sleepts = {.tv_sec = 1, .tv_nsec = 0, };
+        gf_timer_t *event = NULL;
+        xlator_t   *old_THIS = NULL;
 
         if (ctx == NULL)
         {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
 
         reg = gf_timer_registry_init (ctx);
         if (!reg) {
-                gf_log ("timer", GF_LOG_ERROR, "!reg");
+                gf_msg ("timer", GF_LOG_ERROR, 0, LG_MSG_INIT_TIMER_FAILED,
+                        "!reg");
                 return NULL;
         }
 
         while (!reg->fin) {
                 uint64_t now;
                 struct timespec now_ts;
-                gf_timer_t *event = NULL;
 
                 timespec_now (&now_ts);
                 now = TS (now_ts);
@@ -159,11 +185,20 @@ gf_timer_proc (void *ctx)
                                 }
                         }
                         pthread_mutex_unlock (&reg->lock);
-                        if (event->xl)
-                                THIS = event->xl;
-                        if (need_cbk)
-                                event->callbk  (event->data);
-
+                        if (need_cbk) {
+                                old_THIS = NULL;
+                                if (event->xl) {
+                                        old_THIS = THIS;
+                                        THIS = event->xl;
+                                }
+                                event->callbk (event->data);
+                                /*This callbk above would have freed the event
+                                 * by calling timer_cancel, don't ever touch it
+                                 * again*/
+                                if (old_THIS) {
+                                        THIS = old_THIS;
+                                }
+                        }
                         else
                                 break;
                 }
@@ -172,12 +207,19 @@ gf_timer_proc (void *ctx)
 
         pthread_mutex_lock (&reg->lock);
         {
+                /* Do not call gf_timer_call_cancel(),
+                 * it will lead to deadlock
+                 */
                 while (reg->active.next != &reg->active) {
-                        gf_timer_call_cancel (ctx, reg->active.next);
+                        event = reg->active.next;
+                        /* cannot call list_del as the event doesnt have
+                         * list_head*/
+                        __delete_entry (event);
                 }
 
                 while (reg->stale.next != &reg->stale) {
-                        gf_timer_call_cancel (ctx, reg->stale.next);
+                        event = reg->stale.next;
+                        __delete_entry (event);
                 }
         }
         pthread_mutex_unlock (&reg->lock);
@@ -191,7 +233,8 @@ gf_timer_registry_t *
 gf_timer_registry_init (glusterfs_ctx_t *ctx)
 {
         if (ctx == NULL) {
-                gf_log_callingfn ("timer", GF_LOG_ERROR, "invalid argument");
+                gf_msg_callingfn ("timer", GF_LOG_ERROR, EINVAL,
+                                  LG_MSG_INVALID_ARG, "invalid argument");
                 return NULL;
         }
 
@@ -215,3 +258,18 @@ gf_timer_registry_init (glusterfs_ctx_t *ctx)
 out:
         return ctx->timer;
 }
+
+void
+gf_timer_registry_destroy (glusterfs_ctx_t *ctx)
+{
+        pthread_t thr_id;
+        gf_timer_registry_t *reg = NULL;
+
+        if (ctx == NULL)
+                return;
+
+        reg = ctx->timer;
+        thr_id = reg->th;
+        reg->fin = 1;
+        pthread_join (thr_id, NULL);
+}
