@@ -59,6 +59,7 @@ enum gf_fop_procnum {
 	GFS3_OP_FALLOCATE,
 	GFS3_OP_DISCARD,
         GFS3_OP_ZEROFILL,
+        GFS3_OP_IPC,
         GFS3_OP_MAXVALUE,
 } ;
 
@@ -70,6 +71,7 @@ enum gf_handshake_procnum {
         GF_HNDSK_SET_LK_VER,
         GF_HNDSK_EVENT_NOTIFY,
         GF_HNDSK_GET_VOLUME_INFO,
+        GF_HNDSK_GET_SNAPSHOT_INFO,
         GF_HNDSK_MAXVALUE,
 };
 
@@ -108,7 +110,10 @@ enum gf_probe_resp {
         GF_PROBE_SAME_UUID,
         GF_PROBE_UNKNOWN_PEER,
         GF_PROBE_ADD_FAILED,
-        GF_PROBE_QUORUM_NOT_MET
+        GF_PROBE_QUORUM_NOT_MET,
+        GF_PROBE_MISSED_SNAP_CONFLICT,
+        GF_PROBE_SNAP_CONFLICT,
+        GF_PROBE_FRIEND_DETACHING,
 };
 
 enum gf_deprobe_resp {
@@ -118,6 +123,7 @@ enum gf_deprobe_resp {
         GF_DEPROBE_BRICK_EXIST,
         GF_DEPROBE_FRIEND_DOWN,
         GF_DEPROBE_QUORUM_NOT_MET,
+        GF_DEPROBE_FRIEND_DETACHING,
 };
 
 enum gf_cbk_procnum {
@@ -125,6 +131,8 @@ enum gf_cbk_procnum {
         GF_CBK_FETCHSPEC,
         GF_CBK_INO_FLUSH,
         GF_CBK_EVENT_NOTIFY,
+        GF_CBK_GET_SNAPS,
+        GF_CBK_CACHE_INVALIDATION,
         GF_CBK_MAXVALUE,
 };
 
@@ -168,6 +176,14 @@ enum gluster_cli_procnum {
         GLUSTER_CLI_UUID_GET,
         GLUSTER_CLI_COPY_FILE,
         GLUSTER_CLI_SYS_EXEC,
+        GLUSTER_CLI_SNAP,
+        GLUSTER_CLI_BARRIER_VOLUME,
+        GLUSTER_CLI_GET_VOL_OPT,
+        GLUSTER_CLI_GANESHA,
+        GLUSTER_CLI_BITROT,
+        GLUSTER_CLI_ATTACH_TIER,
+        GLUSTER_CLI_DETACH_TIER,
+        GLUSTER_CLI_TIER,
         GLUSTER_CLI_MAXVALUE,
 };
 
@@ -199,6 +215,8 @@ enum glusterd_brick_procnum {
         GLUSTERD_BRICK_XLATOR_DEFRAG,
         GLUSTERD_NODE_PROFILE,
         GLUSTERD_NODE_STATUS,
+        GLUSTERD_VOLUME_BARRIER_OP,
+        GLUSTERD_BRICK_BARRIER,
         GLUSTERD_BRICK_MAXVALUE,
 };
 
@@ -210,31 +228,56 @@ enum glusterd_mgmt_hndsk_procnum {
 };
 
 typedef enum {
-        GF_AFR_OP_INVALID,
-        GF_AFR_OP_HEAL_INDEX,
-        GF_AFR_OP_HEAL_FULL,
-        GF_AFR_OP_INDEX_SUMMARY,
-        GF_AFR_OP_HEALED_FILES,
-        GF_AFR_OP_HEAL_FAILED_FILES,
-        GF_AFR_OP_SPLIT_BRAIN_FILES,
-        GF_AFR_OP_STATISTICS,
-        GF_AFR_OP_STATISTICS_HEAL_COUNT,
-        GF_AFR_OP_STATISTICS_HEAL_COUNT_PER_REPLICA,
+        GF_SHD_OP_INVALID,
+        GF_SHD_OP_HEAL_INDEX,
+        GF_SHD_OP_HEAL_FULL,
+        GF_SHD_OP_INDEX_SUMMARY,
+        GF_SHD_OP_HEALED_FILES,
+        GF_SHD_OP_HEAL_FAILED_FILES,
+        GF_SHD_OP_SPLIT_BRAIN_FILES,
+        GF_SHD_OP_STATISTICS,
+        GF_SHD_OP_STATISTICS_HEAL_COUNT,
+        GF_SHD_OP_STATISTICS_HEAL_COUNT_PER_REPLICA,
+        GF_SHD_OP_SBRAIN_HEAL_FROM_BIGGER_FILE,
+        GF_SHD_OP_SBRAIN_HEAL_FROM_BRICK,
+        GF_SHD_OP_HEAL_ENABLE,
+        GF_SHD_OP_HEAL_DISABLE,
 } gf_xl_afr_op_t ;
 
 struct gf_gsync_detailed_status_ {
         char node[NAME_MAX];
         char master[NAME_MAX];
         char brick[NAME_MAX];
+        char slave_user[NAME_MAX];
+        char slave[NAME_MAX];
         char slave_node[NAME_MAX];
         char worker_status[NAME_MAX];
-        char checkpoint_status[NAME_MAX];
         char crawl_status[NAME_MAX];
-        char files_syncd[NAME_MAX];
-        char files_remaining[NAME_MAX];
-        char bytes_remaining[NAME_MAX];
-        char purges_remaining[NAME_MAX];
-        char total_files_skipped[NAME_MAX];
+        char last_synced[NAME_MAX];
+        char last_synced_utc[NAME_MAX];
+        char entry[NAME_MAX];
+        char data[NAME_MAX];
+        char meta[NAME_MAX];
+        char failures[NAME_MAX];
+        char checkpoint_time[NAME_MAX];
+        char checkpoint_time_utc[NAME_MAX];
+        char checkpoint_completed[NAME_MAX];
+        char checkpoint_completion_time[NAME_MAX];
+        char checkpoint_completion_time_utc[NAME_MAX];
+        char brick_host_uuid[NAME_MAX];
+        char slavekey[NAME_MAX];
+        char session_slave[NAME_MAX];
+};
+
+enum glusterd_mgmt_v3_procnum {
+        GLUSTERD_MGMT_V3_NULL,    /* 0 */
+        GLUSTERD_MGMT_V3_LOCK,
+        GLUSTERD_MGMT_V3_PRE_VALIDATE,
+        GLUSTERD_MGMT_V3_BRICK_OP,
+        GLUSTERD_MGMT_V3_COMMIT,
+        GLUSTERD_MGMT_V3_POST_VALIDATE,
+        GLUSTERD_MGMT_V3_UNLOCK,
+        GLUSTERD_MGMT_V3_MAXVALUE,
 };
 
 typedef struct gf_gsync_detailed_status_ gf_gsync_status_t;
@@ -246,6 +289,12 @@ enum gf_get_volume_info_type {
 
 typedef enum gf_get_volume_info_type gf_get_volume_info_type;
 
+
+enum gf_get_snapshot_info_type {
+        GF_GET_SNAPSHOT_LIST,
+};
+typedef enum gf_get_snapshot_info_type gf_get_snapshot_info_type;
+
 #define GLUSTER_HNDSK_PROGRAM    14398633 /* Completely random */
 #define GLUSTER_HNDSK_VERSION    2   /* 0.0.2 */
 
@@ -276,8 +325,13 @@ typedef enum gf_get_volume_info_type gf_get_volume_info_type;
 #define GD_BRICK_PROGRAM         4867634 /*Completely random*/
 #define GD_BRICK_VERSION         2
 
+/* Third version */
+#define GD_MGMT_V3_VERSION       3
+
 /* OP-VERSION handshake */
 #define GD_MGMT_HNDSK_PROGRAM    1239873 /* Completely random */
 #define GD_MGMT_HNDSK_VERSION    1
 
+#define GD_VOLUME_NAME_MAX 256 /* Maximum size of volume name */
+
 #endif /* !_PROTOCOL_COMMON_H */
