@@ -500,6 +500,7 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
 {
         struct saved_frames    *saved_frames = NULL;
         struct rpc_clnt         *clnt  = NULL;
+        int                     unref = 0;
 
         if (!conn) {
                 goto out;
@@ -521,12 +522,7 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
 
                 conn->connected = 0;
 
-                if (conn->ping_timer) {
-                        gf_timer_call_cancel (clnt->ctx, conn->ping_timer);
-                        conn->ping_timer = NULL;
-                        conn->ping_started = 0;
-                        rpc_clnt_unref (clnt);
-                }
+                unref = rpc_clnt_remove_ping_timer_locked (clnt);
                 /*reset rpc msgs stats*/
                 conn->pingcnt = 0;
                 conn->msgcnt = 0;
@@ -534,6 +530,8 @@ rpc_clnt_connection_cleanup (rpc_clnt_connection_t *conn)
         pthread_mutex_unlock (&conn->lock);
 
         saved_frames_destroy (saved_frames);
+        if (unref)
+                rpc_clnt_unref (clnt);
 
 out:
         return 0;
@@ -817,6 +815,16 @@ out:
 static void
 rpc_clnt_destroy (struct rpc_clnt *rpc);
 
+#define RPC_THIS_SAVE(xl) do {                                  \
+        old_THIS = THIS ;                                       \
+        if (!old_THIS)                                          \
+                gf_log_callingfn ("rpc", GF_LOG_CRITICAL,       \
+                                  "THIS is not initialised.");  \
+        THIS = xl;                                              \
+} while (0)
+
+#define RPC_THIS_RESTORE        (THIS = old_THIS)
+
 int
 rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
                  rpc_transport_event_t event, void *data, ...)
@@ -828,6 +836,7 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         rpc_transport_pollin_t *pollin      = NULL;
         struct timespec         ts          = {0, };
         void                   *clnt_mydata = NULL;
+        DECLARE_OLD_THIS;
 
         conn = mydata;
         if (conn == NULL) {
@@ -837,6 +846,8 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         if (!clnt)
                 goto out;
 
+        RPC_THIS_SAVE (clnt->owner);
+
         switch (event) {
         case RPC_TRANSPORT_DISCONNECT:
         {
@@ -937,6 +948,7 @@ rpc_clnt_notify (rpc_transport_t *trans, void *mydata,
         }
 
 out:
+        RPC_THIS_RESTORE;
         return ret;
 }
 
@@ -1041,11 +1053,13 @@ out:
 }
 
 struct rpc_clnt *
-rpc_clnt_new (dict_t *options, glusterfs_ctx_t *ctx, char *name,
+rpc_clnt_new (dict_t *options, xlator_t *owner, char *name,
               uint32_t reqpool_size)
 {
         int                    ret  = -1;
         struct rpc_clnt       *rpc  = NULL;
+        glusterfs_ctx_t       *ctx  = owner->ctx;
+
 
         rpc = GF_CALLOC (1, sizeof (*rpc), gf_common_mt_rpcclnt_t);
         if (!rpc) {
@@ -1054,6 +1068,7 @@ rpc_clnt_new (dict_t *options, glusterfs_ctx_t *ctx, char *name,
 
         pthread_mutex_init (&rpc->lock, NULL);
         rpc->ctx = ctx;
+        rpc->owner = owner;
 
         if (!reqpool_size)
                 reqpool_size = RPC_CLNT_DEFAULT_REQUEST_COUNT;
@@ -1714,6 +1729,7 @@ rpc_clnt_disable (struct rpc_clnt *rpc)
 {
         rpc_clnt_connection_t *conn = NULL;
         rpc_transport_t       *trans = NULL;
+        int                    unref = 0;
 
         if (!rpc) {
                 goto out;
@@ -1736,12 +1752,7 @@ rpc_clnt_disable (struct rpc_clnt *rpc)
                 }
                 conn->connected = 0;
 
-                if (conn->ping_timer) {
-                        gf_timer_call_cancel (rpc->ctx, conn->ping_timer);
-                        conn->ping_timer = NULL;
-                        conn->ping_started = 0;
-                        rpc_clnt_unref (rpc);
-                }
+                unref = rpc_clnt_remove_ping_timer_locked (rpc);
                 trans = conn->trans;
                 conn->trans = NULL;
 
@@ -1752,6 +1763,9 @@ rpc_clnt_disable (struct rpc_clnt *rpc)
                 rpc_transport_disconnect (trans);
         }
 
+        if (unref)
+                rpc_clnt_unref (rpc);
+
 out:
         return;
 }
@@ -1761,6 +1775,7 @@ rpc_clnt_disconnect (struct rpc_clnt *rpc)
 {
         rpc_clnt_connection_t *conn  = NULL;
         rpc_transport_t       *trans = NULL;
+        int                    unref = 0;
 
         if (!rpc)
                 goto out;
@@ -1781,11 +1796,7 @@ rpc_clnt_disconnect (struct rpc_clnt *rpc)
                 }
                 conn->connected = 0;
 
-                if (conn->ping_timer) {
-                        gf_timer_call_cancel (rpc->ctx, conn->ping_timer);
-                        conn->ping_timer = NULL;
-                        conn->ping_started = 0;
-                }
+                unref = rpc_clnt_remove_ping_timer_locked (rpc);
                 trans = conn->trans;
         }
         pthread_mutex_unlock (&conn->lock);
@@ -1793,6 +1804,8 @@ rpc_clnt_disconnect (struct rpc_clnt *rpc)
         if (trans) {
                 rpc_transport_disconnect (trans);
         }
+        if (unref)
+                rpc_clnt_unref (rpc);
 
 out:
         return;
@@ -1802,6 +1815,21 @@ out:
 void
 rpc_clnt_reconfig (struct rpc_clnt *rpc, struct rpc_clnt_config *config)
 {
+        if (config->ping_timeout) {
+                if (config->ping_timeout != rpc->conn.ping_timeout)
+                        gf_log (rpc->conn.name, GF_LOG_INFO,
+                                "changing ping timeout to %d (from %d)",
+                                config->ping_timeout,
+                                rpc->conn.ping_timeout);
+
+                pthread_mutex_lock (&rpc->conn.lock);
+                {
+                rpc->conn.ping_timeout = config->ping_timeout;
+                }
+                pthread_mutex_unlock (&rpc->conn.lock);
+
+        }
+
         if (config->rpc_timeout) {
                 if (config->rpc_timeout != rpc->conn.config.rpc_timeout)
                         gf_log (rpc->conn.name, GF_LOG_INFO,
