@@ -125,6 +125,7 @@ struct rpc_clnt_config {
         int    rpc_timeout;
         int    remote_port;
         char * remote_host;
+        int    ping_timeout;
 };
 
 
@@ -144,6 +145,10 @@ struct rpc_clnt_connection {
 	struct timeval           last_sent;
 	struct timeval           last_received;
 	int32_t                  ping_started;
+        char                    *name;
+	int32_t                  ping_timeout;
+        uint64_t                 pingcnt;
+        uint64_t                 msgcnt;
 };
 typedef struct rpc_clnt_connection rpc_clnt_connection_t;
 
@@ -183,10 +188,11 @@ typedef struct rpc_clnt {
         int                   refcount;
         int                   auth_null;
         char                  disabled;
+        xlator_t             *owner;
 } rpc_clnt_t;
 
 
-struct rpc_clnt *rpc_clnt_new (dict_t *options, glusterfs_ctx_t *ctx,
+struct rpc_clnt *rpc_clnt_new (dict_t *options, xlator_t *owner,
                                char *name, uint32_t reqpool_size);
 
 int rpc_clnt_start (struct rpc_clnt *rpc);
@@ -241,6 +247,9 @@ int rpcclnt_cbk_program_register (struct rpc_clnt *svc,
 void
 rpc_clnt_disable (struct rpc_clnt *rpc);
 
+void
+rpc_clnt_disconnect (struct rpc_clnt *rpc);
+
 char
 rpc_clnt_is_disabled (struct rpc_clnt *rpc);
 
