@@ -163,7 +163,7 @@ typedef struct rpc_request_info rpc_request_info_t;
 
 
 struct rpc_transport_pollin {
-        struct iovec vector[2];
+        struct iovec vector[MAX_IOVEC];
         int count;
         char vectored;
         void *private;
@@ -214,6 +214,9 @@ struct rpc_transport {
         struct list_head           list;
         int                        bind_insecure;
         void                      *dl_handle; /* handle of dlopen() */
+        char                      *ssl_name;
+        dict_t                    *clnt_options; /* store options received from
+                                                  * client */
 };
 
 struct rpc_transport_ops {
@@ -240,6 +243,8 @@ struct rpc_transport_ops {
         int32_t (*throttle)       (rpc_transport_t *this, gf_boolean_t onoff);
 };
 
+int32_t
+rpc_transport_count (const char *transport_type);
 
 int32_t
 rpc_transport_listen (rpc_transport_t *this);
@@ -303,7 +308,7 @@ rpc_transport_pollin_destroy (rpc_transport_pollin_t *pollin);
 
 int
 rpc_transport_keepalive_options_set (dict_t *options, int32_t interval,
-                                     int32_t time);
+                                     int32_t time, int32_t timeout);
 
 int
 rpc_transport_unix_options_build (dict_t **options, char *filepath,
