@@ -38,7 +38,7 @@ rpcsvc_auth_add_initer (struct list_head *list, char *idfier,
         }
 
         new->init = init;
-        strcpy (new->name, idfier);
+        strncpy (new->name, idfier, sizeof (new->name) - 1);
         INIT_LIST_HEAD (&new->authlist);
         list_add_tail (&new->authlist, list);
         return 0;
@@ -221,15 +221,28 @@ rpcsvc_set_allow_insecure (rpcsvc_t *svc, dict_t *options)
                         else
                                 svc->allow_insecure = 0;
                 }
+        } else {
+                /* By default set allow-insecure to true */
+                svc->allow_insecure = 1;
+
+                /* setting in options for the sake of functions that look
+                 * configuration params for allow insecure,  eg: gf_auth
+                 */
+                ret = dict_set_str (options, "rpc-auth-allow-insecure", "on");
+                if (ret < 0)
+                        gf_log ("rpc-auth", GF_LOG_DEBUG,
+                                        "dict_set failed for 'allow-insecure'");
         }
 
-        return 0;
+        return ret;
 }
 
 int
 rpcsvc_set_root_squash (rpcsvc_t *svc, dict_t *options)
 {
         int  ret = -1;
+        uid_t anonuid = -1;
+        gid_t anongid = -1;
 
         GF_ASSERT (svc);
         GF_ASSERT (options);
@@ -240,8 +253,21 @@ rpcsvc_set_root_squash (rpcsvc_t *svc, dict_t *options)
         else
                 svc->root_squash = _gf_false;
 
+        ret = dict_get_uint32 (options, "anonuid", &anonuid);
+        if (!ret)
+                svc->anonuid = anonuid;
+        else
+                svc->anonuid = RPC_NOBODY_UID;
+
+        ret = dict_get_uint32 (options, "anongid", &anongid);
+        if (!ret)
+                svc->anongid = anongid;
+        else
+                svc->anongid = RPC_NOBODY_GID;
+
         if (svc->root_squash)
-                gf_log (GF_RPCSVC, GF_LOG_DEBUG, "root squashing enabled ");
+                gf_log (GF_RPCSVC, GF_LOG_DEBUG, "root squashing enabled "
+                        "(uid=%d, gid=%d)", svc->anonuid, svc->anongid);
 
         return 0;
 }
@@ -354,25 +380,36 @@ ret:
 
 
 int
-rpcsvc_auth_request_init (rpcsvc_request_t *req)
+rpcsvc_auth_request_init (rpcsvc_request_t *req, struct rpc_msg *callmsg)
 {
-        int                     ret = -1;
+        int32_t                 ret = 0;
         rpcsvc_auth_t           *auth = NULL;
 
-        if (!req)
-                return -1;
+        if (!req || !callmsg) {
+                ret = -1;
+                goto err;
+        }
+
+        req->cred.flavour = rpc_call_cred_flavour (callmsg);
+        req->cred.datalen = rpc_call_cred_len (callmsg);
+        req->verf.flavour = rpc_call_verf_flavour (callmsg);
+        req->verf.datalen = rpc_call_verf_len (callmsg);
 
         auth = rpcsvc_auth_get_handler (req);
-        if (!auth)
+        if (!auth) {
+                ret = -1;
                 goto err;
-        ret = 0;
-        gf_log (GF_RPCSVC, GF_LOG_TRACE, "Auth handler: %s", auth->authname);
-        if (!auth->authops->request_init)
-                ret = auth->authops->request_init (req, auth->authprivate);
+        }
 
-	req->auxgids = req->auxgidsmall; /* reset to auxgidlarge during
-					    unsersialize if necessary */
-	req->auxgidlarge = NULL;
+        gf_log (GF_RPCSVC, GF_LOG_TRACE, "Auth handler: %s", auth->authname);
+
+        if (auth->authops->request_init)
+              ret = auth->authops->request_init (req, auth->authprivate);
+
+        /* reset to auxgidlarge during
+           unsersialize if necessary */
+        req->auxgids = req->auxgidsmall;
+        req->auxgidlarge = NULL;
 err:
         return ret;
 }
