@@ -616,7 +616,7 @@ rpcsvc_handle_rpc_call (rpcsvc_t *svc, rpc_transport_t *trans,
 
                 gf_log ("rpcsvc", GF_LOG_TRACE, "Client port: %d", (int)port);
 
-                if (port > 1024)
+                if (port >= 1024)
                         unprivileged = _gf_true;
         }
 
