@@ -283,14 +283,15 @@ struct rpcsvc_request {
                 int gidcount = 0;                                       \
                 if (req->svc->root_squash) {                            \
                         if (req->uid == RPC_ROOT_UID)                   \
-                                req->uid = RPC_NOBODY_UID;              \
+                                req->uid = req->svc->anonuid;           \
                         if (req->gid == RPC_ROOT_GID)                   \
-                                req->gid = RPC_NOBODY_GID;              \
+                                req->gid = req->svc->anongid;           \
+                                                                        \
                         for (gidcount = 0; gidcount < req->auxgidcount; \
                              ++gidcount) {                              \
                                 if (!req->auxgids[gidcount])            \
                                         req->auxgids[gidcount] =        \
-                                                RPC_NOBODY_GID;         \
+                                                req->svc->anongid;      \
                         }                                               \
                 }                                                       \
         } while (0);
@@ -499,11 +500,10 @@ rpcsvc_transport_peeraddr (rpc_transport_t *trans, char *addrstr, int addrlen,
                            struct sockaddr_storage *returnsa, socklen_t sasize);
 
 extern int
-rpcsvc_auth_check (rpcsvc_t *svc, char *volname, rpc_transport_t *trans);
+rpcsvc_auth_check (rpcsvc_t *svc, char *volname, char *ipaddr);
 
 extern int
-rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname,
-                                 rpc_transport_t *trans);
+rpcsvc_transport_privport_check (rpcsvc_t *svc, char *volname, uint16_t port);
 
 #define rpcsvc_request_seterr(req, err)                 (req)->rpc_err = err
 #define rpcsvc_request_set_autherr(req, err)            (req)->auth_err = err
@@ -554,7 +554,7 @@ struct rpcsvc_auth_list {
 };
 
 extern int
-rpcsvc_auth_request_init (rpcsvc_request_t *req);
+rpcsvc_auth_request_init (rpcsvc_request_t *req, struct rpc_msg *callmsg);
 
 extern int
 rpcsvc_auth_init (rpcsvc_t *svc, dict_t *options);
@@ -582,6 +582,11 @@ rpcsvc_auth_unix_auxgids (rpcsvc_request_t *req, int *arrlen);
 extern char *
 rpcsvc_volume_allowed (dict_t *options, char *volname);
 
+int rpcsvc_request_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
+                           rpcsvc_cbk_program_t *prog, int procnum,
+                           void *req, glusterfs_ctx_t *ctx,
+                           xdrproc_t xdrproc);
+
 int rpcsvc_callback_submit (rpcsvc_t *rpc, rpc_transport_t *trans,
                             rpcsvc_cbk_program_t *prog, int procnum,
                             struct iovec *proghdr, int proghdrcount);
@@ -599,9 +604,19 @@ int
 rpcsvc_set_root_squash (rpcsvc_t *svc, dict_t *options);
 int
 rpcsvc_set_outstanding_rpc_limit (rpcsvc_t *svc, dict_t *options, int defvalue);
+
+int
+rpcsvc_set_throttle_on (rpcsvc_t *svc);
+
+int
+rpcsvc_set_throttle_off (rpcsvc_t *svc);
+
+gf_boolean_t
+rpcsvc_get_throttle (rpcsvc_t *svc);
+
 int
 rpcsvc_auth_array (rpcsvc_t *svc, char *volname, int *autharr, int arrlen);
 rpcsvc_vector_sizer
 rpcsvc_get_program_vector_sizer (rpcsvc_t *svc, uint32_t prognum,
-                                 uint32_t progver, uint32_t procnum);
+                                 uint32_t progver, int procnum);
 #endif
