@@ -29,6 +29,7 @@
 enum gf_dump_procnum {
         GF_DUMP_NULL,
         GF_DUMP_DUMP,
+        GF_DUMP_PING,
         GF_DUMP_MAXVALUE,
 };
 
@@ -64,11 +65,18 @@ enum gf_dump_procnum {
 #define GF_AUTH_GLUSTERFS_MAX_LKOWNER(groups_len)  \
            (95 - groups_len)
 
-#if GF_DARWIN_HOST_OS
+#ifdef GF_LINUX_HOST_OS
+#define xdr_u_int32_t xdr_uint32_t
+#define xdr_u_int64_t xdr_uint64_t
+#endif
+
+#ifdef GF_DARWIN_HOST_OS
 #define xdr_u_quad_t xdr_u_int64_t
 #define xdr_quad_t   xdr_int64_t
 #define xdr_uint32_t xdr_u_int32_t
+#define xdr_uint64_t xdr_u_int64_t
 #define uint64_t u_int64_t
+unsigned long xdr_sizeof (xdrproc_t func, void *data);
 #endif
 
 #if defined(__NetBSD__)
@@ -78,13 +86,11 @@ enum gf_dump_procnum {
 #define xdr_uint64_t xdr_u_int64_t
 #endif
 
-
-#if GF_SOLARIS_HOST_OS
+#ifdef GF_SOLARIS_HOST_OS
 #define u_quad_t uint64_t
 #define quad_t int64_t
 #define xdr_u_quad_t xdr_uint64_t
 #define xdr_quad_t   xdr_int64_t
-#define xdr_uint32_t xdr_uint32_t
 #endif
 
 /* Returns the address of the byte that follows the
