@@ -193,7 +193,7 @@ xdr_to_auth_unix_cred (char *msgbuf, int msglen, struct authunix_parms *au,
         GF_VALIDATE_OR_GOTO ("rpc", au, out);
 
         au->aup_machname = machname;
-#ifdef GF_DARWIN_HOST_OS
+#if defined(GF_DARWIN_HOST_OS) || defined(__FreeBSD__)
         au->aup_gids = (int *)gids;
 #else
         au->aup_gids = gids;
