@@ -13,6 +13,14 @@
 
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#include <openssl/x509v3.h>
+#ifdef ERR_R_DH_LIB
+#include <openssl/dh.h>
+#endif
+#ifdef ERR_R_ECDH_LIB
+#include <openssl/objects.h>
+#include <openssl/ecdh.h>
+#endif
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
@@ -59,6 +67,7 @@ typedef enum {
         SP_STATE_RPCFRAG_INIT,
         SP_STATE_READING_MSGTYPE,
         SP_STATE_READ_MSGTYPE,
+        SP_STATE_NOTIFYING_XID
 } sp_rpcfrag_state_t;
 
 typedef enum {
@@ -186,8 +195,10 @@ struct gf_sock_incoming {
 
 typedef enum {
         OT_IDLE,        /* Uninitialized or termination complete. */
-        OT_ALIVE,       /* Past pthread_create, no error/disconnect. */
-        OT_DYING,       /* Disconnect in progress. */
+        OT_SPAWNING,    /* Past pthread_create but not in thread yet. */
+        OT_RUNNING,     /* Poller thread running normally. */
+        OT_CALLBACK,    /* Poller thread in the middle of a callback. */
+        OT_PLEASE_DIE,  /* Poller termination requested. */
 } ot_state_t;
 
 typedef struct {
@@ -213,9 +224,12 @@ typedef struct {
         int                    keepalive;
         int                    keepaliveidle;
         int                    keepaliveintvl;
+        int                    timeout;
         uint32_t               backlog;
         gf_boolean_t           read_fail_log;
-        gf_boolean_t           ssl_enabled;
+        gf_boolean_t           ssl_enabled;     /* outbound I/O */
+        gf_boolean_t           mgmt_ssl;        /* outbound mgmt */
+        mgmt_ssl_t             srvr_ssl;
 	gf_boolean_t           use_ssl;
 	SSL_METHOD            *ssl_meth;
 	SSL_CTX               *ssl_ctx;
@@ -229,7 +243,8 @@ typedef struct {
 	int                    pipe[2];
 	gf_boolean_t           own_thread;
         ot_state_t             ot_state;
-        pthread_cond_t         ot_event;
+        uint32_t               ot_gen;
+        gf_boolean_t           is_server;
 } socket_private_t;
 
 
