@@ -17,6 +17,7 @@ main (int argc, char **argv)
         int ret = 0;
         int fd = 0;
         char    *cmd = argv[1];
+        struct stat stbuf = {0, };
 
         printf ("cmd is: %s\n", cmd);
         fd = open("a.txt", O_CREAT|O_RDWR, 0644);
@@ -30,6 +31,9 @@ main (int argc, char **argv)
                 printf ("Not able to print %s\n", strerror (errno));
         system(cmd);
         sleep(1); /* No way to confirm graph switch so sleep 1 */
+        ret = fstat (fd, &stbuf);
+        if (ret < 0)
+                printf ("fstat failed %\n", strerror (errno));
         ret = fsync(fd);
         if (ret < 0)
                 printf ("Not able to fsync %s\n", strerror (errno));
