@@ -41,10 +41,11 @@ enum gf_afr_mem_types_ {
         gf_afr_mt_shd_event_t,
         gf_afr_mt_time_t,
         gf_afr_mt_pos_data_t,
-        gf_afr_mt_reply_t,
-        gf_afr_mt_stats_t,
-        gf_afr_mt_shd_crawl_event_t,
-        gf_afr_mt_uint64_t,
+	gf_afr_mt_reply_t,
+	gf_afr_mt_subvol_healer_t,
+	gf_afr_mt_spbc_timeout_t,
+        gf_afr_mt_spb_status_t,
+        gf_afr_mt_replace_brick_t,
         gf_afr_mt_end
 };
 #endif
