@@ -44,12 +44,16 @@ __afr_txn_write_fop (call_frame_t *frame, xlator_t *this)
         afr_local_t *local = NULL;
         afr_private_t *priv = NULL;
         int call_count = -1;
+        unsigned char *failed_subvols = NULL;
         int i = 0;
 
         local = frame->local;
         priv = this->private;
 
-        call_count = AFR_COUNT (local->transaction.pre_op, priv->child_count);
+        failed_subvols = local->transaction.failed_subvols;
+
+        call_count = priv->child_count - AFR_COUNT (failed_subvols,
+                                                    priv->child_count);
 
         if (call_count == 0) {
                 local->transaction.resume (frame, this);
@@ -59,7 +63,7 @@ __afr_txn_write_fop (call_frame_t *frame, xlator_t *this)
         local->call_count = call_count;
 
         for (i = 0; i < priv->child_count; i++) {
-                if (local->transaction.pre_op[i]) {
+                if (local->transaction.pre_op[i] && !failed_subvols[i]) {
 			local->transaction.wind (frame, this, i);
 
                         if (!--call_count)
@@ -493,18 +497,14 @@ afr_txn_nothing_failed (call_frame_t *frame, xlator_t *this)
 {
         afr_private_t *priv = NULL;
         afr_local_t *local = NULL;
-        int pre_op_count = 0;
         int i = 0;
 
         local = frame->local;
 	priv = this->private;
 
-        pre_op_count = AFR_COUNT (local->transaction.pre_op, priv->child_count);
-        if (pre_op_count < priv->child_count)
-                return _gf_false;
-
         for (i = 0; i < priv->child_count; i++) {
-                if (local->transaction.failed_subvols[i])
+                if (local->transaction.pre_op[i] &&
+                    local->transaction.failed_subvols[i])
                         return _gf_false;
         }
 
@@ -735,20 +735,6 @@ afr_changelog_post_op_now (call_frame_t *frame, xlator_t *this)
 		goto out;
 	}
 
-	if (need_undirty)
-		local->dirty[idx] = hton32(-1);
-	else
-		local->dirty[idx] = hton32(0);
-
-	ret = dict_set_static_bin (xattr, AFR_DIRTY, local->dirty,
-				   sizeof(int) * AFR_NUM_CHANGE_LOGS);
-	if (ret) {
-		local->op_ret = -1;
-		local->op_errno = ENOMEM;
-		afr_changelog_post_op_done (frame, this);
-		goto out;
-	}
-
 	for (i = 0; i < priv->child_count; i++) {
 		if (local->transaction.failed_subvols[i])
 			local->pending[i][idx] = hton32(1);
@@ -762,6 +748,20 @@ afr_changelog_post_op_now (call_frame_t *frame, xlator_t *this)
 		goto out;
 	}
 
+        if (need_undirty)
+		local->dirty[idx] = hton32(-1);
+	else
+		local->dirty[idx] = hton32(0);
+
+	ret = dict_set_static_bin (xattr, AFR_DIRTY, local->dirty,
+				   sizeof(int) * AFR_NUM_CHANGE_LOGS);
+	if (ret) {
+		local->op_ret = -1;
+		local->op_errno = ENOMEM;
+		afr_changelog_post_op_done (frame, this);
+		goto out;
+	}
+
 	afr_changelog_do (frame, this, xattr, afr_changelog_post_op_done);
 out:
 	if (xattr)
@@ -977,8 +977,10 @@ afr_changelog_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         priv = this->private;
         child_index = (long) cookie;
 
-	if (op_ret == -1)
+	if (op_ret == -1) {
+                local->op_errno = op_errno;
 		afr_transaction_fop_failed (frame, this, child_index);
+        }
 
         if (priv->arbiter_count == 1 && !op_ret) {
                 if (xattr)
@@ -1156,14 +1158,9 @@ afr_changelog_pre_op (call_frame_t *frame, xlator_t *this)
 		goto err;
 	}
 
-	if (call_count > 1 &&
-	    (local->transaction.type == AFR_DATA_TRANSACTION ||
+	if ((local->transaction.type == AFR_DATA_TRANSACTION ||
 	     !local->optimistic_change_log)) {
 
-		/* If we are performing change on only one subvol, no
-		   need to mark dirty, because we are setting the pending
-		   counts already anyways
-		*/
 		local->dirty[idx] = hton32(1);
 
 		ret = dict_set_static_bin (xdata_req, AFR_DIRTY, local->dirty,
