@@ -42,8 +42,12 @@ typedef int (*dht_selfheal_dir_cbk_t) (call_frame_t *frame, void *cookie,
                                        int32_t       op_ret, int32_t op_errno,
                                        dict_t       *xdata);
 typedef int (*dht_defrag_cbk_fn_t) (xlator_t        *this, xlator_t *dst_node,
-                                    call_frame_t    *frame);
+                                    call_frame_t    *frame, int ret);
 
+typedef int (*dht_refresh_layout_unlock) (call_frame_t *frame, xlator_t *this,
+                                         int op_ret);
+
+typedef int (*dht_refresh_layout_done_handle) (call_frame_t *frame);
 
 struct dht_layout {
         int                spread_cnt;  /* layout spread count per directory,
@@ -117,8 +121,12 @@ struct dht_rebalance_ {
         struct iobref       *iobref;
         struct iovec        *vector;
         struct iatt          stbuf;
+        struct iatt          prebuf;
+        struct iatt          postbuf;
         dht_defrag_cbk_fn_t  target_op_fn;
         dict_t              *xdata;
+        dict_t              *xattr;
+        int32_t              set;
 };
 
 /**
@@ -212,6 +220,10 @@ struct dht_local {
                 gf_boolean_t            force_mkdir;
                 dht_layout_t           *layout, *refreshed_layout;
         } selfheal;
+
+        dht_refresh_layout_unlock              refresh_layout_unlock;
+        dht_refresh_layout_done_handle         refresh_layout_done;
+
         uint32_t                 uid;
         uint32_t                 gid;
 
@@ -286,7 +298,8 @@ enum gf_defrag_type {
         GF_DEFRAG_CMD_STATUS_TIER = 1 + 6,
 	GF_DEFRAG_CMD_START_DETACH_TIER = 1 + 7,
 	GF_DEFRAG_CMD_STOP_DETACH_TIER = 1 + 8,
-
+	GF_DEFRAG_CMD_PAUSE_TIER = 1 + 9,
+	GF_DEFRAG_CMD_RESUME_TIER = 1 + 10,
 };
 typedef enum gf_defrag_type gf_defrag_type;
 
@@ -325,6 +338,31 @@ struct dht_container {
         dict_t          *migrate_data;
 };
 
+typedef enum tier_mode_ {
+        TIER_MODE_NONE = 0,
+        TIER_MODE_TEST,
+        TIER_MODE_WM
+} tier_mode_t;
+
+typedef struct gf_tier_conf {
+        int                          is_tier;
+        int                          watermark_hi;
+        int                          watermark_low;
+        int                          watermark_last;
+        fsblkcnt_t                   blocks_total;
+        fsblkcnt_t                   blocks_used;
+        int                          percent_full;
+        uint64_t                     max_migrate_bytes;
+        int                          max_migrate_files;
+        tier_mode_t                  mode;
+        int                          tier_promote_frequency;
+        int                          tier_demote_frequency;
+        uint64_t                     st_last_promoted_size;
+        uint64_t                     st_last_demoted_size;
+        int                          request_pause;
+        gf_boolean_t                 paused;
+} gf_tier_conf_t;
+
 struct gf_defrag_info_ {
         uint64_t                     total_files;
         uint64_t                     total_data;
@@ -345,8 +383,7 @@ struct gf_defrag_info_ {
         gf_boolean_t                 stats;
         uint32_t                     new_commit_hash;
         gf_defrag_pattern_list_t    *defrag_pattern;
-        int                          tier_promote_frequency;
-        int                          tier_demote_frequency;
+        gf_tier_conf_t               tier_conf;
 
         /*Data Tiering params for scanner*/
         uint64_t                     total_files_promoted;
@@ -510,6 +547,7 @@ typedef struct dht_migrate_info {
         GF_REF_DECL;
 } dht_migrate_info_t;
 
+
 #define ENTRY_MISSING(op_ret, op_errno) (op_ret == -1 && op_errno == ENOENT)
 
 #define is_revalidate(loc) (dht_inode_ctx_layout_get (loc->inode, this, NULL) == 0)
@@ -544,6 +582,8 @@ typedef struct dht_migrate_info {
 
 #define layout_is_sane(layout) ((layout) && (layout->cnt > 0))
 
+#define we_are_not_migrating(x)   ((x) == 1)
+
 #define DHT_STACK_UNWIND(fop, frame, params ...) do {           \
                 dht_local_t *__local = NULL;                    \
                 xlator_t    *__xl    = NULL;                    \
@@ -949,6 +989,12 @@ int dht_newfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 int
 gf_defrag_status_get (gf_defrag_info_t *defrag, dict_t *dict);
 
+int
+gf_defrag_pause_tier (xlator_t *this, gf_defrag_info_t *defrag);
+
+int
+gf_defrag_resume_tier (xlator_t *this, gf_defrag_info_t *defrag);
+
 int
 gf_defrag_start_detach_tier (gf_defrag_info_t *defrag);
 
@@ -1067,4 +1113,18 @@ dht_layout_sort (dht_layout_t *layout);
 int
 dht_layout_missing_dirs (dht_layout_t *layout);
 
+int
+dht_refresh_layout (call_frame_t *frame);
+
+int
+dht_build_parent_loc (xlator_t *this, loc_t *parent, loc_t *child,
+                                                 int32_t *op_errno);
+
+int32_t dht_set_local_rebalance (xlator_t *this, dht_local_t *local,
+                                 struct iatt *stbuf,
+                                 struct iatt *prebuf,
+                                 struct iatt *postbuf, dict_t *xdata);
+void
+dht_build_root_loc (inode_t *inode, loc_t *loc);
+
 #endif/* _DHT_H */
