@@ -18,6 +18,9 @@
 #include "xlator.h"
 #include "dht-common.h"
 #include "byte-order.h"
+#include "dht-messages.h"
+#include "unittest/unittest.h"
+
 
 #define layout_base_size (sizeof (dht_layout_t))
 
@@ -25,13 +28,14 @@
 
 #define layout_size(cnt) (layout_base_size + (cnt * layout_entry_size))
 
-
 dht_layout_t *
 dht_layout_new (xlator_t *this, int cnt)
 {
         dht_layout_t *layout = NULL;
         dht_conf_t   *conf = NULL;
 
+        REQUIRE(NULL != this);
+        REQUIRE(cnt >= 0);
 
         conf = this->private;
 
@@ -50,6 +54,11 @@ dht_layout_new (xlator_t *this, int cnt)
         }
 
         layout->ref = 1;
+
+        ENSURE(NULL != layout);
+        ENSURE(layout->type == DHT_HASH_TYPE_DM);
+        ENSURE(layout->cnt == cnt);
+        ENSURE(layout->ref == 1);
 out:
         return layout;
 }
@@ -60,6 +69,7 @@ dht_layout_get (xlator_t *this, inode_t *inode)
 {
         dht_conf_t   *conf = NULL;
         dht_layout_t *layout = NULL;
+        int           ret = 0;
 
         conf = this->private;
         if (!conf)
@@ -67,8 +77,8 @@ dht_layout_get (xlator_t *this, inode_t *inode)
 
         LOCK (&conf->layout_lock);
         {
-                dht_inode_ctx_layout_get (inode, this, &layout);
-                if (layout) {
+                ret = dht_inode_ctx_layout_get (inode, this, &layout);
+                if ((!ret) && layout) {
                         layout->ref++;
                 }
         }
@@ -88,13 +98,14 @@ dht_layout_set (xlator_t *this, inode_t *inode, dht_layout_t *layout)
         dht_layout_t *old_layout;
 
         conf = this->private;
-        if (!conf)
+        if (!conf || !layout)
                 goto out;
 
         LOCK (&conf->layout_lock);
         {
                 oldret = dht_inode_ctx_layout_get (inode, this, &old_layout);
-                layout->ref++;
+                if (layout)
+                        layout->ref++;
                 dht_inode_ctx_layout_set (inode, this, layout);
         }
         UNLOCK (&conf->layout_lock);
@@ -157,10 +168,10 @@ dht_layout_search (xlator_t *this, dht_layout_t *layout, const char *name)
         int        i = 0;
         int        ret = 0;
 
-
         ret = dht_hash_compute (this, layout->type, name, &hash);
         if (ret != 0) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_COMPUTE_HASH_FAILED,
                         "hash computation failed for type=%d name=%s",
                         layout->type, name);
                 goto out;
@@ -175,7 +186,8 @@ dht_layout_search (xlator_t *this, dht_layout_t *layout, const char *name)
         }
 
         if (!subvol) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_HASHED_SUBVOL_GET_FAILED,
                         "no subvolume for hash (value) = %u", hash);
         }
 
@@ -257,7 +269,7 @@ dht_disk_layout_extract (xlator_t *this, dht_layout_t *layout,
                 goto out;
         }
 
-        disk_layout[0] = hton32 (1);
+        disk_layout[0] = hton32 (layout->list[pos].commit_hash);
         disk_layout[1] = hton32 (layout->type);
         disk_layout[2] = hton32 (layout->list[pos].start);
         disk_layout[3] = hton32 (layout->list[pos].stop);
@@ -278,14 +290,15 @@ int
 dht_disk_layout_merge (xlator_t *this, dht_layout_t *layout,
 		       int pos, void *disk_layout_raw, int disk_layout_len)
 {
-        int      cnt = 0;
         int      type = 0;
         int      start_off = 0;
         int      stop_off = 0;
+        int      commit_hash = 0;
         int      disk_layout[4];
 
 	if (!disk_layout_raw) {
-		gf_log (this->name, GF_LOG_CRITICAL,
+		gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        DHT_MSG_LAYOUT_MERGE_FAILED,
                         "error no layout on disk for merge");
 		return -1;
 	}
@@ -294,43 +307,39 @@ dht_disk_layout_merge (xlator_t *this, dht_layout_t *layout,
 
         memcpy (disk_layout, disk_layout_raw, disk_layout_len);
 
-        cnt  = ntoh32 (disk_layout[0]);
-        if (cnt != 1) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "disk layout has invalid count %d", cnt);
-                return -1;
-        }
-
         type = ntoh32 (disk_layout[1]);
 	switch (type) {
         case DHT_HASH_TYPE_DM_USER:
-                gf_log (this->name, GF_LOG_DEBUG, "found user-set layout");
+                gf_msg_debug (this->name, 0, "found user-set layout");
                 layout->type = type;
                 /* Fall through. */
 	case DHT_HASH_TYPE_DM:
 		break;
         default:
-		gf_log (this->name, GF_LOG_CRITICAL,
-			"Catastrophic error layout with unknown type found %d",
+		gf_msg (this->name, GF_LOG_CRITICAL, 0,
+                        DHT_MSG_INVALID_DISK_LAYOUT,
+			"Invalid disk layout: "
+                        "Catastrophic error layout with unknown type found %d",
 			disk_layout[1]);
 		return -1;
 	}
 
+        commit_hash = ntoh32 (disk_layout[0]);
         start_off = ntoh32 (disk_layout[2]);
         stop_off  = ntoh32 (disk_layout[3]);
 
+        layout->list[pos].commit_hash = commit_hash;
         layout->list[pos].start = start_off;
         layout->list[pos].stop  = stop_off;
 
-        gf_log (this->name, GF_LOG_TRACE,
-                "merged to layout: %u - %u (type %d) from %s",
-                start_off, stop_off, type,
-                layout->list[pos].xlator->name);
+        gf_msg_trace (this->name, 0,
+                      "merged to layout: %u - %u (type %d, hash %d) from %s",
+                      start_off, stop_off, commit_hash, type,
+                      layout->list[pos].xlator->name);
 
         return 0;
 }
 
-
 int
 dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
                   int op_ret, int op_errno, dict_t *xattr)
@@ -367,9 +376,9 @@ dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 
         if (ret != 0) {
                 layout->list[i].err = 0;
-                gf_log (this->name, GF_LOG_TRACE,
-                        "missing disk layout on %s. err = %d",
-                        subvol->name, err);
+                gf_msg_trace (this->name, 0,
+                              "Missing disk layout on %s. err = %d",
+                              subvol->name, err);
                 ret = 0;
                 goto out;
         }
@@ -377,11 +386,19 @@ dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
         ret = dht_disk_layout_merge (this, layout, i, disk_layout_raw,
 				     disk_layout_len);
         if (ret != 0) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_LAYOUT_MERGE_FAILED,
                         "layout merge from subvolume %s failed",
                         subvol->name);
                 goto out;
         }
+
+        if (layout->commit_hash == 0) {
+                layout->commit_hash = layout->list[i].commit_hash;
+        } else if (layout->commit_hash != layout->list[i].commit_hash) {
+                layout->commit_hash = DHT_LAYOUT_HASH_INVALID;
+        }
+
         layout->list[i].err = 0;
 
 out:
@@ -394,6 +411,7 @@ dht_layout_entry_swap (dht_layout_t *layout, int i, int j)
 {
         uint32_t  start_swap = 0;
         uint32_t  stop_swap = 0;
+        uint32_t  commit_hash_swap = 0;
         xlator_t *xlator_swap = 0;
         int       err_swap = 0;
 
@@ -401,16 +419,19 @@ dht_layout_entry_swap (dht_layout_t *layout, int i, int j)
         stop_swap   = layout->list[i].stop;
         xlator_swap = layout->list[i].xlator;
         err_swap    = layout->list[i].err;
+        commit_hash_swap = layout->list[i].commit_hash;
 
         layout->list[i].start  = layout->list[j].start;
         layout->list[i].stop   = layout->list[j].stop;
         layout->list[i].xlator = layout->list[j].xlator;
         layout->list[i].err    = layout->list[j].err;
+        layout->list[i].commit_hash = layout->list[j].commit_hash;
 
         layout->list[j].start  = start_swap;
         layout->list[j].stop   = stop_swap;
         layout->list[j].xlator = xlator_swap;
         layout->list[j].err    = err_swap;
+        layout->list[j].commit_hash = commit_hash_swap;
 }
 
 void
@@ -436,15 +457,19 @@ dht_layout_entry_cmp_volname (dht_layout_t *layout, int i, int j)
                         layout->list[j].xlator->name));
 }
 
-
 gf_boolean_t
 dht_is_subvol_in_layout (dht_layout_t *layout, xlator_t *xlator)
 {
         int i = 0;
 
         for (i = 0; i < layout->cnt; i++) {
-                if (!strcmp (layout->list[i].xlator->name, xlator->name))
-                        return _gf_true;
+                /* Check if xlator is already part of layout, and layout is
+                 * non-zero. */
+                if (!strcmp (layout->list[i].xlator->name, xlator->name)) {
+                        if (layout->list[i].start != layout->list[i].stop)
+                                return _gf_true;
+                        break;
+                }
         }
         return _gf_false;
 }
@@ -460,11 +485,8 @@ dht_layout_entry_cmp (dht_layout_t *layout, int i, int j)
                        - (int64_t) layout->list[j].stop;
                        goto out;
         }
-        if (layout->list[i].err || layout->list[j].err)
-                diff = layout->list[i].err - layout->list[j].err;
-        else
-                diff = (int64_t) layout->list[i].start
-                        - (int64_t) layout->list[j].start;
+        diff = (int64_t) layout->list[i].start
+                - (int64_t) layout->list[j].start;
 
 out:
         return diff;
@@ -531,8 +553,20 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
         char        is_virgin = 1;
         uint32_t    no_space  = 0;
 
-        /* TODO: explain what is happening */
+        /* This function scans through the layout spread of a directory to
+           check if there are any anomalies. Prior to calling this function
+           the layout entries should be sorted in the ascending order.
 
+           If the layout entry has err != 0
+                then increment the corresponding anomaly.
+           else
+                if (start of the current layout entry > stop + 1 of previous
+                   non erroneous layout entry)
+                        then it indicates a hole in the layout
+                if (start of the current layout entry < stop + 1 of previous
+                    non erroneous layout entry)
+                         then it indicates an overlap in the layout
+        */
         last_stop = layout->list[0].start - 1;
         prev_stop = last_stop;
 
@@ -601,66 +635,84 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
 }
 
 
+int
+dht_layout_missing_dirs (dht_layout_t *layout)
+{
+        int i = 0, missing = 0;
+
+        if (layout == NULL)
+                goto out;
+
+        for (i = 0; i < layout->cnt; i++) {
+                if ((layout->list[i].err == ENOENT)
+                    || ((layout->list[i].err == -1)
+                        && (layout->list[i].start == 0)
+                        && (layout->list[i].stop == 0))) {
+                        missing++;
+                }
+        }
+
+out:
+        return missing;
+}
+
+
 int
 dht_layout_normalize (xlator_t *this, loc_t *loc, dht_layout_t *layout)
 {
         int          ret   = 0;
-        int          i = 0;
         uint32_t     holes = 0;
         uint32_t     overlaps = 0;
         uint32_t     missing = 0;
         uint32_t     down = 0;
-        uint32_t     misc = 0;
+        uint32_t     misc = 0, missing_dirs = 0;
+        char         gfid[GF_UUID_BUF_SIZE] = {0};
 
         ret = dht_layout_sort (layout);
         if (ret == -1) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_LAYOUT_SORT_FAILED,
                         "sort failed?! how the ....");
                 goto out;
         }
 
+        gf_uuid_unparse(loc->gfid, gfid);
+
         ret = dht_layout_anomalies (this, loc, layout,
                                     &holes, &overlaps,
                                     &missing, &down, &misc, NULL);
         if (ret == -1) {
-                gf_log (this->name, GF_LOG_WARNING,
-                        "error while finding anomalies in %s -- not good news",
-                        loc->path);
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        DHT_MSG_FIND_LAYOUT_ANOMALIES_ERROR,
+                        "Error finding anomalies in %s, gfid = %s",
+                        loc->path, gfid);
                 goto out;
         }
 
         if (holes || overlaps) {
                 if (missing == layout->cnt) {
-                        gf_log (this->name, GF_LOG_DEBUG,
-                                "directory %s looked up first time",
-                                loc->path);
+                        gf_msg_debug (this->name, 0,
+                                      "Directory %s looked up first time"
+                                      " gfid = %s", loc->path, gfid);
                 } else {
-                        gf_log (this->name, GF_LOG_INFO,
-                                "found anomalies in %s. holes=%d overlaps=%d",
-                                loc->path, holes, overlaps);
+                        gf_msg (this->name, GF_LOG_INFO, 0,
+                                DHT_MSG_ANOMALIES_INFO,
+                                "Found anomalies in %s (gfid = %s). "
+                                "Holes=%d overlaps=%d",
+                                loc->path, gfid, holes, overlaps );
                 }
                 ret = -1;
         }
 
-        for (i = 0; i < layout->cnt; i++) {
+        if (ret >= 0) {
+                missing_dirs = dht_layout_missing_dirs (layout);
                 /* TODO During DHT selfheal rewrite (almost) find a better place
                  * to detect this - probably in dht_layout_anomalies()
                  */
-                if (layout->list[i].err > 0) {
-                        gf_log_callingfn (this->name, GF_LOG_DEBUG,
-                                          "path=%s err=%s on subvol=%s",
-                                          loc->path,
-                                          strerror (layout->list[i].err),
-                                          (layout->list[i].xlator ?
-                                           layout->list[i].xlator->name
-                                           : "<>"));
-                        if ((layout->list[i].err == ENOENT) && (ret >= 0)) {
-                                ret++;
-                        }
-                }
+                if (missing_dirs > 0)
+                        ret += missing_dirs;
         }
 
-
 out:
         return ret;
 }
@@ -685,11 +737,14 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
         int         dict_ret = 0;
         int32_t     disk_layout[4];
         void       *disk_layout_raw = NULL;
-        int32_t     count = -1;
         uint32_t    start_off = -1;
         uint32_t    stop_off = -1;
+        uint32_t    commit_hash = -1;
         dht_conf_t *conf = this->private;
+        char        gfid[GF_UUID_BUF_SIZE] = {0};
 
+        if(loc && loc->inode)
+                gf_uuid_unparse(loc->inode->gfid, gfid);
 
         for (idx = 0; idx < layout->cnt; idx++) {
                 if (layout->list[idx].xlator == subvol) {
@@ -699,9 +754,9 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
         }
 
         if (pos == -1) {
-                gf_log (this->name, GF_LOG_DEBUG,
-                        "%s - no layout info for subvolume %s",
-                        loc->path, subvol->name);
+                gf_msg_debug (this->name, 0,
+                              "%s - no layout info for subvolume %s",
+                              loc->path, subvol->name);
                 ret = 1;
                 goto out;
         }
@@ -710,8 +765,9 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 
         if (!xattr) {
                 if (err == 0) {
-                        gf_log (this->name, GF_LOG_INFO,
-                                "%s - xattr dictionary is NULL",
+                        gf_msg (this->name, GF_LOG_INFO, 0,
+                                DHT_MSG_DICT_GET_FAILED,
+                                "%s: xattr dictionary is NULL",
                                 loc->path);
                         ret = -1;
                 }
@@ -722,9 +778,11 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
                                  &disk_layout_raw);
 
         if (dict_ret < 0) {
-                if (err == 0) {
-                        gf_log (this->name, GF_LOG_INFO,
-                                "%s - disk layout missing", loc->path);
+                if (err == 0 && layout->list[pos].stop) {
+                        gf_msg (this->name, GF_LOG_INFO, 0,
+                                DHT_MSG_DISK_LAYOUT_MISSING,
+                                "%s: Disk layout missing, gfid = %s",
+                                loc->path, gfid);
                         ret = -1;
                 }
                 goto out;
@@ -732,26 +790,22 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 
         memcpy (disk_layout, disk_layout_raw, sizeof (disk_layout));
 
-        count  = ntoh32 (disk_layout[0]);
-        if (count != 1) {
-                gf_log (this->name, GF_LOG_ERROR,
-                        "%s - disk layout has invalid count %d",
-                        loc->path, count);
-                ret = -1;
-                goto out;
-        }
-
         start_off = ntoh32 (disk_layout[2]);
         stop_off  = ntoh32 (disk_layout[3]);
+        commit_hash = ntoh32 (disk_layout[0]);
 
         if ((layout->list[pos].start != start_off)
-            || (layout->list[pos].stop != stop_off)) {
-                gf_log (this->name, GF_LOG_INFO,
-                        "subvol: %s; inode layout - %"PRIu32" - %"PRIu32"; "
-                        "disk layout - %"PRIu32" - %"PRIu32,
+            || (layout->list[pos].stop != stop_off)
+            || (layout->list[pos].commit_hash != commit_hash)) {
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        DHT_MSG_LAYOUT_INFO,
+                        "subvol: %s; inode layout - %"PRIu32" - %"PRIu32
+                        " - %"PRIu32"; "
+                        "disk layout - %"PRIu32" - %"PRIu32" - %"PRIu32,
                         layout->list[pos].xlator->name,
                         layout->list[pos].start, layout->list[pos].stop,
-                        start_off, stop_off);
+                        layout->list[pos].commit_hash,
+                        start_off, stop_off, commit_hash);
                 ret = 1;
         } else {
                 ret = 0;
@@ -774,7 +828,8 @@ dht_layout_preset (xlator_t *this, xlator_t *subvol, inode_t *inode)
 
         layout = dht_layout_for_subvol (this, subvol);
         if (!layout) {
-                gf_log (this->name, GF_LOG_INFO,
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        DHT_MSG_SUBVOL_NO_LAYOUT_INFO,
                         "no pre-set layout for subvolume %s",
                         subvol ? subvol->name : "<nil>");
                 ret = -1;
@@ -791,3 +846,18 @@ dht_layout_preset (xlator_t *this, xlator_t *subvol, inode_t *inode)
 out:
         return ret;
 }
+
+int
+dht_layout_index_for_subvol (dht_layout_t *layout, xlator_t *subvol)
+{
+        int i = 0, ret = -1;
+
+        for (i = 0; i < layout->cnt; i++) {
+                if (layout->list[i].xlator == subvol) {
+                        ret = i;
+                        break;
+                }
+        }
+
+        return ret;
+}
