@@ -88,7 +88,7 @@ void ec_trace(const char * event, ec_fop_data_t * fop, const char * fmt, ...)
            fop->expected, fop->error, fop->state,
            ec_bin(str1, sizeof(str1), fop->mask, ec->nodes),
            ec_bin(str2, sizeof(str2), fop->remaining, ec->nodes),
-           ec_bin(str3, sizeof(str3), fop->bad, ec->nodes), msg);
+           ec_bin(str3, sizeof(str3), fop->good, ec->nodes), msg);
 
     if (ret >= 0)
     {
@@ -163,11 +163,14 @@ int32_t ec_dict_set_array(dict_t *dict, char *key, uint64_t value[],
 {
     uint64_t   *ptr = NULL;
     int32_t     vindex;
-    if (value == NULL)
-        return -1;
+
+    if (value == NULL) {
+        return -EINVAL;
+    }
+
     ptr = GF_MALLOC(sizeof(uint64_t) * size, gf_common_mt_char);
     if (ptr == NULL) {
-        return -1;
+        return -ENOMEM;
     }
     for (vindex = 0; vindex < size; vindex++) {
          ptr[vindex] = hton64(value[vindex]);
@@ -183,14 +186,19 @@ int32_t ec_dict_del_array(dict_t *dict, char *key, uint64_t value[],
     int32_t len;
     int32_t vindex;
     int32_t old_size = 0;
+    int32_t err;
 
-    if ((dict == NULL) || (dict_get_ptr_and_len(dict, key, &ptr, &len) != 0)) {
-        return -1;
+    if (dict == NULL) {
+        return -EINVAL;
+    }
+    err = dict_get_ptr_and_len(dict, key, &ptr, &len);
+    if (err != 0) {
+        return err;
     }
 
-    if (len > (size * sizeof(uint64_t)) ||
-        (len % sizeof (uint64_t)))
-            return -1;
+    if (len > (size * sizeof(uint64_t)) || (len % sizeof (uint64_t))) {
+        return -EINVAL;
+    }
 
     memset (value, 0, size * sizeof(uint64_t));
     /* 3.6 version ec would have stored version in 64 bit. In that case treat
@@ -217,9 +225,8 @@ int32_t ec_dict_set_number(dict_t * dict, char * key, uint64_t value)
     uint64_t * ptr;
 
     ptr = GF_MALLOC(sizeof(value), gf_common_mt_char);
-    if (ptr == NULL)
-    {
-        return -1;
+    if (ptr == NULL) {
+        return -ENOMEM;
     }
 
     *ptr = hton64(value);
@@ -230,12 +237,17 @@ int32_t ec_dict_set_number(dict_t * dict, char * key, uint64_t value)
 int32_t ec_dict_del_number(dict_t * dict, char * key, uint64_t * value)
 {
     void * ptr;
-    int32_t len;
+    int32_t len, err;
 
-    if ((dict == NULL) || (dict_get_ptr_and_len(dict, key, &ptr, &len) != 0) ||
-        (len != sizeof(uint64_t)))
-    {
-        return -1;
+    if (dict == NULL) {
+        return -EINVAL;
+    }
+    err = dict_get_ptr_and_len(dict, key, &ptr, &len);
+    if (err != 0) {
+        return err;
+    }
+    if (len != sizeof(uint64_t)) {
+        return -EINVAL;
     }
 
     *value = ntoh64(*(uint64_t *)ptr);
@@ -256,13 +268,13 @@ int32_t ec_dict_set_config(dict_t * dict, char * key, ec_config_t * config)
                 "Trying to store an unsupported config "
                 "version (%u)", config->version);
 
-        return -1;
+        return -EINVAL;
     }
 
     ptr = GF_MALLOC(sizeof(uint64_t), gf_common_mt_char);
     if (ptr == NULL)
     {
-        return -1;
+        return -ENOMEM;
     }
 
     data = ((uint64_t)config->version) << 56;
@@ -281,12 +293,17 @@ int32_t ec_dict_del_config(dict_t * dict, char * key, ec_config_t * config)
 {
     void * ptr;
     uint64_t data;
-    int32_t len;
+    int32_t len, err;
 
-    if ((dict == NULL) || (dict_get_ptr_and_len(dict, key, &ptr, &len) != 0) ||
-        (len != sizeof(uint64_t)))
-    {
-        return -1;
+    if (dict == NULL) {
+        return -EINVAL;
+    }
+    err = dict_get_ptr_and_len(dict, key, &ptr, &len);
+    if (err != 0) {
+        return err;
+    }
+    if (len != sizeof(uint64_t)) {
+        return -EINVAL;
     }
 
     data = ntoh64(*(uint64_t *)ptr);
@@ -299,7 +316,7 @@ int32_t ec_dict_del_config(dict_t * dict, char * key, ec_config_t * config)
                 "Found an unsupported config version (%u)",
                 config->version);
 
-        return -1;
+        return -EINVAL;
     }
 
     config->algorithm = (data >> 48) & 0xff;
@@ -313,35 +330,32 @@ int32_t ec_dict_del_config(dict_t * dict, char * key, ec_config_t * config)
     return 0;
 }
 
-int32_t ec_loc_gfid_check(xlator_t * xl, uuid_t dst, uuid_t src)
+gf_boolean_t ec_loc_gfid_check(xlator_t *xl, uuid_t dst, uuid_t src)
 {
-    if (gf_uuid_is_null(src))
-    {
-        return 1;
+    if (gf_uuid_is_null(src)) {
+        return _gf_true;
     }
 
-    if (gf_uuid_is_null(dst))
-    {
+    if (gf_uuid_is_null(dst)) {
         gf_uuid_copy(dst, src);
 
-        return 1;
+        return _gf_true;
     }
 
-    if (gf_uuid_compare(dst, src) != 0)
-    {
+    if (gf_uuid_compare(dst, src) != 0) {
         gf_msg (xl->name, GF_LOG_WARNING, 0,
                 EC_MSG_GFID_MISMATCH,
                 "Mismatching GFID's in loc");
 
-        return 0;
+        return _gf_false;
     }
 
-    return 1;
+    return _gf_true;
 }
 
 int32_t ec_loc_setup_inode(xlator_t *xl, inode_table_t *table, loc_t *loc)
 {
-    int32_t ret = -1;
+    int32_t ret = -EINVAL;
 
     if (loc->inode != NULL) {
         if (!ec_loc_gfid_check(xl, loc->gfid, loc->inode->gfid)) {
@@ -364,7 +378,7 @@ out:
 int32_t ec_loc_setup_parent(xlator_t *xl, inode_table_t *table, loc_t *loc)
 {
     char *path, *parent;
-    int32_t ret = -1;
+    int32_t ret = -EINVAL;
 
     if (loc->parent != NULL) {
         if (!ec_loc_gfid_check(xl, loc->pargfid, loc->parent->gfid)) {
@@ -381,6 +395,8 @@ int32_t ec_loc_setup_parent(xlator_t *xl, inode_table_t *table, loc_t *loc)
                         "Unable to duplicate path '%s'",
                         loc->path);
 
+                ret = -ENOMEM;
+
                 goto out;
             }
             parent = dirname(path);
@@ -408,13 +424,16 @@ int32_t ec_loc_setup_path(xlator_t *xl, loc_t *loc)
 {
     uuid_t root = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
     char *name;
-    int32_t ret = -1;
+    int32_t ret = -EINVAL;
 
     if (loc->path != NULL) {
         name = strrchr(loc->path, '/');
         if (name == NULL) {
-                ret = 0; /*<gfid:gfid-str>*/
-                goto out;
+            /* Allow gfid paths: <gfid:...> */
+            if (strncmp(loc->path, "<gfid:", 6) == 0) {
+                ret = 0;
+            }
+            goto out;
         }
         if (name == loc->path) {
             if (name[1] == 0) {
@@ -453,7 +472,7 @@ int32_t ec_loc_parent(xlator_t *xl, loc_t *loc, loc_t *parent)
 {
     inode_table_t *table = NULL;
     char *str = NULL;
-    int32_t ret = -1;
+    int32_t ret = -ENOMEM;
 
     memset(parent, 0, sizeof(loc_t));
 
@@ -487,18 +506,25 @@ int32_t ec_loc_parent(xlator_t *xl, loc_t *loc, loc_t *parent)
         }
     }
 
-    if ((ec_loc_setup_path(xl, parent) != 0) ||
-        (ec_loc_setup_inode(xl, table, parent) != 0) ||
-        (ec_loc_setup_parent(xl, table, parent) != 0)) {
+    ret = ec_loc_setup_path(xl, parent);
+    if (ret == 0) {
+        ret = ec_loc_setup_inode(xl, table, parent);
+    }
+    if (ret == 0) {
+        ret = ec_loc_setup_parent(xl, table, parent);
+    }
+    if (ret != 0) {
         goto out;
     }
 
     if ((parent->inode == NULL) && (parent->path == NULL) &&
         gf_uuid_is_null(parent->gfid)) {
-        gf_msg (xl->name, GF_LOG_ERROR, 0,
+        gf_msg (xl->name, GF_LOG_ERROR, EINVAL,
                 EC_MSG_LOC_PARENT_INODE_MISSING,
                 "Parent inode missing for loc_t");
 
+        ret = -EINVAL;
+
         goto out;
     }
 
@@ -507,8 +533,7 @@ int32_t ec_loc_parent(xlator_t *xl, loc_t *loc, loc_t *parent)
 out:
     GF_FREE(str);
 
-    if (ret != 0)
-    {
+    if (ret != 0) {
         loc_wipe(parent);
     }
 
@@ -519,7 +544,7 @@ int32_t ec_loc_update(xlator_t *xl, loc_t *loc, inode_t *inode,
                       struct iatt *iatt)
 {
     inode_table_t *table = NULL;
-    int32_t ret = -1;
+    int32_t ret = -EINVAL;
 
     if (inode != NULL) {
         table = inode->table;
@@ -542,14 +567,17 @@ int32_t ec_loc_update(xlator_t *xl, loc_t *loc, inode_t *inode,
         }
     }
 
-    if ((ec_loc_setup_path(xl, loc) != 0) ||
-        (ec_loc_setup_inode(xl, table, loc) != 0) ||
-        (ec_loc_setup_parent(xl, table, loc) != 0)) {
+    ret = ec_loc_setup_path(xl, loc);
+    if (ret == 0) {
+        ret = ec_loc_setup_inode(xl, table, loc);
+    }
+    if (ret == 0) {
+        ret = ec_loc_setup_parent(xl, table, loc);
+    }
+    if (ret != 0) {
         goto out;
     }
 
-    ret = 0;
-
 out:
     return ret;
 }
@@ -557,7 +585,7 @@ out:
 int32_t ec_loc_from_fd(xlator_t * xl, loc_t * loc, fd_t * fd)
 {
     ec_fd_t * ctx;
-    int32_t ret = -1;
+    int32_t ret = -ENOMEM;
 
     memset(loc, 0, sizeof(*loc));
 
@@ -568,12 +596,11 @@ int32_t ec_loc_from_fd(xlator_t * xl, loc_t * loc, fd_t * fd)
         }
     }
 
-    if (ec_loc_update(xl, loc, fd->inode, NULL) != 0) {
+    ret = ec_loc_update(xl, loc, fd->inode, NULL);
+    if (ret != 0) {
         goto out;
     }
 
-    ret = 0;
-
 out:
     if (ret != 0) {
         loc_wipe(loc);
@@ -584,7 +611,7 @@ out:
 
 int32_t ec_loc_from_loc(xlator_t * xl, loc_t * dst, loc_t * src)
 {
-    int32_t ret = -1;
+    int32_t ret = -ENOMEM;
 
     memset(dst, 0, sizeof(*dst));
 
@@ -592,12 +619,11 @@ int32_t ec_loc_from_loc(xlator_t * xl, loc_t * dst, loc_t * src)
         goto out;
     }
 
-    if (ec_loc_update(xl, dst, NULL, NULL) != 0) {
+    ret = ec_loc_update(xl, dst, NULL, NULL);
+    if (ret != 0) {
         goto out;
     }
 
-    ret = 0;
-
 out:
     if (ret != 0) {
         loc_wipe(dst);
