@@ -243,8 +243,8 @@ bitd_compare_ckum (xlator_t *this,
                 goto dictfree;
         }
 
-        gf_msg (this->name, GF_LOG_INFO, 0, BRB_MSG_MARK_CORRUPTED, "Marking %s"
-                " [GFID: %s | Brick: %s] as corrupted..", loc->path,
+        gf_msg (this->name, GF_LOG_ALERT, 0, BRB_MSG_MARK_CORRUPTED, "Marking"
+                " %s [GFID: %s | Brick: %s] as corrupted..", loc->path,
                 uuid_utoa (linked_inode->gfid), child->brick_path);
         ret = syncop_fsetxattr (child->xl, fd, xattr, 0, NULL, NULL);
         if (ret)
@@ -693,15 +693,9 @@ br_kickstart_scanner (struct gf_tw_timer_list *timer,
 }
 
 static inline uint32_t
-br_fsscan_calculate_delta (uint32_t boot, uint32_t now, uint32_t times)
+br_fsscan_calculate_delta (uint32_t times)
 {
-        uint32_t secs = 0;
-        uint32_t diff = 0;
-
-        diff = (now - boot);
-        secs = times * ((diff / times) + 1);
-
-        return (secs - diff);
+        return times;
 }
 
 #define BR_SCRUB_HOURLY     (60 * 60)
@@ -711,25 +705,25 @@ br_fsscan_calculate_delta (uint32_t boot, uint32_t now, uint32_t times)
 #define BR_SCRUB_MONTHLY    (30 * 24 * 60 * 60)
 
 static unsigned int
-br_fsscan_calculate_timeout (uint32_t boot, uint32_t now, scrub_freq_t freq)
+br_fsscan_calculate_timeout (scrub_freq_t freq)
 {
         uint32_t timo = 0;
 
         switch (freq) {
         case BR_FSSCRUB_FREQ_HOURLY:
-                timo = br_fsscan_calculate_delta (boot, now, BR_SCRUB_HOURLY);
+                timo = br_fsscan_calculate_delta (BR_SCRUB_HOURLY);
                 break;
         case BR_FSSCRUB_FREQ_DAILY:
-                timo = br_fsscan_calculate_delta (boot, now, BR_SCRUB_DAILY);
+                timo = br_fsscan_calculate_delta (BR_SCRUB_DAILY);
                 break;
         case BR_FSSCRUB_FREQ_WEEKLY:
-                timo = br_fsscan_calculate_delta (boot, now, BR_SCRUB_WEEKLY);
+                timo = br_fsscan_calculate_delta (BR_SCRUB_WEEKLY);
                 break;
         case BR_FSSCRUB_FREQ_BIWEEKLY:
-                timo = br_fsscan_calculate_delta (boot, now, BR_SCRUB_BIWEEKLY);
+                timo = br_fsscan_calculate_delta (BR_SCRUB_BIWEEKLY);
                 break;
         case BR_FSSCRUB_FREQ_MONTHLY:
-                timo = br_fsscan_calculate_delta (boot, now, BR_SCRUB_MONTHLY);
+                timo = br_fsscan_calculate_delta (BR_SCRUB_MONTHLY);
                 break;
         default:
                 timo = 0;
@@ -756,8 +750,7 @@ br_fsscan_schedule (xlator_t *this, br_child_t *child)
         (void) gettimeofday (&tv, NULL);
         fsscan->boot = tv.tv_sec;
 
-        timo = br_fsscan_calculate_timeout (fsscan->boot,
-                                            fsscan->boot, fsscrub->frequency);
+        timo = br_fsscan_calculate_timeout (fsscrub->frequency);
         if (timo == 0) {
                 gf_msg (this->name, GF_LOG_ERROR, 0, BRB_MSG_ZERO_TIMEOUT_BUG,
                         "BUG: Zero schedule timeout");
@@ -805,8 +798,7 @@ br_fsscan_activate (xlator_t *this, br_child_t *child)
         fsscrub = &priv->fsscrub;
 
         (void) gettimeofday (&now, NULL);
-        timo = br_fsscan_calculate_timeout (fsscan->boot,
-                                            now.tv_sec, fsscrub->frequency);
+        timo = br_fsscan_calculate_timeout (fsscrub->frequency);
         if (timo == 0) {
                 gf_msg (this->name, GF_LOG_ERROR, 0, BRB_MSG_ZERO_TIMEOUT_BUG,
                         "BUG: Zero schedule timeout");
@@ -840,9 +832,11 @@ br_fsscan_reschedule (xlator_t *this, br_child_t *child)
         fsscan = &child->fsscan;
         fsscrub = &priv->fsscrub;
 
+        if (!fsscrub->frequency_reconf)
+                return 0;
+
         (void) gettimeofday (&now, NULL);
-        timo = br_fsscan_calculate_timeout (fsscan->boot,
-                                            now.tv_sec, fsscrub->frequency);
+        timo = br_fsscan_calculate_timeout (fsscrub->frequency);
         if (timo == 0) {
                 gf_msg (this->name, GF_LOG_ERROR, 0, BRB_MSG_ZERO_TIMEOUT_BUG,
                         "BUG: Zero schedule timeout");
@@ -1196,6 +1190,7 @@ br_scrubber_handle_throttle (xlator_t *this, br_private_t *priv,
         scrub_throttle_t nthrottle = BR_SCRUB_THROTTLE_VOID;
 
         fsscrub = &priv->fsscrub;
+        fsscrub->throttle_reconf = _gf_false;
 
         ret = br_scrubber_fetch_option (this, "scrub-throttle", options, &tmp);
         if (ret)
@@ -1220,6 +1215,9 @@ br_scrubber_handle_throttle (xlator_t *this, br_private_t *priv,
         if (ret)
                 goto error_return;
 
+        if (fsscrub->throttle != nthrottle)
+                fsscrub->throttle_reconf = _gf_true;
+
         fsscrub->throttle = nthrottle;
         return 0;
 
@@ -1257,6 +1255,7 @@ br_scrubber_handle_freq (xlator_t *this, br_private_t *priv,
         struct br_scrubber *fsscrub = NULL;
 
         fsscrub = &priv->fsscrub;
+        fsscrub->frequency_reconf = _gf_true;
 
         ret = br_scrubber_fetch_option (this, "scrub-freq", options, &tmp);
         if (ret)
@@ -1280,7 +1279,11 @@ br_scrubber_handle_freq (xlator_t *this, br_private_t *priv,
         } else
                 goto error_return;
 
-        fsscrub->frequency = frequency;
+        if (fsscrub->frequency == frequency)
+                fsscrub->frequency_reconf = _gf_false;
+        else
+                fsscrub->frequency = frequency;
+
         return 0;
 
  error_return:
@@ -1308,10 +1311,12 @@ static void br_scrubber_log_option (xlator_t *this,
         if (scrubstall)
                 return; /* logged as pause */
 
-        gf_msg (this->name, GF_LOG_INFO, 0, BRB_MSG_SCRUB_TUNABLE, "SCRUB "
-                "TUNABLES:: [Frequency: %s, Throttle: %s]",
-                scrub_freq_str[fsscrub->frequency],
-                scrub_throttle_str[fsscrub->throttle]);
+        if (fsscrub->frequency_reconf || fsscrub->throttle_reconf) {
+                gf_msg (this->name, GF_LOG_INFO, 0, BRB_MSG_SCRUB_TUNABLE,
+                        "SCRUB TUNABLES:: [Frequency: %s, Throttle: %s]",
+                        scrub_freq_str[fsscrub->frequency],
+                        scrub_throttle_str[fsscrub->throttle]);
+        }
 }
 
 int32_t
