@@ -22,6 +22,7 @@
 #include "iatt.h"
 #include "glusterfs.h"
 #include "xlator.h"
+#include "defaults.h"
 #include "logging.h"
 #include "common-utils.h"
 #include <time.h>
@@ -34,6 +35,13 @@
 #define CTR_DEFAULT_HARDLINK_EXP_PERIOD 300  /* Five mins */
 #define CTR_DEFAULT_INODE_EXP_PERIOD    300 /* Five mins */
 
+
+typedef struct ctr_query_cbk_args {
+        int query_fd;
+        int count;
+} ctr_query_cbk_args_t;
+
+
 /*CTR Xlator Private structure*/
 typedef struct gf_ctr_private {
         gf_boolean_t                    enabled;
@@ -185,12 +193,12 @@ do {\
 } while (0)
 
 /*Clear and fill the ctr_link_context with values*/
-#define FILL_CTR_LINK_CX(ctr_link_cx, _pargfid, _basename, _basepath)\
+#define FILL_CTR_LINK_CX(ctr_link_cx, _pargfid, _basename, _basepath, label)\
 do {\
-        GF_ASSERT (ctr_link_cx);\
-        GF_ASSERT (_pargfid);\
-        GF_ASSERT (_basename);\
-        GF_ASSERT (_basepath);\
+        GF_VALIDATE_OR_GOTO ("ctr", ctr_link_cx, label);\
+        GF_VALIDATE_OR_GOTO ("ctr", _pargfid, label);\
+        GF_VALIDATE_OR_GOTO ("ctr", _basename, label);\
+        GF_VALIDATE_OR_GOTO ("ctr", _basepath, label);\
         memset (ctr_link_cx, 0, sizeof (*ctr_link_cx));\
         ctr_link_cx->pargfid = &_pargfid;\
         ctr_link_cx->basename = _basename;\
@@ -248,6 +256,13 @@ do {\
  *
  * ****************************************************************************/
 
+/*
+ * If a bitrot fop
+ * */
+#define BITROT_FOP(frame)\
+        (frame->root->pid == GF_CLIENT_PID_BITD)
+
+
 /*
  * If a rebalancer fop
  * */
@@ -279,21 +294,37 @@ do {\
  * Internal fop
  *
  * */
-#define CTR_IS_INTERNAL_FOP(frame, dict)\
-        (AFR_SELF_HEAL_FOP (frame) \
-        || REBALANCE_FOP (frame) \
-        || TIER_REBALANCE_FOP (frame) \
-        || (dict && \
-        dict_get (dict, GLUSTERFS_INTERNAL_FOP_KEY)))
+static inline
+gf_boolean_t is_internal_fop (call_frame_t *frame,
+                              dict_t       *xdata)
+{
+        gf_boolean_t ret = _gf_false;
+
+        GF_ASSERT(frame);
+        GF_ASSERT(frame->root);
+
+        if (AFR_SELF_HEAL_FOP (frame)) {
+                ret = _gf_true;
+        }
+        if (BITROT_FOP (frame)) {
+                ret = _gf_true;
+        }
+        if (REBALANCE_FOP (frame) || TIER_REBALANCE_FOP (frame)) {
+                ret = _gf_true;
+                if (xdata && dict_get (xdata, CTR_ATTACH_TIER_LOOKUP)) {
+                        ret = _gf_false;
+                }
+        }
+        if (xdata && dict_get (xdata, GLUSTERFS_INTERNAL_FOP_KEY)) {
+                ret = _gf_true;
+        }
+
+        return ret;
+}
 
-/**
- * ignore internal fops for all clients except AFR self-heal daemon
- */
 #define CTR_IF_INTERNAL_FOP_THEN_GOTO(frame, dict, label)\
 do {\
-        GF_ASSERT(frame);\
-        GF_ASSERT(frame->root);\
-        if (CTR_IS_INTERNAL_FOP(frame, dict)) \
+        if (is_internal_fop (frame, dict)) \
                         goto label; \
 } while (0)
 
