@@ -15,6 +15,7 @@
 #include "gfid-access.h"
 #include "inode.h"
 #include "byte-order.h"
+#include "statedump.h"
 
 
 int
@@ -41,7 +42,7 @@ ga_valid_inode_loc_copy (loc_t *dst, loc_t *src, xlator_t *this)
                 }
                 inode_unref (dst->parent);
                 dst->parent = inode_ref ((inode_t*)value);
-                uuid_copy (dst->pargfid, dst->parent->gfid);
+                gf_uuid_copy (dst->pargfid, dst->parent->gfid);
         }
 
         if (dst->inode) {
@@ -52,7 +53,7 @@ ga_valid_inode_loc_copy (loc_t *dst, loc_t *src, xlator_t *this)
                 }
                 inode_unref (dst->inode);
                 dst->inode = inode_ref ((inode_t*)value);
-                uuid_copy (dst->gfid, dst->inode->gfid);
+                gf_uuid_copy (dst->gfid, dst->inode->gfid);
         }
 out:
 
@@ -304,17 +305,19 @@ ga_fill_tmp_loc (loc_t *loc, xlator_t *this, uuid_t gfid,
         ret = inode_ctx_get (loc->inode, this, &value);
         if (!ret) {
                 parent = (void *)value;
-                if (uuid_is_null (parent->gfid))
+                if (gf_uuid_is_null (parent->gfid))
                         parent = loc->inode;
         }
 
         /* parent itself should be looked up */
-        uuid_copy (new_loc->pargfid, parent->gfid);
+        gf_uuid_copy (new_loc->pargfid, parent->gfid);
         new_loc->parent = inode_ref (parent);
 
         new_loc->inode = inode_grep (parent->table, parent, bname);
-        if (!new_loc->inode)
+        if (!new_loc->inode) {
                 new_loc->inode = inode_new (parent->table);
+                gf_uuid_copy (new_loc->inode->gfid, gfid);
+        }
 
         loc_path (new_loc, bname);
         if (new_loc->path) {
@@ -323,12 +326,12 @@ ga_fill_tmp_loc (loc_t *loc, xlator_t *this, uuid_t gfid,
                         new_loc->name++;
         }
 
-        gfid_ptr = GF_CALLOC (1, sizeof(uuid_t), gf_common_mt_char);
+        gfid_ptr = GF_CALLOC (1, sizeof(uuid_t), gf_common_mt_uuid_t);
         if (!gfid_ptr) {
                 ret = -1;
                 goto out;
         }
-        uuid_copy (*gfid_ptr, gfid);
+        gf_uuid_copy (*gfid_ptr, gfid);
         ret = dict_set_dynptr (xdata, "gfid-req", gfid_ptr, sizeof (uuid_t));
         if (ret < 0)
                 goto out;
@@ -351,7 +354,7 @@ __is_gfid_access_dir (uuid_t gfid)
         memset (aux_gfid, 0, 16);
         aux_gfid[15] = GF_AUX_GFID;
 
-        if (uuid_compare (gfid, aux_gfid) == 0)
+        if (gf_uuid_compare (gfid, aux_gfid) == 0)
                 return _gf_true;
 
         return _gf_false;
@@ -416,6 +419,8 @@ ga_newentry_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         STACK_UNWIND_STRICT (setxattr, local->orig_frame, op_ret,
                              op_errno, xdata);
 
+        if (local->xdata)
+                dict_unref (local->xdata);
         loc_wipe (&local->loc);
         mem_put (local);
 
@@ -434,7 +439,8 @@ ga_newentry_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         local = frame->local;
 
-        if (!local->uid && !local->gid)
+        /* no need to proceed if things don't look good here */
+        if (op_ret == -1)
                 goto done;
 
         temp_stat.ia_uid = local->uid;
@@ -456,6 +462,40 @@ done:
         STACK_UNWIND_STRICT (setxattr, local->orig_frame, op_ret,
                              op_errno, xdata);
 
+        if (local->xdata)
+                dict_unref (local->xdata);
+        loc_wipe (&local->loc);
+        mem_put (local);
+
+        return 0;
+}
+
+static int
+ga_newentry_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                        int32_t op_ret, int32_t op_errno, inode_t *inode,
+                        struct iatt *stat, dict_t *xdata,
+                        struct iatt *postparent)
+
+{
+        ga_local_t *local = NULL;
+
+        local = frame->local;
+
+        if ((op_ret < 0) && ((op_errno != ENOENT) && (op_errno != ESTALE)))
+                goto err;
+
+        STACK_WIND (frame, ga_newentry_cbk, FIRST_CHILD(this),
+                    FIRST_CHILD(this)->fops->mknod, &local->loc, local->mode,
+                    local->rdev, local->umask, local->xdata);
+        return 0;
+
+err:
+        frame->local = NULL;
+        STACK_DESTROY (frame->root);
+        STACK_UNWIND_STRICT (setxattr, local->orig_frame, op_ret, op_errno,
+                             xdata);
+        if (local->xdata)
+                dict_unref (local->xdata);
         loc_wipe (&local->loc);
         mem_put (local);
 
@@ -470,7 +510,6 @@ ga_new_entry (call_frame_t *frame, xlator_t *this, loc_t *loc, data_t *data,
         ga_newfile_args_t *args      = NULL;
         loc_t              tmp_loc   = {0,};
         call_frame_t      *new_frame = NULL;
-        mode_t             mode      = 0;
         ga_local_t        *local     = NULL;
         uuid_t             gfid      = {0,};
 
@@ -478,7 +517,7 @@ ga_new_entry (call_frame_t *frame, xlator_t *this, loc_t *loc, data_t *data,
         if (!args)
                 goto out;
 
-        ret = uuid_parse (args->gfid, gfid);
+        ret = gf_uuid_parse (args->gfid, gfid);
         if (ret)
                 goto out;
 
@@ -524,25 +563,31 @@ ga_new_entry (call_frame_t *frame, xlator_t *this, loc_t *loc, data_t *data,
                             &tmp_loc, 0, xdata);
         } else {
                 /* use 07777 (4 7s) for considering the Sticky bits etc) */
-                mode = (S_IFMT & args->st_mode) |
-                        (07777 & args->args.mknod.mode);;
+                ((ga_local_t *)new_frame->local)->mode =
+                     (S_IFMT & args->st_mode) | (07777 & args->args.mknod.mode);
 
-                STACK_WIND (new_frame, ga_newentry_cbk,
-                            FIRST_CHILD(this), FIRST_CHILD(this)->fops->mknod,
-                            &tmp_loc, mode,
-                            args->args.mknod.rdev, args->args.mknod.umask,
-                            xdata);
+                ((ga_local_t *)new_frame->local)->umask =
+                                                         args->args.mknod.umask;
+                ((ga_local_t *)new_frame->local)->rdev  = args->args.mknod.rdev;
+                ((ga_local_t *)new_frame->local)->xdata = dict_ref (xdata);
+
+                /* send a named lookup, so that dht can cleanup up stale linkto
+                 * files etc.
+                 */
+                STACK_WIND (new_frame, ga_newentry_lookup_cbk,
+                            FIRST_CHILD(this), FIRST_CHILD(this)->fops->lookup,
+                            &tmp_loc, NULL);
         }
 
         ret = 0;
 out:
         ga_newfile_args_free (args);
 
-        loc_wipe (&tmp_loc);
-
         if (xdata)
                 dict_unref (xdata);
 
+        loc_wipe (&tmp_loc);
+
         return ret;
 }
 
@@ -560,7 +605,7 @@ ga_heal_entry (call_frame_t *frame, xlator_t *this, loc_t *loc, data_t *data,
         if (!args)
                 goto out;
 
-        ret = uuid_parse (args->gfid, gfid);
+        ret = gf_uuid_parse (args->gfid, gfid);
         if (ret)
                 goto out;
 
@@ -661,18 +706,19 @@ ga_virtual_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                        int32_t op_ret, int32_t op_errno, inode_t *inode,
                        struct iatt *buf, dict_t *xdata, struct iatt *postparent)
 {
-        int       j           = 0;
-        int       i           = 0;
-        int       ret         = 0;
-        uint64_t  temp_ino    = 0;
-        inode_t  *cbk_inode   = NULL;
-        inode_t  *true_inode  = NULL;
-        uuid_t    random_gfid = {0,};
+        int             j                       = 0;
+        int             i                       = 0;
+        int             ret                     = 0;
+        uint64_t        temp_ino                = 0;
+        inode_t         *cbk_inode              = NULL;
+        inode_t         *true_inode             = NULL;
+        uuid_t          random_gfid             = {0,};
+        inode_t         *linked_inode           = NULL;
 
         if (frame->local)
                 cbk_inode = frame->local;
         else
-                cbk_inode = inode;
+                cbk_inode = inode_ref (inode);
 
         frame->local = NULL;
         if (op_ret)
@@ -687,6 +733,10 @@ ga_virtual_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                    if its just previously discover()'d inode */
                 true_inode = inode_find (inode->table, buf->ia_gfid);
                 if (!true_inode) {
+                        /* This unref is for 'inode_ref()' done in beginning.
+                           This is needed as cbk_inode is allocated new inode
+                           whose unref is taken at the end*/
+                        inode_unref (cbk_inode);
                         cbk_inode = inode_new (inode->table);
 
                         if (!cbk_inode) {
@@ -698,7 +748,8 @@ ga_virtual_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                            path is not yet looked up. Use the current inode
                            itself for now */
 
-                        inode_link (inode, NULL, NULL, buf);
+                        linked_inode = inode_link (inode, NULL, NULL, buf);
+                        inode = linked_inode;
                 } else {
                         /* 'inode_ref()' has been done in inode_find() */
                         inode = true_inode;
@@ -715,17 +766,17 @@ ga_virtual_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 inode = NULL;
         }
 
-        if (!uuid_is_null (cbk_inode->gfid)) {
+        if (!gf_uuid_is_null (cbk_inode->gfid)) {
                 /* if the previous linked inode is used, use the
                    same gfid */
-                uuid_copy (random_gfid, cbk_inode->gfid);
+                gf_uuid_copy (random_gfid, cbk_inode->gfid);
         } else {
                 /* replace the buf->ia_gfid to a random gfid
                    for directory, for files, what we received is fine */
-                uuid_generate (random_gfid);
+                gf_uuid_generate (random_gfid);
         }
 
-        uuid_copy (buf->ia_gfid, random_gfid);
+        gf_uuid_copy (buf->ia_gfid, random_gfid);
 
         for (i = 15; i > (15 - 8); i--) {
                 temp_ino += (uint64_t)(buf->ia_gfid[i]) << j;
@@ -742,6 +793,10 @@ unwind:
         STACK_UNWIND_STRICT (lookup, frame, op_ret, op_errno, cbk_inode, buf,
                              xdata, postparent);
 
+        /* Also handles inode_unref of frame->local if done in ga_lookup */
+        if (cbk_inode)
+               inode_unref (cbk_inode);
+
         return 0;
 }
 
@@ -803,7 +858,7 @@ ga_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 
                         /* here, just send 'loc->gfid' and 'loc->inode' */
                         tmp_loc.inode = inode_ref (loc->inode);
-                        uuid_copy (tmp_loc.gfid, loc->inode->gfid);
+                        gf_uuid_copy (tmp_loc.gfid, loc->inode->gfid);
 
                         STACK_WIND (frame, default_lookup_cbk,
                                     FIRST_CHILD(this),
@@ -860,7 +915,7 @@ ga_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 
         /* make sure the 'basename' is actually a 'canonical-gfid',
            otherwise, return error */
-        ret = uuid_parse (loc->name, tmp_gfid);
+        ret = gf_uuid_parse (loc->name, tmp_gfid);
         if (ret)
                 goto err;
 
@@ -887,6 +942,8 @@ ga_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
                 /* time do another lookup and update the context
                    with proper inode */
                 op_errno = ESTALE;
+                /* 'inode_ref()' done in inode_find */
+                inode_unref (true_inode);
                 goto err;
         }
 
@@ -896,12 +953,12 @@ discover:
         if (xdata)
                 dict_del (xdata, "gfid-req");
 
-        uuid_copy (tmp_loc.gfid, tmp_gfid);
+        gf_uuid_copy (tmp_loc.gfid, tmp_gfid);
 
         /* if revalidate, then we need to have the proper reference */
         if (inode) {
                 tmp_loc.inode = inode_ref (inode);
-                frame->local = loc->inode;
+                frame->local = inode_ref (loc->inode);
         } else {
                 tmp_loc.inode = inode_ref (loc->inode);
         }
@@ -1363,6 +1420,25 @@ fini (xlator_t *this)
         return;
 }
 
+int32_t
+ga_dump_inodectx (xlator_t *this, inode_t *inode)
+{
+        int       ret = -1;
+        uint64_t  value = 0;
+        inode_t  *tmp_inode = NULL;
+        char      key_prefix[GF_DUMP_MAX_BUF_LEN] = {0, };
+
+        ret = inode_ctx_get (inode, this, &value);
+        if (ret == 0) {
+                tmp_inode = (void*) value;
+                gf_proc_dump_build_key (key_prefix, this->name, "inode");
+                gf_proc_dump_add_section (key_prefix);
+                gf_proc_dump_write ("real-gfid", "%s",
+                                    uuid_utoa (tmp_inode->gfid));
+        }
+
+        return 0;
+}
 
 struct xlator_fops fops = {
         .lookup = ga_lookup,
@@ -1392,6 +1468,10 @@ struct xlator_cbks cbks = {
         .forget = ga_forget,
 };
 
+struct xlator_dumpops dumpops = {
+        .inodectx       = ga_dump_inodectx,
+};
+
 struct volume_options options[] = {
         /* This translator doesn't take any options, or provide any options */
         { .key  = {NULL} },
