@@ -9,7 +9,7 @@
 */
 
 #ifndef _MARKER_QUOTA_HELPER_H
-#define _MARKER_QUOTA_HELPER
+#define _MARKER_QUOTA_HELPER_H
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
@@ -18,10 +18,14 @@
 
 #include "marker.h"
 
-#define QUOTA_FREE_CONTRIBUTION_NODE(_contribution)     \
-        do {                                            \
-                list_del (&_contribution->contri_list); \
-                GF_FREE (_contribution);                \
+#define QUOTA_FREE_CONTRIBUTION_NODE(ctx, _contribution)             \
+        do {                                                         \
+                LOCK (&ctx->lock);                                   \
+                {                                                    \
+                        list_del_init (&_contribution->contri_list); \
+                        GF_REF_PUT (_contribution);                  \
+                }                                                    \
+                UNLOCK (&ctx->lock);                                 \
         } while (0)
 
 #define QUOTA_SAFE_INCREMENT(lock, var)                 \
@@ -44,7 +48,7 @@ inode_contribution_t *
 mq_add_new_contribution_node (xlator_t *, quota_inode_ctx_t *, loc_t *);
 
 int32_t
-mq_dict_set_contribution (xlator_t *, dict_t *, loc_t *);
+mq_dict_set_contribution (xlator_t *, dict_t *, loc_t *, uuid_t, char *);
 
 quota_inode_ctx_t *
 mq_inode_ctx_new (inode_t *, xlator_t *);
@@ -67,6 +71,12 @@ mq_local_ref (quota_local_t *);
 int32_t
 mq_local_unref (xlator_t *, quota_local_t *);
 
+void
+mq_contri_fini (void *data);
+
+inode_contribution_t*
+mq_contri_init (inode_t *inode);
+
 inode_contribution_t *
 mq_get_contribution_node (inode_t *, quota_inode_ctx_t *);
 
