@@ -18,12 +18,13 @@
 #include "xlator.h"
 #include "marker-mem-types.h"
 #include "refcount.h"
+#include "quota-common-utils.h"
 
 #define QUOTA_XATTR_PREFIX "trusted.glusterfs"
 #define QUOTA_DIRTY_KEY "trusted.glusterfs.quota.dirty"
 
 #define CONTRIBUTION  "contri"
-#define CONTRI_KEY_MAX 512
+#define QUOTA_KEY_MAX 512
 #define READDIR_BUF 4096
 
 
@@ -60,22 +61,40 @@
                 ret = 0;                                \
         } while (0);
 
-#define GET_CONTRI_KEY(var, _gfid, _ret)                                  \
+#define GET_CONTRI_KEY(_this, var, _gfid, _ret)                           \
         do {                                                              \
+                marker_conf_t  *_priv = _this->private;                   \
                 if (_gfid != NULL) {                                      \
                         char _gfid_unparsed[40];                          \
                         gf_uuid_unparse (_gfid, _gfid_unparsed);          \
-                        _ret = snprintf (var, CONTRI_KEY_MAX,             \
+                        _ret = snprintf (var, QUOTA_KEY_MAX,              \
                                          QUOTA_XATTR_PREFIX               \
-                                         ".%s.%s." CONTRIBUTION, "quota", \
-                                         _gfid_unparsed);                 \
+                                         ".%s.%s." CONTRIBUTION ".%d",    \
+                                         "quota", _gfid_unparsed,         \
+                                         _priv->version);                 \
                 } else {                                                  \
-                        _ret = snprintf (var, CONTRI_KEY_MAX,             \
+                        _ret = snprintf (var, QUOTA_KEY_MAX,              \
                                          QUOTA_XATTR_PREFIX               \
-                                         ".%s.." CONTRIBUTION, "quota");  \
+                                         ".%s.." CONTRIBUTION ".%d",      \
+                                         "quota", _priv->version);        \
                 }                                                         \
         } while (0)
 
+#define GET_QUOTA_KEY(_this, var, key, _ret)                              \
+        do {                                                              \
+                marker_conf_t  *_priv = _this->private;                   \
+                if (_priv->version > 0)                                   \
+                        _ret = snprintf (var, QUOTA_KEY_MAX, "%s.%d",     \
+                                         key, _priv->version);            \
+                else                                                      \
+                        _ret = snprintf (var, QUOTA_KEY_MAX, "%s", key);  \
+        } while (0)
+
+#define GET_SIZE_KEY(_this, var, _ret)                                    \
+        {                                                                 \
+                GET_QUOTA_KEY (_this, var, QUOTA_SIZE_KEY, _ret);         \
+        }
+
 #define QUOTA_SAFE_INCREMENT(lock, var)         \
         do {                                    \
                 LOCK (lock);                    \
@@ -90,6 +109,7 @@ struct quota_inode_ctx {
         int8_t                 dirty;
         gf_boolean_t           create_status;
         gf_boolean_t           updation_status;
+        gf_boolean_t           dirty_status;
         gf_lock_t              lock;
         struct list_head       contribution_head;
 };
@@ -98,7 +118,7 @@ typedef struct quota_inode_ctx quota_inode_ctx_t;
 struct quota_synctask {
         xlator_t      *this;
         loc_t          loc;
-        int64_t        contri;
+        quota_meta_t   contri;
         gf_boolean_t   is_static;
 };
 typedef struct quota_synctask quota_synctask_t;
@@ -115,41 +135,22 @@ struct inode_contribution {
 typedef struct inode_contribution inode_contribution_t;
 
 int32_t
-mq_get_lock_on_parent (call_frame_t *, xlator_t *);
-
-int32_t
-mq_req_xattr (xlator_t *, loc_t *, dict_t *, char *);
-
-int32_t
-init_quota_priv (xlator_t *);
+mq_req_xattr (xlator_t *, loc_t *, dict_t *, char *, char *);
 
 int32_t
 mq_xattr_state (xlator_t *, loc_t *, dict_t *, struct iatt);
 
-int32_t
-mq_set_inode_xattr (xlator_t *, loc_t *);
+int
+mq_initiate_quota_txn (xlator_t *, loc_t *, struct iatt *);
 
 int
-mq_initiate_quota_txn (xlator_t *, loc_t *);
+mq_initiate_quota_blocking_txn (xlator_t *, loc_t *, struct iatt *);
 
 int
-mq_initiate_quota_blocking_txn (xlator_t *, loc_t *);
-
-int
-mq_create_xattrs_txn (xlator_t *this, loc_t *loc);
+mq_create_xattrs_txn (xlator_t *this, loc_t *loc, struct iatt *buf);
 
 int32_t
-mq_dirty_inode_readdir (call_frame_t *, void *, xlator_t *,
-                        int32_t, int32_t, fd_t *, dict_t *);
-
-int32_t
-mq_reduce_parent_size (xlator_t *, loc_t *, int64_t);
-
-int32_t
-mq_reduce_parent_size_txn (xlator_t *, loc_t *, int64_t);
-
-int32_t
-mq_rename_update_newpath (xlator_t *, loc_t *);
+mq_reduce_parent_size_txn (xlator_t *, loc_t *, quota_meta_t *);
 
 int32_t
 mq_forget (xlator_t *, quota_inode_ctx_t *);
