@@ -141,7 +141,7 @@ qb_format_extract (xlator_t *this, char *format, inode_t *inode)
 			goto invalid;
 		ret = sscanf(s, "<gfid:%[^>]s", gfid_str);
 		if (ret == 1) {
-			ret = uuid_parse(gfid_str, gfid);
+			ret = gf_uuid_parse(gfid_str, gfid);
 			if (ret < 0)
 				goto invalid;
 		}
@@ -164,8 +164,8 @@ qb_format_extract (xlator_t *this, char *format, inode_t *inode)
 	 * associated with a backing image as a filename local to the parent
 	 * directory. The format processing will validate further.
 	 */
-	if (!uuid_is_null(gfid))
-		uuid_copy(qb_inode->backing_gfid, gfid);
+	if (!gf_uuid_is_null(gfid))
+		gf_uuid_copy(qb_inode->backing_gfid, gfid);
 	else if (s)
 		qb_inode->backing_fname = gf_strdup(s);
 
@@ -316,7 +316,7 @@ qb_setxattr_format (call_frame_t *frame, xlator_t *this, call_stub_t *stub,
 	qb_local->stub = stub;
 	qb_local->inode = inode_ref (inode);
 
-	snprintf(qb_local->fmt, QB_XATTR_VAL_MAX, "%s:%lu", qb_inode->fmt,
+	snprintf(qb_local->fmt, QB_XATTR_VAL_MAX, "%s:%" PRId64, qb_inode->fmt,
 		 qb_inode->size);
 
 	qb_coroutine (frame, qb_format_and_resume);
@@ -415,24 +415,23 @@ int
 qb_setxattr_common (call_frame_t *frame, xlator_t *this, call_stub_t *stub,
 		    dict_t *xattr, inode_t *inode)
 {
-	data_t *data = NULL;
 
-	if ((data = dict_get (xattr, "trusted.glusterfs.block-format"))) {
+	if (dict_get (xattr, "trusted.glusterfs.block-format")) {
 		qb_setxattr_format (frame, this, stub, xattr, inode);
 		return 0;
 	}
 
-	if ((data = dict_get (xattr, "trusted.glusterfs.block-snapshot-create"))) {
+	if (dict_get (xattr, "trusted.glusterfs.block-snapshot-create")) {
 		qb_setxattr_snapshot_create (frame, this, stub, xattr, inode);
 		return 0;
 	}
 
-	if ((data = dict_get (xattr, "trusted.glusterfs.block-snapshot-delete"))) {
+	if (dict_get (xattr, "trusted.glusterfs.block-snapshot-delete")) {
 		qb_setxattr_snapshot_delete (frame, this, stub, xattr, inode);
 		return 0;
 	}
 
-	if ((data = dict_get (xattr, "trusted.glusterfs.block-snapshot-goto"))) {
+	if (dict_get (xattr, "trusted.glusterfs.block-snapshot-goto")) {
 		qb_setxattr_snapshot_goto (frame, this, stub, xattr, inode);
 		return 0;
 	}
@@ -1089,8 +1088,8 @@ fini (xlator_t *this)
 
         this->private = NULL;
 
-	if (conf->root_inode)
-		inode_unref(conf->root_inode);
+        /* No need to do inode_unref of conf->root_inode as ref/unref doesn't
+         * apply for it*/
         GF_FREE (conf);
 
 	return;
