@@ -263,7 +263,7 @@ out:
 }
 
 
-int32_t
+static int32_t
 svc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 int32_t op_ret, int32_t op_errno, inode_t *inode,
                 struct iatt *buf, dict_t *xdata, struct iatt *postparent)
@@ -378,7 +378,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
             dict_t *xdata)
 {
@@ -498,7 +498,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_statfs (call_frame_t *frame, xlator_t *this, loc_t *loc,
             dict_t *xdata)
 {
@@ -539,14 +539,15 @@ svc_statfs (call_frame_t *frame, xlator_t *this, loc_t *loc,
                         root_loc.path = gf_strdup("/");
                         gf_uuid_clear(root_loc.gfid);
                         root_loc.gfid[15] = 1;
-                        root_loc.inode = loc->inode->table->root;
-                        root_loc.inode->ia_type = IA_IFDIR;
+                        root_loc.inode = inode_ref (loc->inode->table->root);
                         temp_loc = &root_loc;
                 }
         }
 
         STACK_WIND_TAIL (frame, subvolume, subvolume->fops->statfs,
                          temp_loc, xdata);
+        if (temp_loc == &root_loc)
+                loc_wipe (temp_loc);
 
         wind = _gf_true;
 out:
@@ -560,7 +561,7 @@ out:
    handled? i.e just based on inode type decide where the call should
    be sent and in the call back update the contexts.
 */
-int32_t
+static int32_t
 svc_stat (call_frame_t *frame, xlator_t *this, loc_t *loc,
           dict_t *xdata)
 {
@@ -590,7 +591,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
 {
         int32_t      ret        = -1;
@@ -619,7 +620,7 @@ out:
         return ret;
 }
 
-int32_t
+static int32_t
 svc_opendir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, fd_t *fd, dict_t *xdata)
 {
@@ -686,7 +687,7 @@ out:
    will not be any corresponding fd opened on svs and
    svc has to do things that open-behind is doing.
 */
-int32_t
+static int32_t
 svc_opendir (call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
              dict_t *xdata)
 {
@@ -732,7 +733,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
              struct iatt *stbuf, int32_t valid, dict_t *xdata)
 {
@@ -776,6 +777,7 @@ out:
         return 0;
 }
 
+/* XXX: This function is currently not used. Mark it 'static' when required */
 int32_t
 svc_fsetattr (call_frame_t *frame, xlator_t *this, fd_t *fd, struct iatt *stbuf,
              int32_t valid, dict_t *xdata)
@@ -819,7 +821,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, const char *name,
               dict_t *xdata)
 {
@@ -899,6 +901,7 @@ out:
         return 0;
 }
 
+/* XXX: This function is currently not used. Mark it 'static' when required */
 int32_t
 svc_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, const char *name,
               dict_t *xdata)
@@ -930,7 +933,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *dict,
               int32_t flags, dict_t *xdata)
 {
@@ -975,7 +978,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *dict,
               int32_t flags, dict_t *xdata)
 {
@@ -1019,7 +1022,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
            dict_t *xdata)
 {
@@ -1063,7 +1066,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, inode_t *inode,
                struct iatt *buf, struct iatt *preparent,
@@ -1088,7 +1091,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
            mode_t umask, dict_t *xdata)
 {
@@ -1135,7 +1138,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno, inode_t *inode,
                struct iatt *buf, struct iatt *preparent,
@@ -1159,7 +1162,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_mknod (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
            dev_t rdev, mode_t umask, dict_t *xdata)
 {
@@ -1210,7 +1213,7 @@ out:
    a virtual inode, then unwind the call back with EROFS. Otherwise simply
    STACK_WIND the call to the first child of svc xlator.
 */
-int32_t
+static int32_t
 svc_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
           fd_t *fd, dict_t *xdata)
 {
@@ -1255,7 +1258,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 int32_t op_ret, int32_t op_errno, fd_t *fd, inode_t *inode,
                 struct iatt *stbuf, struct iatt *preparent,
@@ -1280,7 +1283,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_create (call_frame_t *frame, xlator_t *this,
             loc_t *loc, int32_t flags, mode_t mode,
             mode_t umask, fd_t *fd, dict_t *xdata)
@@ -1329,7 +1332,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                      int32_t op_ret, int32_t op_errno, inode_t *inode,
                      struct iatt *buf, struct iatt *preparent,
@@ -1354,7 +1357,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_symlink (call_frame_t *frame, xlator_t *this, const char *linkpath,
              loc_t *loc, mode_t umask, dict_t *xdata)
 {
@@ -1401,7 +1404,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
             dict_t *xdata)
 {
@@ -1444,7 +1447,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
            off_t offset, uint32_t flags, dict_t *xdata)
 {
@@ -1475,7 +1478,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_readlink (call_frame_t *frame, xlator_t *this,
               loc_t *loc, size_t size, dict_t *xdata)
 {
@@ -1506,7 +1509,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_access (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t mask,
             dict_t *xdata)
 {
@@ -1537,7 +1540,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd,
              size_t size, off_t off,
              dict_t *xdata)
@@ -1613,7 +1616,7 @@ out:
  * windows.
  */
 
-int32_t
+static int32_t
 svc_readdirp_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                          int32_t op_ret, int32_t op_errno, inode_t *inode,
                          struct iatt *buf, dict_t *xdata,
@@ -1786,7 +1789,7 @@ out:
         return unwind;
 }
 
-int32_t
+static int32_t
 svc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno,
                   gf_dirent_t *entries, dict_t *xdata)
@@ -1846,7 +1849,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd,
               size_t size, off_t off,
               dict_t *xdata)
@@ -1922,7 +1925,7 @@ out:
 /* Renaming the entries from or to snapshots is not allowed as the snapshots
    are read-only.
 */
-int32_t
+static int32_t
 svc_rename (call_frame_t *frame, xlator_t *this, loc_t *oldloc,
             loc_t *newloc, dict_t *xdata)
 {
@@ -2000,7 +2003,7 @@ out:
    will be equivalent of creating hardlinks across different filesystems.
    And so is vise versa.
 */
-int32_t
+static int32_t
 svc_link (call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
           dict_t *xdata)
 {
@@ -2048,7 +2051,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_removexattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
                  const char *name, dict_t *xdata)
 {
@@ -2093,7 +2096,7 @@ out:
         return 0;
 }
 
-int
+static int
 svc_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int datasync,
            dict_t *xdata)
 {
@@ -2137,7 +2140,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_flush (call_frame_t *frame, xlator_t *this,
            fd_t *fd, dict_t *xdata)
 {
@@ -2167,7 +2170,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_releasedir (xlator_t *this, fd_t *fd)
 {
         svc_fd_t *sfd      = NULL;
@@ -2190,7 +2193,7 @@ out:
         return 0;
 }
 
-int32_t
+static int32_t
 svc_forget (xlator_t *this, inode_t *inode)
 {
         int            ret      = -1;
