@@ -293,7 +293,7 @@ svs_lookup_entry (xlator_t *this, loc_t *loc, struct iatt *buf,
         fs = parent_ctx->fs;
 
         object = glfs_h_lookupat (fs, parent_object, loc->name,
-                                  &statbuf);
+                                  &statbuf, 0);
         if (!object) {
                 gf_log (this->name, GF_LOG_DEBUG, "failed to do lookup and "
                         "get the handle for entry %s (path: %s)", loc->name,
@@ -1246,7 +1246,6 @@ svs_glfs_readdir (xlator_t *this, glfs_fd_t *glfd, gf_dirent_t *entries,
         GF_VALIDATE_OR_GOTO ("svs", this, out);
         GF_VALIDATE_OR_GOTO (this->name, glfd, out);
         GF_VALIDATE_OR_GOTO (this->name, entries, out);
-        GF_VALIDATE_OR_GOTO (this->name, buf, out);
 
         while (filled_size < size) {
                 in_case = glfs_telldir (glfd);
@@ -1287,9 +1286,10 @@ svs_glfs_readdir (xlator_t *this, glfs_fd_t *glfd, gf_dirent_t *entries,
                         entry->d_off = glfs_telldir (glfd);
                         entry->d_ino = de.d_ino;
                         entry->d_type = de.d_type;
-                        iatt_from_stat (buf, &statbuf);
-                        if (readdirplus)
+                        if (readdirplus) {
+                                iatt_from_stat (buf, &statbuf);
                                 entry->d_stat = *buf;
+                        }
                         list_add_tail (&entry->list, &entries->list);
 
                         filled_size += this_size;
