@@ -1,22 +1,12 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
-  This file is part of GlusterFS.
+   Copyright (c) 2006-2012 Red Hat, Inc. <http://www.redhat.com>
+   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+   This file is licensed to you under your choice of the GNU Lesser
+   General Public License, version 3 or any later version (LGPLv3 or
+   later), or the GNU General Public License, version 2 (GPLv2), in all
+   cases as published by the Free Software Foundation.
 */
-
 #ifndef __TRASH_H__
 #define __TRASH_H__
 
@@ -47,7 +37,6 @@
 #define GF_ALLOWED_MAX_FILE_SIZE (1 * GF_UNIT_GB)
 #endif
 
-
 struct trash_struct {
         fd_t    *fd;         /* for the fd of existing file */
         fd_t    *newfd;      /* for the newly created file */
@@ -61,29 +50,35 @@ struct trash_struct {
         int32_t  loop_count;
         struct iatt preparent;
         struct iatt postparent;
+        gf_boolean_t ctr_link_count_req;
 };
 typedef struct trash_struct trash_local_t;
 
-struct _trash_elim_pattern;
-typedef struct _trash_elim_pattern {
-        struct _trash_elim_pattern *next;
-        char                       *pattern;
-} trash_elim_pattern_t;
+struct _trash_elim_path {
+        struct _trash_elim_path    *next;
+        char                       *path;
+};
+typedef struct _trash_elim_path trash_elim_path;
 
 struct trash_priv {
-        char                 *trash_dir;
-        trash_elim_pattern_t *eliminate;
+        char                 *oldtrash_dir;
+        char                 *newtrash_dir;
+        char                 *brick_path;
+        trash_elim_path      *eliminate;
         size_t                max_trash_file_size;
+        gf_boolean_t          state;
+        gf_boolean_t          internal;
+        inode_t              *trash_inode;
+        inode_table_t        *trash_itable;
 };
 typedef struct trash_priv trash_private_t;
 
-#define TRASH_STACK_UNWIND(op, frame, params ...) do {     \
-		trash_local_t *__local = NULL;         \
-		__local = frame->local;                \
-		frame->local = NULL;		       \
-		STACK_UNWIND_STRICT (op, frame, params);          \
-		trash_local_wipe (__local);	       \
-	} while (0)
-
+#define TRASH_STACK_UNWIND(op, frame, params ...) do {    \
+                trash_local_t *__local = NULL;            \
+                __local = frame->local;                   \
+                frame->local = NULL;                      \
+                STACK_UNWIND_STRICT (op, frame, params);  \
+                trash_local_wipe (__local);               \
+        } while (0)
 
 #endif /* __TRASH_H__ */
