@@ -15,12 +15,9 @@
 #include "config.h"
 #endif
 
-#ifndef GSYNC_CONF_TEMPLATE
-#define GSYNC_CONF_TEMPLATE GEOREP"/gsyncd_template.conf"
-#endif
 
 #include <pthread.h>
-#include "uuid.h"
+#include "compat-uuid.h"
 
 #include "glusterfs.h"
 #include "xlator.h"
@@ -31,7 +28,6 @@
 #include "glusterd.h"
 #include "protocol-common.h"
 
-#define GD_VOLUME_NAME_MAX 256
 #define GD_OP_PROTECTED    (0x02)
 #define GD_OP_UNPROTECTED  (0x04)
 
@@ -74,9 +70,10 @@ typedef enum glusterd_op_sm_event_type_ {
 
 
 struct glusterd_op_sm_event_ {
-        struct list_head                list;
+        struct cds_list_head            list;
         void                            *ctx;
         glusterd_op_sm_event_type_t     event;
+        uuid_t                          txn_id;
 };
 
 typedef struct glusterd_op_sm_event_ glusterd_op_sm_event_t;
@@ -99,13 +96,14 @@ struct glusterd_op_info_ {
         int32_t                         brick_pending_count;
         int32_t                         op_count;
         glusterd_op_t                   op;
-        struct list_head                op_peers;
+        struct cds_list_head            op_peers;
         void                            *op_ctx;
         rpcsvc_request_t                *req;
         int32_t                         op_ret;
         int32_t                         op_errno;
         char                            *op_errstr;
-        struct  list_head               pending_bricks;
+        struct  cds_list_head           pending_bricks;
+        uint32_t                        txn_generation;
 };
 
 typedef struct glusterd_op_info_ glusterd_op_info_t;
@@ -119,6 +117,7 @@ typedef struct glusterd_op_log_filename_ctx_ glusterd_op_log_filename_ctx_t;
 
 struct glusterd_op_lock_ctx_ {
         uuid_t                  uuid;
+        dict_t                 *dict;
         rpcsvc_request_t        *req;
 };
 
@@ -159,28 +158,26 @@ typedef struct glusterd_status_rsp_conv_ {
         dict_t *dict;
 } glusterd_status_rsp_conv_t;
 
-typedef struct glusterd_gsync_status_temp {
-        dict_t *rsp_dict;
-        glusterd_volinfo_t *volinfo;
-        char *node;
-}glusterd_gsync_status_temp_t;
 
-typedef struct gsync_status_param {
-        int is_active;
-        glusterd_volinfo_t *volinfo;
-}gsync_status_param_t;
+typedef struct glusterd_txn_opinfo_object_ {
+        glusterd_op_info_t    opinfo;
+} glusterd_txn_opinfo_obj;
 
 typedef enum cli_cmd_type_ {
-        PER_REPLICA,
-        ALL_REPLICA,
+        PER_HEAL_XL,
+        ALL_HEAL_XL,
  } cli_cmd_type;
 
+typedef struct glusterd_all_volume_options {
+        char          *option;
+} glusterd_all_vol_opts;
+
 int
 glusterd_op_sm_new_event (glusterd_op_sm_event_type_t event_type,
                           glusterd_op_sm_event_t **new_event);
 int
 glusterd_op_sm_inject_event (glusterd_op_sm_event_type_t event_type,
-                             void *ctx);
+                             uuid_t *txn_id, void *ctx);
 
 int
 glusterd_op_sm_init ();
@@ -237,9 +234,6 @@ glusterd_check_option_exists(char *optstring, char **completion);
 int
 set_xlator_option (dict_t *dict, char *key, char *value);
 
-void
-glusterd_do_replace_brick (void *data);
-
 char*
 glusterd_op_sm_state_name_get (int state);
 
@@ -247,7 +241,7 @@ char*
 glusterd_op_sm_event_name_get (int event);
 int32_t
 glusterd_op_bricks_select (glusterd_op_t op, dict_t *dict, char **op_errstr,
-                           struct list_head *selected, dict_t *rsp_dict);
+                           struct cds_list_head *selected, dict_t *rsp_dict);
 int
 glusterd_brick_op_build_payload (glusterd_op_t op, glusterd_brickinfo_t *brickinfo,
                                  gd1_mgmt_brick_op_req **req, dict_t *dict);
@@ -264,10 +258,7 @@ glusterd_op_init_commit_rsp_dict (glusterd_op_t op);
 
 void
 glusterd_op_modify_op_ctx (glusterd_op_t op, void *op_ctx);
-int32_t
-glusterd_op_init_ctx (glusterd_op_t op);
-int32_t
-glusterd_op_fini_ctx ();
+
 int32_t
 glusterd_volume_stats_read_perf (char *brick_path, int32_t blk_size,
                 int32_t blk_count, double *throughput, double *time);
@@ -282,12 +273,6 @@ gf_boolean_t
 glusterd_are_all_volumes_stopped ();
 int
 glusterd_stop_bricks (glusterd_volinfo_t *volinfo);
-int
-gsync_status (char *master, char *slave, char *conf_path, int *status);
-
-int
-glusterd_check_gsync_running (glusterd_volinfo_t *volinfo, gf_boolean_t *flag);
-
 int
 glusterd_defrag_volume_node_rsp (dict_t *req_dict, dict_t *rsp_dict,
                                  dict_t *op_ctx);
@@ -296,4 +281,21 @@ int
 glusterd_is_valid_vg (glusterd_brickinfo_t *brick, int check_tag, char *msg);
 #endif
 
+int32_t
+glusterd_get_txn_opinfo (uuid_t *txn_id, glusterd_op_info_t  *opinfo);
+
+int32_t
+glusterd_set_txn_opinfo (uuid_t *txn_id, glusterd_op_info_t  *opinfo);
+
+int32_t
+glusterd_clear_txn_opinfo (uuid_t *txn_id);
+
+int32_t
+glusterd_generate_txn_id (dict_t *dict, uuid_t **txn_id);
+
+void
+glusterd_set_opinfo (char *errstr, int32_t op_errno, int32_t op_ret);
+
+int
+glusterd_dict_set_volid (dict_t *dict, char *volname, char **op_errstr);
 #endif
