@@ -140,7 +140,8 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
                 UNLOCK (&defrag->lock);
 
                 if (!gf_is_service_running (pidfile, NULL)) {
-                        if (volinfo->rebal.defrag_status ==
+                        if (volinfo->type != GF_CLUSTER_TYPE_TIER &&
+                            volinfo->rebal.defrag_status ==
                                                 GF_DEFRAG_STATUS_STARTED) {
                                 volinfo->rebal.defrag_status =
                                                    GF_DEFRAG_STATUS_FAILED;
@@ -197,15 +198,13 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         char                   logfile[PATH_MAX] = {0,};
         char                   volname[PATH_MAX] = {0,};
         char                   valgrind_logfile[PATH_MAX] = {0,};
+        char                   *volfileserver = NULL;
 
         priv    = THIS->private;
 
         GF_ASSERT (volinfo);
         GF_ASSERT (op_errstr);
 
-        if ((cmd == GF_OP_CMD_DETACH_START) &&
-            (volinfo->rebal.defrag_status == GF_DEFRAG_STATUS_STARTED))
-                return 0;
 
         ret = glusterd_defrag_start_validate (volinfo, op_errstr, len, op);
         if (ret)
@@ -260,8 +259,26 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         }
 
         snprintf (volname, sizeof(volname), "rebalance/%s", volinfo->volname);
+
+        if (dict_get_str (THIS->options, "transport.socket.bind-address",
+                          &volfileserver) == 0) {
+               /*In the case of running multiple glusterds on a single machine,
+                *we should ensure that log file and unix socket file shouls be
+                *unique in given cluster */
+
+                GLUSTERD_GET_DEFRAG_SOCK_FILE_OLD (sockfile, volinfo,
+                                                   priv);
+                snprintf (logfile, PATH_MAX, "%s/%s-%s-%s.log",
+                          DEFAULT_LOG_FILE_DIRECTORY, volinfo->volname,
+                          (cmd == GF_DEFRAG_CMD_START_TIER ? "tier":"rebalance"),
+                          uuid_utoa(MY_UUID));
+
+        } else {
+                volfileserver = "localhost";
+        }
+
         runner_add_args (&runner, SBIN_DIR"/glusterfs",
-                         "-s", "localhost", "--volfile-id", volname,
+                         "-s", volfileserver, "--volfile-id", volname,
                          "--xlator-option", "*dht.use-readdirp=yes",
                          "--xlator-option", "*dht.lookup-unhashed=yes",
                          "--xlator-option", "*dht.assert-no-child-down=yes",
@@ -275,7 +292,7 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
         if (volinfo->type == GF_CLUSTER_TYPE_TIER) {
                 runner_add_arg (&runner, "--xlator-option");
                 runner_argprintf (&runner,
-                                  "*tier-dht.xattr-name=trusted.tier-gfid");
+                                  "*tier-dht.xattr-name=trusted.tier.tier-dht");
         }
 
         runner_add_arg (&runner, "--xlator-option");
@@ -617,6 +634,13 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                         ret = -1;
                         goto out;
                 }
+                if (glusterd_is_tier_daemon_running (volinfo)) {
+                        ret = gf_asprintf (op_errstr, "A Tier daemon is "
+                                           "already running on volume %s",
+                                           volname);
+                        ret = -1;
+                        goto out;
+                }
         case GF_DEFRAG_CMD_START:
         case GF_DEFRAG_CMD_START_LAYOUT_FIX:
                 /* Check if the connected clients are all of version
@@ -716,11 +740,19 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                                 goto out;
                         }
                 }
-                if ((strstr(cmd_str, "detach-tier") != NULL) &&
-                    (volinfo->rebal.op != GD_OP_REMOVE_BRICK)) {
-                        snprintf (msg, sizeof(msg), "Detach-tier not started.");
-                        ret = -1;
-                        goto out;
+                if (strstr(cmd_str, "detach-tier") != NULL) {
+                        if (volinfo->type != GF_CLUSTER_TYPE_TIER) {
+                                snprintf (msg, sizeof(msg), "volume %s is not "
+                                          "a tier volume.", volinfo->volname);
+                                ret = -1;
+                                goto out;
+                        }
+                        if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
+                                snprintf (msg, sizeof(msg), "Detach-tier"
+                                          "not started");
+                                ret = -1;
+                                goto out;
+                        }
                 }
 
                 break;
