@@ -225,6 +225,10 @@ glusterd_brick_stop (glusterd_volinfo_t *volinfo,
                      glusterd_brickinfo_t *brickinfo,
                      gf_boolean_t del_brick);
 
+gf_boolean_t
+glusterd_is_tier_daemon_running (glusterd_volinfo_t *volinfo);
+
+
 int
 glusterd_is_defrag_on (glusterd_volinfo_t *volinfo);
 
@@ -362,6 +366,18 @@ glusterd_get_trusted_client_filepath (char *filepath,
 int
 glusterd_restart_rebalance (glusterd_conf_t *conf);
 
+int32_t
+glusterd_create_sub_tier_volinfo (glusterd_volinfo_t *volinfo,
+                                   glusterd_volinfo_t **dup_volinfo,
+                                   gf_boolean_t is_hot_tier,
+                                   const char *new_name);
+void
+glusterd_restart_rebalance_for_volume (glusterd_volinfo_t *volinfo);
+
+void
+glusterd_defrag_info_set (glusterd_volinfo_t *volinfo, dict_t *dict, int cmd,
+                          int status, int op);
+
 int32_t
 glusterd_add_bricks_hname_path_to_dict (dict_t *dict,
                                         glusterd_volinfo_t *volinfo);
@@ -632,6 +648,8 @@ glusterd_import_quota_conf (dict_t *peer_data, int vol_idx,
 gf_boolean_t
 glusterd_is_shd_compatible_volume (glusterd_volinfo_t *volinfo);
 
+gf_boolean_t
+glusterd_is_shd_compatible_type (int type);
 
 gf_boolean_t
 glusterd_are_all_volumes_stopped ();
