@@ -28,6 +28,7 @@
 #include "defaults.h"
 #include "list.h"
 #include "dict.h"
+#include "options.h"
 #include "compat.h"
 #include "compat-errno.h"
 #include "syscall.h"
@@ -104,6 +105,8 @@ const char *gd_op_list[GD_OP_MAX + 1] = {
         [GD_OP_STOP_VOLUME]             = "Stop",
         [GD_OP_DEFRAG_VOLUME]           = "Rebalance",
         [GD_OP_ADD_BRICK]               = "Add brick",
+        [GD_OP_DETACH_TIER]             = "Detach tier",
+        [GD_OP_TIER_MIGRATE]            = "Tier migration",
         [GD_OP_REMOVE_BRICK]            = "Remove brick",
         [GD_OP_REPLACE_BRICK]           = "Replace brick",
         [GD_OP_SET_VOLUME]              = "Set",
@@ -1327,6 +1330,7 @@ init (xlator_t *this)
         char              *valgrind_str               = NULL;
         char              *transport_type             = NULL;
         char               var_run_dir[PATH_MAX]      = {0,};
+        int32_t            workers                    = 0;
 
 #ifndef GF_DARWIN_HOST_OS
         {
@@ -1775,6 +1779,15 @@ init (xlator_t *this)
         if (ret)
                 goto out;
 
+        GF_OPTION_INIT ("event-threads", workers, int32, out);
+        if (workers > 0 && workers != conf->workers) {
+                conf->workers = workers;
+                ret = event_reconfigure_threads (this->ctx->event_pool,
+                                                 workers);
+                if (ret)
+                        goto out;
+        }
+
         ret = 0;
 out:
         if (ret < 0) {
@@ -1953,5 +1966,15 @@ struct volume_options options[] = {
           .max  = 300,
           .default_value = TOSTRING(RPC_DEFAULT_PING_TIMEOUT),
         },
+        { .key   = {"event-threads"},
+          .type  = GF_OPTION_TYPE_INT,
+          .min   = 1,
+          .max   = 32,
+          .default_value = "2",
+          .description = "Specifies the number of event threads to execute "
+                         "in parallel. Larger values would help process"
+                         " responses faster, depending on available processing"
+                         " power. Range 1-32 threads."
+        },
         { .key   = {NULL} },
 };
