@@ -82,6 +82,9 @@ typedef struct glusterd_volinfo_ glusterd_volinfo_t;
 struct glusterd_snap_;
 typedef struct glusterd_snap_ glusterd_snap_t;
 
+/* For every new feature please add respective enum of new feature
+ * at the end of latest enum (just before the GD_OP_MAX enum)
+ */
 typedef enum glusterd_op_ {
         GD_OP_NONE = 0,
         GD_OP_CREATE_VOLUME,
@@ -115,6 +118,8 @@ typedef enum glusterd_op_ {
         GD_OP_BARRIER,
         GD_OP_GANESHA,
         GD_OP_BITROT,
+        GD_OP_DETACH_TIER,
+        GD_OP_TIER_MIGRATE,
         GD_OP_MAX,
 } glusterd_op_t;
 
@@ -175,6 +180,7 @@ typedef struct {
         struct cds_list_head       missed_snaps_list;
         int                        ping_timeout;
         uint32_t                   generation;
+        int32_t                    workers;
 } glusterd_conf_t;
 
 
@@ -318,6 +324,7 @@ typedef struct tier_info_ {
         int                       hot_replica_count;
         int                       promoted;
         int                       demoted;
+        uint16_t                  cur_tier_hot;
 } gd_tier_info_t;
 
 struct glusterd_volinfo_ {
@@ -333,7 +340,12 @@ struct glusterd_volinfo_ {
                                             the volume which is snapped. In
                                             case of a non-snap volume, this
                                             field will be initialized as N/A */
-        char                      volname[GD_VOLUME_NAME_MAX];
+        char                      volname[GD_VOLUME_NAME_MAX + 5];
+                                        /* An extra 5 bytes are allocated.
+                                         * Reason is, As part of the tiering
+                                         * volfile generation code, we are
+                                         * temporarily appending either hot
+                                         * or cold */
         int                       type;
         int                       brick_count;
         uint64_t                  snap_count;
@@ -402,6 +414,7 @@ struct glusterd_volinfo_ {
         gd_quorum_status_t        quorum_status;
 
         glusterd_snapdsvc_t       snapd;
+        int32_t                   quota_version;
 };
 
 typedef enum gd_snap_status_ {
@@ -1110,7 +1123,8 @@ glusterd_build_snap_device_path (char *device, char *snapname,
 
 int32_t
 glusterd_snap_remove (dict_t *rsp_dict, glusterd_snap_t *snap,
-                      gf_boolean_t remove_lvm, gf_boolean_t force);
+                      gf_boolean_t remove_lvm, gf_boolean_t force,
+                      gf_boolean_t is_clone);
 int32_t
 glusterd_snapshot_cleanup (dict_t *dict, char **op_errstr, dict_t *rsp_dict);
 
