@@ -29,6 +29,8 @@
 #include "nfs-generics.h"
 #include "acl3.h"
 #include "byte-order.h"
+#include "compat-errno.h"
+#include "nfs-messages.h"
 
 static int
 acl3_nfs_acl_to_xattr (aclentry *ace, void *xattrbuf,
@@ -48,14 +50,16 @@ nfs3_call_state_init (struct nfs3_state *s, rpcsvc_request_t *req, xlator_t *v);
 extern int
 nfs3_fh_validate (struct nfs3_fh *fh);
 
-extern fattr3
-nfs3_stat_to_fattr3 (struct iatt *buf);
+extern void
+nfs3_stat_to_fattr3 (struct iatt *buf, fattr3 *fa);
 
 #define acl3_validate_nfs3_state(request, state, status, label, retval) \
         do      {                                                       \
                 state = rpcsvc_request_program_private (request);       \
                 if (!state) {                                           \
-                        gf_log (GF_ACL, GF_LOG_ERROR, "NFSv3 state "    \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, errno,            \
+                                NFS_MSG_STATE_MISSING,                  \
+                                "NFSv3 state "                          \
                                 "missing from RPC request");            \
                         rpcsvc_request_seterr (req, SYSTEM_ERR);        \
                         status = NFS3ERR_SERVERFAULT;                   \
@@ -66,7 +70,9 @@ nfs3_stat_to_fattr3 (struct iatt *buf);
 #define acl3_validate_gluster_fh(handle, status, errlabel)              \
         do {                                                            \
                 if (!nfs3_fh_validate (handle)) {                       \
-                        gf_log (GF_ACL, GF_LOG_ERROR, "Bad Handle");    \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL,           \
+                                NFS_MSG_BAD_HANDLE,                     \
+                                "Bad Handle");                          \
                         status = NFS3ERR_BADHANDLE;                     \
                         goto errlabel;                                  \
                 }                                                       \
@@ -82,22 +88,24 @@ nfs3_fh_to_xlator (struct nfs3_state *nfs3, struct nfs3_fh *fh);
                 rpc_transport_t *trans = NULL;                          \
                 volume = nfs3_fh_to_xlator ((nfs3state), handle);      \
                 if (!volume) {                                          \
-                        uuid_unparse (handle->exportid, exportid);       \
-                        uuid_unparse (handle->gfid, gfid);               \
+                        gf_uuid_unparse (handle->exportid, exportid);   \
+                        gf_uuid_unparse (handle->gfid, gfid);           \
                         trans = rpcsvc_request_transport (req);         \
-                        gf_log (GF_ACL, GF_LOG_ERROR, "Failed to map "  \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, 0,                \
+                                NFS_MSG_FH_TO_VOL_FAIL, "Failed to map "  \
                                 "FH to vol: client=%s, exportid=%s, gfid=%s",\
                                 trans->peerinfo.identifier, exportid,   \
                                 gfid);                                  \
-                        gf_log (GF_ACL, GF_LOG_ERROR,                   \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, ESTALE,           \
+                                NFS_MSG_VOLUME_ERROR,                   \
                                 "Stale nfs client %s must be trying to "\
                                 "connect to a deleted volume, please "  \
                                 "unmount it.", trans->peerinfo.identifier);\
                         status = NFS3ERR_STALE;                         \
                         goto label;                                     \
                 } else {                                                \
-                        gf_log (GF_ACL, GF_LOG_TRACE, "FH to Volume: %s"\
-                                ,volume->name);                         \
+                        gf_msg_trace (GF_ACL, 0, "FH to Volume: %s",    \
+                                      volume->name);                    \
                         rpcsvc_request_set_private (req, volume);       \
                 }                                                       \
         } while (0);                                                    \
@@ -105,8 +113,9 @@ nfs3_fh_to_xlator (struct nfs3_state *nfs3, struct nfs3_fh *fh);
 #define acl3_volume_started_check(nfs3state, vlm, rtval, erlbl)         \
         do {                                                            \
               if ((!nfs_subvolume_started (nfs_state (nfs3state->nfsx), vlm))){\
-                      gf_log (GF_ACL, GF_LOG_ERROR, "Volume is disabled: %s",\
-                              vlm->name);                               \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, 0, NFS_MSG_VOL_DISABLE, \
+                                "Volume is disabled: %s",                 \
+                                vlm->name);                               \
                       rtval = RPCSVC_ACTOR_IGNORE;                      \
                       goto erlbl;                                       \
               }                                                         \
@@ -121,13 +130,14 @@ nfs3_fh_to_xlator (struct nfs3_state *nfs3, struct nfs3_fh *fh);
                         trans = rpcsvc_request_transport (cst->req);    \
                         xlatorp = nfs3_fh_to_xlator (cst->nfs3state,    \
                                                      &cst->resolvefh);  \
-                        uuid_unparse (cst->resolvefh.gfid, gfid);       \
-                        snprintf (buf, sizeof (buf), "(%s) %s : %s",             \
+                        gf_uuid_unparse (cst->resolvefh.gfid, gfid);    \
+                        snprintf (buf, sizeof (buf), "(%s) %s : %s",    \
                                   trans->peerinfo.identifier,           \
                                   xlatorp ? xlatorp->name : "ERR",      \
                                   gfid);                                \
-                        gf_log (GF_ACL, GF_LOG_ERROR, "Unable to resolve FH"\
-                                ": %s", buf);                           \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, cst->resolve_errno, \
+                                NFS_MSG_RESOLVE_FH_FAIL, "Unable to resolve "\
+                                "FH: %s", buf);                           \
                         nfstat = nfs3_errno_to_nfsstat3 (cst->resolve_errno);\
                         goto erlabl;                                    \
                 }                                                       \
@@ -135,9 +145,10 @@ nfs3_fh_to_xlator (struct nfs3_state *nfs3, struct nfs3_fh *fh);
 
 #define acl3_handle_call_state_init(nfs3state, calls, rq, v, opstat, errlabel)\
         do {                                                            \
-                calls = nfs3_call_state_init ((nfs3state), (rq), v); \
+                calls = nfs3_call_state_init ((nfs3state), (rq), v);    \
                 if (!calls) {                                           \
-                        gf_log (GF_ACL, GF_LOG_ERROR, "Failed to "      \
+                        gf_msg (GF_ACL, GF_LOG_ERROR, 0,                \
+                                NFS_MSG_INIT_CALL_STAT_FAIL, "Failed to " \
                                 "init call state");                     \
                         opstat = NFS3ERR_SERVERFAULT;                   \
                         rpcsvc_request_seterr (req, SYSTEM_ERR);        \
@@ -161,7 +172,8 @@ acl3svc_submit_reply (rpcsvc_request_t *req, void *arg, acl3_serializer sfunc)
 
         nfs3 = (struct nfs3_state *)rpcsvc_request_program_private (req);
         if (!nfs3) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "mount state not found");
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL,
+                        NFS_MSG_MNT_STATE_NOT_FOUND, "mount state not found");
                 goto ret;
         }
 
@@ -170,7 +182,8 @@ acl3svc_submit_reply (rpcsvc_request_t *req, void *arg, acl3_serializer sfunc)
          */
         iob = iobuf_get (nfs3->iobpool);
         if (!iob) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to get iobuf");
+                gf_msg (GF_ACL, GF_LOG_ERROR, ENOMEM, NFS_MSG_NO_MEMORY,
+                        "Failed to get iobuf");
                 goto ret;
         }
 
@@ -180,27 +193,31 @@ acl3svc_submit_reply (rpcsvc_request_t *req, void *arg, acl3_serializer sfunc)
          */
         msglen = sfunc (outmsg, arg);
         if (msglen < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to encode message");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno, NFS_MSG_ENCODE_MSG_FAIL,
+                        "Failed to encode message");
                 goto ret;
         }
         outmsg.iov_len = msglen;
 
         iobref = iobref_new ();
         if (iobref == NULL) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to get iobref");
+                gf_msg (GF_ACL, GF_LOG_ERROR, ENOMEM, NFS_MSG_NO_MEMORY,
+                        "Failed to get iobref");
                 goto ret;
         }
 
         ret = iobref_add (iobref, iob);
         if (ret) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to add iob to iobref");
+                gf_msg (GF_ACL, GF_LOG_ERROR, ENOMEM, NFS_MSG_NO_MEMORY,
+                        "Failed to add iob to iobref");
                 goto ret;
         }
 
         /* Then, submit the message for transmission. */
         ret = rpcsvc_submit_message (req, &outmsg, 1, NULL, 0, iobref);
         if (ret == -1) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Reply submission failed");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno, NFS_MSG_REP_SUBMIT_FAIL,
+                        "Reply submission failed");
                 goto ret;
         }
 
@@ -221,7 +238,8 @@ acl3svc_null (rpcsvc_request_t *req)
         struct iovec    dummyvec = {0, };
 
         if (!req) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Got NULL request!");
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL, NFS_MSG_INVALID_ENTRY,
+                        "Got NULL request!");
                 return 0;
         }
         rpcsvc_submit_generic (req, &dummyvec, 1,  NULL, 0, NULL);
@@ -229,14 +247,25 @@ acl3svc_null (rpcsvc_request_t *req)
 }
 
 int
-acl3_getacl_reply (nfs3_call_state_t *cs, getaclreply *reply)
+acl3_getacl_reply (rpcsvc_request_t *req, getaclreply *reply)
 {
-        acl3svc_submit_reply (cs->req, (void *)reply,
+        acl3svc_submit_reply (req, (void *)reply,
                               (acl3_serializer)xdr_serialize_getaclreply);
         return 0;
 }
 
+int
+acl3_setacl_reply (rpcsvc_request_t *req, setaclreply *reply)
+{
+        acl3svc_submit_reply (req, (void *)reply,
+                              (acl3_serializer)xdr_serialize_setaclreply);
+        return 0;
+}
 
+/* acl3_getacl_cbk: fetch and decode the ACL in the POSIX_ACL_ACCESS_XATTR
+ *
+ * The POSIX_ACL_ACCESS_XATTR can be set on files and directories.
+ */
 int
 acl3_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno, dict_t *dict,
@@ -250,19 +279,22 @@ acl3_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         int                      defacl = 1; /* DEFAULT ACL */
 
         if (!frame->local) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Invalid argument,"
-                       " frame->local NULL");
-                return  EINVAL;
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL, NFS_MSG_INVALID_ENTRY,
+                        "Invalid argument, frame->local NULL");
+                return -EINVAL;
         }
         cs = frame->local;
         getaclreply = &cs->args.getaclreply;
-        if (op_ret == -1) {
+        if ((op_ret < 0) && (op_errno != ENODATA && op_errno != ENOATTR)) {
                 stat = nfs3_cbk_errno_status (op_ret, op_errno);
                 goto err;
+        } else if (!dict) {
+                /* no ACL has been set */
+                stat = NFS3_OK;
+                goto err;
         }
 
         getaclreply->aclentry.aclentry_val = cs->aclentry;
-        getaclreply->daclentry.daclentry_val = cs->daclentry;
 
         /* getfacl: NFS USER ACL */
         data = dict_get (dict, POSIX_ACL_ACCESS_XATTR);
@@ -272,16 +304,68 @@ acl3_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                                     data->len,
                                                     !defacl);
                 if (aclcount < 0) {
-                        gf_log (GF_ACL, GF_LOG_ERROR,
+                        gf_msg (GF_ACL, GF_LOG_ERROR, aclcount,
+                                NFS_MSG_GET_USER_ACL_FAIL,
                                 "Failed to get USER ACL");
                         stat = nfs3_errno_to_nfsstat3 (-aclcount);
                         goto err;
                 }
-
                 getaclreply->aclcount = aclcount;
                 getaclreply->aclentry.aclentry_len = aclcount;
         }
 
+        acl3_getacl_reply (cs->req, getaclreply);
+        nfs3_call_state_wipe (cs);
+        return 0;
+
+err:
+        if (getaclreply)
+                getaclreply->status = stat;
+        acl3_getacl_reply (cs->req, getaclreply);
+        nfs3_call_state_wipe (cs);
+        return 0;
+}
+
+/* acl3_default_getacl_cbk: fetch and decode the ACL set in the
+ * POSIX_ACL_DEFAULT_XATTR xattr.
+ *
+ * The POSIX_ACL_DEFAULT_XATTR xattr is only set on directories, not on files.
+ *
+ * When done with POSIX_ACL_DEFAULT_XATTR, we also need to get and decode the
+ * ACL that can be set in POSIX_ACL_DEFAULT_XATTR.
+ */
+int
+acl3_default_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                         int32_t op_ret, int32_t op_errno, dict_t *dict,
+                         dict_t *xdata)
+{
+        nfsstat3                 stat = NFS3ERR_SERVERFAULT;
+        nfs3_call_state_t        *cs = NULL;
+        data_t                   *data = NULL;
+        getaclreply              *getaclreply = NULL;
+        int                      aclcount = 0;
+        int                      defacl = 1; /* DEFAULT ACL */
+        nfs_user_t               nfu = {0, };
+        int                      ret = -1;
+
+        if (!frame->local) {
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL, NFS_MSG_INVALID_ENTRY,
+                        "Invalid argument, frame->local NULL");
+                return -EINVAL;
+        }
+        cs = frame->local;
+        getaclreply = &cs->args.getaclreply;
+        if ((op_ret < 0) && (op_errno != ENODATA && op_errno != ENOATTR)) {
+                stat = nfs3_cbk_errno_status (op_ret, op_errno);
+                goto err;
+        } else if (!dict) {
+                /* no ACL has been set */
+                stat = NFS3_OK;
+                goto err;
+        }
+
+        getaclreply->daclentry.daclentry_val = cs->daclentry;
+
         /* getfacl: NFS DEFAULT ACL */
         data = dict_get (dict, POSIX_ACL_DEFAULT_XATTR);
         if (data && data->data) {
@@ -290,7 +374,8 @@ acl3_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                                     data->len,
                                                     defacl);
                 if (aclcount < 0) {
-                        gf_log (GF_ACL, GF_LOG_ERROR,
+                        gf_msg (GF_ACL, GF_LOG_ERROR, aclcount,
+                                NFS_MSG_GET_DEF_ACL_FAIL,
                                 "Failed to get DEFAULT ACL");
                         stat = nfs3_errno_to_nfsstat3 (-aclcount);
                         goto err;
@@ -300,18 +385,26 @@ acl3_getacl_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 getaclreply->daclentry.daclentry_len = aclcount;
         }
 
-        acl3_getacl_reply (cs, getaclreply);
-        nfs3_call_state_wipe (cs);
+        getaclreply->attr_follows = TRUE;
+        nfs_request_user_init (&nfu, cs->req);
+        ret = nfs_getxattr (cs->nfsx, cs->vol, &nfu, &cs->resolvedloc,
+                            POSIX_ACL_ACCESS_XATTR, NULL, acl3_getacl_cbk, cs);
+        if (ret < 0) {
+                stat = nfs3_errno_to_nfsstat3 (-ret);
+                goto err;
+        }
+
         return 0;
 
 err:
         if (getaclreply)
                 getaclreply->status = stat;
-        acl3_getacl_reply (cs, getaclreply);
+        acl3_getacl_reply (cs->req, getaclreply);
         nfs3_call_state_wipe (cs);
         return 0;
 }
 
+
 int
 acl3_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                   int32_t op_ret, int32_t op_errno, struct iatt *buf,
@@ -325,8 +418,8 @@ acl3_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         uint64_t                        deviceid = 0;
 
         if (!frame->local) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Invalid argument,"
-                       " frame->local NULL");
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL, NFS_MSG_INVALID_ENTRY,
+                        "Invalid argument, frame->local NULL");
                 return EINVAL;
         }
 
@@ -342,20 +435,28 @@ acl3_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         getaclreply->attr_follows = TRUE;
         deviceid = nfs3_request_xlator_deviceid (cs->req);
         nfs3_map_deviceid_to_statdev (buf, deviceid);
-        getaclreply->attr = nfs3_stat_to_fattr3 (buf);
-        getaclreply->mask = (NFS_ACL|NFS_ACLCNT|NFS_DFACL|NFS_DFACLCNT);
+        nfs3_stat_to_fattr3 (buf, &(getaclreply->attr));
 
         nfs_request_user_init (&nfu, cs->req);
-        ret = nfs_getxattr (cs->nfsx, cs->vol, &nfu, &cs->resolvedloc,
-                            NULL, NULL, acl3_getacl_cbk, cs);
+        if (buf->ia_type == IA_IFDIR) {
+                ret = nfs_getxattr (cs->nfsx, cs->vol, &nfu, &cs->resolvedloc,
+                                    POSIX_ACL_DEFAULT_XATTR, NULL,
+                                    acl3_default_getacl_cbk, cs);
+        } else {
+                ret = nfs_getxattr (cs->nfsx, cs->vol, &nfu, &cs->resolvedloc,
+                                    POSIX_ACL_ACCESS_XATTR, NULL,
+                                    acl3_getacl_cbk, cs);
+        }
+
         if (ret < 0) {
                 stat = nfs3_errno_to_nfsstat3 (-ret);
                 goto err;
         }
+
         return 0;
 err:
         getaclreply->status = stat;
-        acl3_getacl_reply (cs, getaclreply);
+        acl3_getacl_reply (cs->req, getaclreply);
         nfs3_call_state_wipe (cs);
         return 0;
 }
@@ -381,9 +482,10 @@ acl3_getacl_resume (void *carg)
         stat = -ret;
 acl3err:
         if (ret < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "unable to open_and_resume");
+                gf_msg (GF_ACL, GF_LOG_ERROR, stat, NFS_MSG_OPEN_FAIL,
+                        "unable to open_and_resume");
                 cs->args.getaclreply.status = nfs3_errno_to_nfsstat3 (stat);
-                acl3_getacl_reply (cs, &cs->args.getaclreply);
+                acl3_getacl_reply (cs->req, &cs->args.getaclreply);
                 nfs3_call_state_wipe (cs);
         }
 
@@ -402,6 +504,7 @@ acl3svc_getacl (rpcsvc_request_t *req)
         nfsstat3                        stat = NFS3ERR_SERVERFAULT;
         struct nfs3_fh                  fh, *fhp = NULL;
         getaclargs                      getaclargs;
+        getaclreply                     getaclreply;
 
         if (!req)
                 return ret;
@@ -409,9 +512,11 @@ acl3svc_getacl (rpcsvc_request_t *req)
         acl3_validate_nfs3_state (req, nfs3, stat, rpcerr, ret);
         nfs = nfs_state (nfs3->nfsx);
         memset (&getaclargs, 0, sizeof (getaclargs));
+        memset (&getaclreply, 0, sizeof (getaclreply));
         getaclargs.fh.n_bytes = (char *)&fh;
         if (xdr_to_getaclargs(req->msg[0], &getaclargs) <= 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Error decoding args");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno, NFS_MSG_ARGS_DECODE_ERROR,
+                        "Error decoding args");
                 rpcsvc_request_seterr (req, GARBAGE_ARGS);
                 goto rpcerr;
         }
@@ -424,25 +529,24 @@ acl3svc_getacl (rpcsvc_request_t *req)
 
         fhp = &fh;
         acl3_validate_gluster_fh (&fh, stat, acl3err);
-        acl3_map_fh_to_volume (nfs->nfs3state, fhp, req,
-                               vol, stat, acl3err);
+        acl3_map_fh_to_volume (nfs->nfs3state, fhp, req, vol, stat, acl3err);
+        acl3_volume_started_check (nfs3, vol, ret, rpcerr);
         acl3_handle_call_state_init (nfs->nfs3state, cs, req,
-                                     vol, stat, rpcerr);
+                                     vol, stat, acl3err);
 
         cs->vol = vol;
-        acl3_volume_started_check (nfs3, vol, ret, acl3err);
+        cs->args.getaclreply.mask = getaclargs.mask;
 
-        ret = nfs3_fh_resolve_and_resume (cs, fhp,
-                                          NULL, acl3_getacl_resume);
+        ret = nfs3_fh_resolve_and_resume (cs, fhp, NULL, acl3_getacl_resume);
+        stat = nfs3_errno_to_nfsstat3 (-ret);
 
 acl3err:
         if (ret < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "unable to resolve and resume");
-                if (cs) {
-                        cs->args.getaclreply.status = stat;
-                        acl3_getacl_reply (cs, &cs->args.getaclreply);
-                        nfs3_call_state_wipe (cs);
-                }
+                gf_msg (GF_ACL, GF_LOG_ERROR, -ret, NFS_MSG_RESOLVE_ERROR,
+                        "unable to resolve and resume");
+                getaclreply.status = stat;
+                acl3_getacl_reply (req, &getaclreply);
+                nfs3_call_state_wipe (cs);
                 return 0;
         }
 
@@ -462,8 +566,10 @@ acl3_setacl_cbk (call_frame_t *frame, void *cookie,
                 cs->args.setaclreply.status = status;
         }
 
-        acl3svc_submit_reply (cs->req, (void *)&cs->args.setaclreply,
-                              (acl3_serializer)xdr_serialize_setaclreply);
+        acl3_setacl_reply (cs->req, &cs->args.setaclreply);
+
+        nfs3_call_state_wipe (cs);
+
         return 0;
 }
 
@@ -498,10 +604,10 @@ acl3_setacl_resume (void *carg)
 acl3err:
         if (ret < 0) {
                 stat = -ret;
-                gf_log (GF_ACL, GF_LOG_ERROR, "unable to open_and_resume");
+                gf_msg (GF_ACL, GF_LOG_ERROR, stat, NFS_MSG_OPEN_FAIL,
+                        "unable to open_and_resume");
                 cs->args.setaclreply.status = nfs3_errno_to_nfsstat3 (stat);
-                acl3svc_submit_reply (cs->req, (void *)&cs->args.setaclreply,
-                                      (acl3_serializer)xdr_serialize_setaclreply);
+                acl3_setacl_reply (cs->req, &cs->args.setaclreply);
                 nfs3_call_state_wipe (cs);
         }
 
@@ -521,8 +627,9 @@ acl3svc_setacl (rpcsvc_request_t *req)
         struct nfs3_fh                 fh;
         struct nfs3_fh                 *fhp = NULL;
         setaclargs                     setaclargs;
+        setaclreply                    setaclreply;
+        aclentry                       *daclentry = NULL;
         aclentry                       *aclentry = NULL;
-        struct aclentry                *daclentry = NULL;
         int                            aclerrno = 0;
         int                            defacl = 1;
 
@@ -542,25 +649,32 @@ acl3svc_setacl (rpcsvc_request_t *req)
         acl3_validate_nfs3_state (req, nfs3, stat, rpcerr, ret);
         nfs = nfs_state (nfs3->nfsx);
         memset (&setaclargs, 0, sizeof (setaclargs));
+        memset (&setaclreply, 0, sizeof (setaclreply));
         memset (&fh, 0, sizeof (fh));
         setaclargs.fh.n_bytes = (char *)&fh;
         setaclargs.aclentry.aclentry_val = aclentry;
         setaclargs.daclentry.daclentry_val = daclentry;
         if (xdr_to_setaclargs(req->msg[0], &setaclargs) <= 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Error decoding args");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno, NFS_MSG_ARGS_DECODE_ERROR,
+                        "Error decoding args");
                 rpcsvc_request_seterr (req, GARBAGE_ARGS);
                 goto rpcerr;
         }
+
+        /* Validate ACL mask */
+        if (setaclargs.mask & ~(NFS_ACL|NFS_ACLCNT|NFS_DFACL|NFS_DFACLCNT)) {
+                stat = NFS3ERR_INVAL;
+                goto acl3err;
+        }
+
         fhp = &fh;
         acl3_validate_gluster_fh (fhp, stat, acl3err);
-        acl3_map_fh_to_volume (nfs->nfs3state, fhp, req,
-                               vol, stat, acl3err);
+        acl3_map_fh_to_volume (nfs->nfs3state, fhp, req, vol, stat, acl3err);
+        acl3_volume_started_check (nfs3, vol, ret, rpcerr);
         acl3_handle_call_state_init (nfs->nfs3state, cs, req,
-                                     vol, stat, rpcerr);
+                                     vol, stat, acl3err);
 
         cs->vol = vol;
-        acl3_volume_started_check (nfs3, vol, ret, rpcerr);
-
         cs->aclcount = setaclargs.aclcount;
         cs->daclcount = setaclargs.daclcount;
 
@@ -570,7 +684,9 @@ acl3svc_setacl (rpcsvc_request_t *req)
                                           cs->aclcount,
                                           !defacl);
         if (aclerrno < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to set USER ACL");
+                gf_msg (GF_ACL, GF_LOG_ERROR, -aclerrno,
+                        NFS_MSG_SET_USER_ACL_FAIL,
+                        "Failed to set USER ACL");
                 stat = nfs3_errno_to_nfsstat3 (-aclerrno);
                 goto acl3err;
         }
@@ -581,20 +697,22 @@ acl3svc_setacl (rpcsvc_request_t *req)
                                           cs->daclcount,
                                           defacl);
         if (aclerrno < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Failed to set DEFAULT ACL");
+                gf_msg (GF_ACL, GF_LOG_ERROR, -aclerrno,
+                        NFS_MSG_SET_DEF_ACL_FAIL,
+                        "Failed to set DEFAULT ACL");
                 stat = nfs3_errno_to_nfsstat3 (-aclerrno);
                 goto acl3err;
         }
 
-        ret = nfs3_fh_resolve_and_resume (cs, fhp,
-                                          NULL, acl3_setacl_resume);
+        ret = nfs3_fh_resolve_and_resume (cs, fhp, NULL, acl3_setacl_resume);
+        stat = nfs3_errno_to_nfsstat3 (-ret);
 
 acl3err:
         if (ret < 0) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "unable to resolve and resume");
-                cs->args.setaclreply.status = stat;
-                acl3svc_submit_reply (cs->req, (void *)&cs->args.setaclreply,
-                                      (acl3_serializer)xdr_serialize_setaclreply);
+                gf_msg (GF_ACL, GF_LOG_ERROR, -ret, NFS_MSG_RESOLVE_ERROR,
+                        "unable to resolve and resume");
+                setaclreply.status = stat;
+                acl3_setacl_reply (req, &setaclreply);
                 nfs3_call_state_wipe (cs);
                 GF_FREE(aclentry);
                 GF_FREE(daclentry);
@@ -647,7 +765,8 @@ acl3svc_init(xlator_t *nfsx)
 
         ns = nfs->nfs3state;
         if (!ns) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "ACL3 init failed");
+                gf_msg (GF_ACL, GF_LOG_ERROR, EINVAL, NFS_MSG_ACL_INIT_FAIL,
+                        "ACL3 init failed");
                 goto err;
         }
         acl3prog.private = ns;
@@ -664,32 +783,41 @@ acl3svc_init(xlator_t *nfsx)
                 goto err;
         ret = dict_set_str (options, "transport-type", "socket");
         if (ret == -1) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "dict_set_str error");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno, NFS_MSG_DICT_SET_FAILED,
+                        "dict_set_str error");
                 goto err;
         }
 
         if (nfs->allow_insecure) {
                 ret = dict_set_str (options, "rpc-auth-allow-insecure", "on");
                 if (ret == -1) {
-                        gf_log (GF_ACL, GF_LOG_ERROR, "dict_set_str error");
+                        gf_msg (GF_ACL, GF_LOG_ERROR, errno,
+                                 NFS_MSG_DICT_SET_FAILED,
+                                 "dict_set_str error");
                         goto err;
                 }
                 ret = dict_set_str (options, "rpc-auth.ports.insecure", "on");
                 if (ret == -1) {
-                        gf_log (GF_ACL, GF_LOG_ERROR, "dict_set_str error");
+                        gf_msg (GF_ACL, GF_LOG_ERROR, errno,
+                                NFS_MSG_DICT_SET_FAILED,
+                                "dict_set_str error");
                         goto err;
                 }
         }
 
         ret = dict_set_str (options, "transport.address-family", "inet");
         if (ret == -1) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "dict_set_str error");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno,
+                        NFS_MSG_DICT_SET_FAILED,
+                        "dict_set_str error");
                 goto err;
         }
 
         ret = rpcsvc_create_listeners (nfs->rpcsvc, options, "ACL");
         if (ret == -1) {
-                gf_log (GF_ACL, GF_LOG_ERROR, "Unable to create listeners");
+                gf_msg (GF_ACL, GF_LOG_ERROR, errno,
+                        NFS_MSG_LISTENERS_CREATE_FAIL,
+                        "Unable to create listeners");
                 dict_unref (options);
                 goto err;
         }
