@@ -38,6 +38,12 @@
 #define GF_NFS_DVM_ON                   1
 #define GF_NFS_DVM_OFF                  0
 
+/* Disable using the exports file by default */
+#define GF_NFS_DEFAULT_EXPORT_AUTH      0
+
+#define GF_NFS_DEFAULT_AUTH_REFRESH_INTERVAL_SEC       2
+#define GF_NFS_DEFAULT_AUTH_CACHE_TTL_SEC              300 /* 5 min */
+
 /* This corresponds to the max 16 number of group IDs that are sent through an
  * RPC request. Since NFS is the only one going to set this, we can be safe
  * in keeping this size hardcoded.
@@ -55,6 +61,7 @@ struct nfs_initer_list {
         struct list_head list;
         nfs_version_initer_t    init;
         rpcsvc_program_t        *program;
+        gf_boolean_t            required;
 };
 
 struct nfs_state {
@@ -79,6 +86,15 @@ struct nfs_state {
         int                     enable_nlm;
         int                     enable_acl;
         int                     mount_udp;
+
+        /* Enable exports auth model */
+        int                     exports_auth;
+        /* Refresh auth params from disk periodically */
+        int                     refresh_auth;
+
+        unsigned int            auth_refresh_time_secs;
+        unsigned int            auth_cache_ttl_sec;
+
         char                    *rmtab;
         struct rpc_clnt         *rpc_clnt;
         gf_boolean_t            server_aux_gids;
@@ -86,6 +102,8 @@ struct nfs_state {
 	gid_cache_t		gid_cache;
         uint32_t                generation;
         gf_boolean_t            register_portmap;
+        char                    *rpc_statd;
+        char                    *rpc_statd_pid_file;
 };
 
 struct nfs_inode_ctx {
