@@ -23,6 +23,7 @@
 #include "nfs-common.h"
 #include "iatt.h"
 #include "common-utils.h"
+#include "nfs-messages.h"
 
 
 int
@@ -58,7 +59,7 @@ nfs3_fh_init (struct nfs3_fh *fh, struct iatt *buf)
         fh->ident[2] = GF_NFSFH_IDENT2;
         fh->ident[3] = GF_NFSFH_IDENT3;
 
-        uuid_copy (fh->gfid, buf->ia_gfid);
+        gf_uuid_copy (fh->gfid, buf->ia_gfid);
 }
 
 
@@ -72,7 +73,7 @@ nfs3_fh_build_indexed_root_fh (xlator_list_t *cl, xlator_t *xl)
         if ((!cl) || (!xl))
                 return fh;
 
-        uuid_copy (buf.ia_gfid, root);
+        gf_uuid_copy (buf.ia_gfid, root);
         nfs3_fh_init (&fh, &buf);
         fh.exportid [15] = nfs_xlator_to_xlid (cl, xl);
 
@@ -81,15 +82,16 @@ nfs3_fh_build_indexed_root_fh (xlator_list_t *cl, xlator_t *xl)
 
 
 struct nfs3_fh
-nfs3_fh_build_uuid_root_fh (uuid_t volumeid)
+nfs3_fh_build_uuid_root_fh (uuid_t volumeid, uuid_t mountid)
 {
         struct nfs3_fh  fh = {{0}, };
         struct iatt     buf = {0, };
         uuid_t          root = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1};
 
-        uuid_copy (buf.ia_gfid, root);
+        gf_uuid_copy (buf.ia_gfid, root);
         nfs3_fh_init (&fh, &buf);
-        uuid_copy (fh.exportid, volumeid);
+        gf_uuid_copy (fh.exportid, volumeid);
+        gf_uuid_copy (fh.mountid, mountid);
 
         return fh;
 }
@@ -103,7 +105,7 @@ nfs3_fh_is_root_fh (struct nfs3_fh *fh)
         if (!fh)
                 return 0;
 
-        if (uuid_compare (fh->gfid, rootgfid) == 0)
+        if (gf_uuid_compare (fh->gfid, rootgfid) == 0)
                 return 1;
 
         return 0;
@@ -115,13 +117,15 @@ nfs3_fh_to_str (struct nfs3_fh *fh, char *str, size_t len)
 {
         char            gfid[GF_UUID_BUF_SIZE];
         char            exportid[GF_UUID_BUF_SIZE];
+        char            mountid[GF_UUID_BUF_SIZE];
 
         if ((!fh) || (!str))
                 return;
 
-        snprintf (str, len, "FH: exportid %s, gfid %s",
+        snprintf (str, len, "FH: exportid %s, gfid %s, mountid %s",
                   uuid_utoa_r (fh->exportid, exportid),
-                  uuid_utoa_r (fh->gfid, gfid));
+                  uuid_utoa_r (fh->gfid, gfid),
+                  uuid_utoa_r (fh->mountid, mountid));
 }
 
 void
@@ -133,10 +137,9 @@ nfs3_log_fh (struct nfs3_fh *fh)
         if (!fh)
                 return;
 
-        gf_log ("nfs3-fh", GF_LOG_TRACE, "filehandle: exportid "
-                "0x%s, gfid 0x%s",
-                 uuid_utoa_r (fh->exportid, exportidstr),
-                 uuid_utoa_r (fh->gfid, gfidstr));
+        gf_msg_trace ("nfs3-fh", 0, "filehandle: exportid 0x%s, gfid 0x%s",
+                      uuid_utoa_r (fh->exportid, exportidstr),
+                      uuid_utoa_r (fh->gfid, gfidstr));
 }
 
 int
@@ -147,8 +150,7 @@ nfs3_fh_build_parent_fh (struct nfs3_fh *child, struct iatt *newstat,
                 return -1;
 
         nfs3_fh_init (newfh, newstat);
-        uuid_copy (newfh->exportid, child->exportid);
-
+        gf_uuid_copy (newfh->exportid, child->exportid);
         return 0;
 }
 
@@ -162,8 +164,9 @@ nfs3_build_fh (inode_t *inode, uuid_t exportid, struct nfs3_fh *newfh)
         newfh->ident[1] = GF_NFSFH_IDENT1;
         newfh->ident[2] = GF_NFSFH_IDENT2;
         newfh->ident[3] = GF_NFSFH_IDENT3;
-        uuid_copy (newfh->gfid, inode->gfid);
-        uuid_copy (newfh->exportid, exportid);
+        gf_uuid_copy (newfh->gfid, inode->gfid);
+        gf_uuid_copy (newfh->exportid, exportid);
+        /*gf_uuid_copy (newfh->mountid, mountid);*/
         return 0;
 }
 
@@ -175,14 +178,14 @@ nfs3_fh_build_child_fh (struct nfs3_fh *parent, struct iatt *newstat,
                 return -1;
 
         nfs3_fh_init (newfh, newstat);
-        uuid_copy (newfh->exportid, parent->exportid);
-
+        gf_uuid_copy (newfh->exportid, parent->exportid);
+        gf_uuid_copy (newfh->mountid, parent->mountid);
         return 0;
 }
 
 
 uint32_t
-nfs3_fh_compute_size (struct nfs3_fh *fh)
+nfs3_fh_compute_size ()
 {
         return GF_NFSFH_STATIC_SIZE;
 }
