@@ -18,9 +18,11 @@
 #include "dict.h"
 #include "xlator.h"
 #include "md-cache-mem-types.h"
+#include "compat-errno.h"
 #include "glusterfs-acl.h"
 #include <assert.h>
 #include <sys/time.h>
+#include "md-cache-messages.h"
 
 
 /* TODO:
@@ -52,6 +54,16 @@ static struct mdc_key {
 		.load = 0,
 		.check = 1,
 	},
+	{
+		.name = GF_POSIX_ACL_ACCESS,
+		.load = 0,
+		.check = 1,
+	},
+	{
+		.name = GF_POSIX_ACL_DEFAULT,
+		.load = 0,
+		.check = 1,
+	},
 	{
 		.name = GF_SELINUX_XATTR_KEY,
 		.load = 0,
@@ -281,8 +293,8 @@ mdc_inode_prep (xlator_t *this, inode_t *inode)
 
                 mdc = GF_CALLOC (sizeof (*mdc), 1, gf_mdc_mt_md_cache_t);
                 if (!mdc) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "out of memory :(");
+                        gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                                MD_CACHE_MSG_NO_MEMORY, "out of memory");
                         goto unlock;
                 }
 
@@ -290,8 +302,8 @@ mdc_inode_prep (xlator_t *this, inode_t *inode)
 
                 ret = __mdc_inode_ctx_set (this, inode, mdc);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "out of memory :(");
+                        gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                                MD_CACHE_MSG_NO_MEMORY, "out of memory");
                         GF_FREE (mdc);
                         mdc = NULL;
                 }
@@ -410,9 +422,13 @@ mdc_inode_iatt_set_validate(xlator_t *this, inode_t *inode, struct iatt *prebuf,
 		 */
 		if (IA_ISREG(inode->ia_type) &&
 		    ((iatt->ia_mtime != mdc->md_mtime) ||
-		    (iatt->ia_ctime != mdc->md_ctime)))
+		    (iatt->ia_mtime_nsec != mdc->md_mtime_nsec) ||
+		    (iatt->ia_ctime != mdc->md_ctime) ||
+		    (iatt->ia_ctime_nsec != mdc->md_ctime_nsec)))
 			if (!prebuf || (prebuf->ia_ctime != mdc->md_ctime) ||
-			    (prebuf->ia_mtime != mdc->md_mtime))
+			    (prebuf->ia_ctime_nsec != mdc->md_ctime_nsec) ||
+			    (prebuf->ia_mtime != mdc->md_mtime) ||
+			    (prebuf->ia_mtime_nsec != mdc->md_mtime_nsec))
 				inode_invalidate(inode);
 
                 mdc_from_iatt (mdc, iatt);
@@ -449,7 +465,7 @@ mdc_inode_iatt_get (xlator_t *this, inode_t *inode, struct iatt *iatt)
         }
         UNLOCK (&mdc->lock);
 
-        uuid_copy (iatt->ia_gfid, inode->gfid);
+        gf_uuid_copy (iatt->ia_gfid, inode->gfid);
         iatt->ia_ino    = gfid_to_ino (inode->gfid);
         iatt->ia_dev    = 42;
         iatt->ia_type   = inode->ia_type;
@@ -485,6 +501,25 @@ updatefn(dict_t *dict, char *key, data_t *value, void *data)
 			}
 		}
 
+                /* posix xlator as part of listxattr will send both names
+                 * and values of the xattrs in the dict. But as per man page
+                 * listxattr is mainly supposed to send names of the all the
+                 * xattrs. gfapi, as of now will put all the keys it obtained
+                 * in the dict (sent by posix) into a buffer provided by the
+                 * caller (thus the values of those xattrs are lost). If some
+                 * xlator makes gfapi based calls (ex: snapview-server), then
+                 * it has to unwind the calls by putting those names it got
+                 * in the buffer again into the dict. But now it would not be
+                 * having the values for those xattrs. So it might just put
+                 * a 0 byte value ("") into the dict for each xattr and unwind
+                 * the call. So the xlators which cache the xattrs (as of now
+                 * md-cache caches the acl and selinux related xattrs), should
+                 * not update their cache if the value of a xattr is a 0 byte
+                 * data (i.e. "").
+                 */
+                if (!strcmp (value->data, ""))
+                        continue;
+
 		if (dict_set(u->dict, key, value) < 0) {
 			u->ret = -1;
 			return -1;
@@ -598,7 +633,7 @@ mdc_inode_xatt_unset (xlator_t *this, inode_t *inode, char *name)
         if (!mdc)
                 goto out;
 
-        if (!name)
+        if (!name || !mdc->xattr)
                 goto out;
 
         LOCK (&mdc->lock);
@@ -800,15 +835,15 @@ mdc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
         if (!local)
                 goto uncached;
 
+        loc_copy (&local->loc, loc);
+
 	if (!loc->name)
-		/* A nameless discovery is dangerous to cache. We
+		/* A nameless discovery is dangerous to serve from cache. We
 		   perform nameless lookup with the intention of
 		   re-establishing an inode "properly"
 		*/
 		goto uncached;
 
-        loc_copy (&local->loc, loc);
-
         ret = mdc_inode_iatt_get (this, loc->inode, &stbuf);
         if (ret != 0)
                 goto uncached;
@@ -1435,7 +1470,7 @@ mdc_readv_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         local = frame->local;
 
-        if (op_ret != 0)
+        if (op_ret < 0)
                 goto out;
 
         if (!local)
@@ -1730,7 +1765,7 @@ mdc_getxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 {
         mdc_local_t  *local = NULL;
 
-        if (op_ret != 0)
+        if (op_ret < 0)
                 goto out;
 
         local = frame->local;
@@ -1792,7 +1827,7 @@ mdc_fgetxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 {
         mdc_local_t  *local = NULL;
 
-        if (op_ret != 0)
+        if (op_ret < 0)
                 goto out;
 
         local = frame->local;
@@ -2189,6 +2224,7 @@ reconfigure (xlator_t *this, dict_t *options)
 
 	GF_OPTION_RECONF ("cache-posix-acl", conf->cache_posix_acl, options, bool, out);
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
 
 	GF_OPTION_RECONF("force-readdirp", conf->force_readdirp, options, bool, out);
 
@@ -2212,8 +2248,8 @@ init (xlator_t *this)
 
 	conf = GF_CALLOC (sizeof (*conf), 1, gf_mdc_mt_mdc_conf_t);
 	if (!conf) {
-		gf_log (this->name, GF_LOG_ERROR,
-			"out of memory");
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                        MD_CACHE_MSG_NO_MEMORY, "out of memory");
 		return -1;
 	}
 
@@ -2224,6 +2260,7 @@ init (xlator_t *this)
 
 	GF_OPTION_INIT ("cache-posix-acl", conf->cache_posix_acl, bool, out);
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
+	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
 
 	GF_OPTION_INIT("force-readdirp", conf->force_readdirp, bool, out);
 out:
