@@ -13,6 +13,7 @@
 #include "statedump.h"
 #include "call-stub.h"
 #include "defaults.h"
+#include "open-behind-messages.h"
 
 typedef struct ob_conf {
 	gf_boolean_t  use_anonymous_fd; /* use anonymous FDs wherever safe
@@ -23,6 +24,11 @@ typedef struct ob_conf {
 					   like mandatory locks
 					*/
 	gf_boolean_t  lazy_open; /* delay backend open as much as possible */
+        gf_boolean_t  read_after_open; /* instead of sending readvs on
+                                               anonymous fds, open the file
+                                               first and then send readv i.e
+                                               similar to what writev does
+                                            */
 } ob_conf_t;
 
 
@@ -335,8 +341,8 @@ ob_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
 
 	return 0;
 err:
-	gf_log (this->name, GF_LOG_ERROR, "%s: %s", loc->path,
-		strerror (op_errno));
+	gf_msg (this->name, GF_LOG_ERROR, op_errno, OPEN_BEHIND_MSG_NO_MEMORY,
+                "%s", loc->path);
 
 	STACK_UNWIND_STRICT (open, frame, -1, op_errno, 0, 0);
 
@@ -367,8 +373,14 @@ ob_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
 {
 	call_stub_t  *stub = NULL;
 	fd_t         *wind_fd = NULL;
+        ob_conf_t    *conf = NULL;
 
-	wind_fd = ob_get_wind_fd (this, fd);
+        conf = this->private;
+
+        if (!conf->read_after_open)
+                wind_fd = ob_get_wind_fd (this, fd);
+        else
+                wind_fd = fd_ref (fd);
 
 	stub = fop_readv_stub (frame, default_readv_resume, wind_fd,
 			       size, offset, flags, xdata);
@@ -876,7 +888,9 @@ mem_acct_init (xlator_t *this)
         ret = xlator_mem_acct_init (this, gf_ob_mt_end + 1);
 
         if (ret)
-                gf_log (this->name, GF_LOG_ERROR, "Memory accounting failed");
+                gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                        OPEN_BEHIND_MSG_NO_MEMORY,
+                        "Memory accounting failed");
 
         return ret;
 }
@@ -894,6 +908,8 @@ reconfigure (xlator_t *this, dict_t *options)
 			  bool, out);
 
         GF_OPTION_RECONF ("lazy-open", conf->lazy_open, options, bool, out);
+        GF_OPTION_RECONF ("read-after-open", conf->read_after_open, options,
+                          bool, out);
 
         ret = 0;
 out:
@@ -907,14 +923,16 @@ init (xlator_t *this)
         ob_conf_t    *conf = NULL;
 
         if (!this->children || this->children->next) {
-                gf_log (this->name, GF_LOG_ERROR,
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        OPEN_BEHIND_MSG_XLATOR_CHILD_MISCONFIGURED,
                         "FATAL: volume (%s) not configured with exactly one "
                         "child", this->name);
                 return -1;
         }
 
         if (!this->parents)
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, 0,
+                        OPEN_BEHIND_MSG_VOL_MISCONFIGURED,
                         "dangling volume. check volfile ");
 
         conf = GF_CALLOC (1, sizeof (*conf), gf_ob_mt_conf_t);
@@ -924,7 +942,7 @@ init (xlator_t *this)
         GF_OPTION_INIT ("use-anonymous-fd", conf->use_anonymous_fd, bool, err);
 
         GF_OPTION_INIT ("lazy-open", conf->lazy_open, bool, err);
-
+        GF_OPTION_INIT ("read-after-open", conf->read_after_open, bool, err);
         this->private = conf;
 
 	return 0;
@@ -996,6 +1014,12 @@ struct volume_options options[] = {
           "FOP arrives (e.g writev on the FD, unlink of the file). When option "
           "is disabled, perform backend open right after unwinding open().",
         },
+        { .key  = {"read-after-open"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "no",
+          .description = "read is sent only after actual open happens and real "
+          "fd is obtained, instead of doing on anonymous fd (similar to write)",
+        },
         { .key  = {NULL} }
 
 };
