@@ -19,6 +19,7 @@
 #include "xlator.h"
 #include "read-ahead.h"
 #include <assert.h>
+#include "read-ahead-messages.h"
 
 ra_page_t *
 ra_page_get (ra_file_t *file, off_t offset)
@@ -155,7 +156,8 @@ ra_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         pending_offset = local->pending_offset;
 
         if (file == NULL) {
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, EBADF,
+                        READ_AHEAD_MSG_FD_CONTEXT_NOT_SET,
                         "read-ahead context not set in fd (%p)", fd);
                 op_ret = -1;
                 op_errno = EBADF;
@@ -170,9 +172,10 @@ ra_fault_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 page = ra_page_get (file, pending_offset);
 
                 if (!page) {
-                        gf_log (this->name, GF_LOG_TRACE,
-                                "wasted copy: %"PRId64"[+%"PRId64"] file=%p",
-                                pending_offset, file->page_size, file);
+                        gf_msg_trace (this->name, 0,
+                                      "wasted copy: "
+                                      "%"PRId64"[+%"PRId64"] file=%p",
+                                      pending_offset, file->page_size, file);
                         goto unlock;
                 }
 
@@ -421,7 +424,12 @@ ra_frame_unwind (call_frame_t *frame)
                                 fill->count * sizeof (*vector));
 
                         copied += (fill->count * sizeof (*vector));
-                        iobref_merge (iobref, fill->iobref);
+                        if (iobref_merge (iobref, fill->iobref)) {
+				local->op_ret = -1;
+				local->op_errno = ENOMEM;
+				iobref_unref (iobref);
+				iobref = NULL;
+			}
                 }
 
                 fill->next->prev = fill->prev;
