@@ -1,20 +1,11 @@
 /*
-  Copyright (c) 2006-2011 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2012 Red Hat, Inc. <http://www.redhat.com>
   This file is part of GlusterFS.
 
-  GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
-  by the Free Software Foundation; either version 3 of the License,
-  or (at your option) any later version.
-
-  GlusterFS is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program.  If not, see
-  <http://www.gnu.org/licenses/>.
+  This file is licensed to you under your choice of the GNU Lesser
+  General Public License, version 3 or any later version (LGPLv3 or
+  later), or the GNU General Public License, version 2 (GPLv2), in all
+  cases as published by the Free Software Foundation.
 */
 
 #ifndef __READ_AHEAD_H
@@ -76,7 +67,8 @@ struct ra_page {
         struct ra_page   *next;
         struct ra_page   *prev;
         struct ra_file   *file;
-        char              dirty;
+        char              dirty;    /* Internal request, not from user. */
+        char              poisoned; /* Pending read invalidated by write. */
         char              ready;
         struct iovec     *vector;
         int32_t           count;
@@ -84,6 +76,7 @@ struct ra_page {
         size_t            size;
         struct ra_waitq  *waitq;
         struct iobref    *iobref;
+        char              stale;
 };
 
 
