@@ -48,7 +48,8 @@ typedef enum {
                         goto label;                                     \
                 }                                                       \
                 if (remote_fd == -1) {                                  \
-                        gf_log (xl->name, GF_LOG_WARNING, " (%s) "      \
+                        gf_msg (xl->name, GF_LOG_WARNING, EBADFD,       \
+                                PC_MSG_BAD_FD, " (%s) "                 \
                                 "remote_fd is -1. EBADFD",              \
                                 uuid_utoa (fd->inode->gfid));           \
                         op_errno = EBADFD;                              \
@@ -57,6 +58,8 @@ typedef enum {
         } while (0)
 
 #define CLIENT_STACK_UNWIND(op, frame, params ...) do {             \
+                if (!frame)                                         \
+                        break;                                      \
                 clnt_local_t *__local = frame->local;               \
                 frame->local = NULL;                                \
                 STACK_UNWIND_STRICT (op, frame, params);            \
@@ -83,6 +86,7 @@ typedef struct clnt_conf {
         rpc_clnt_prog_t       *handshake;
         rpc_clnt_prog_t       *dump;
 
+        int                    client_id;
         uint64_t               reopen_fd_count; /* Count of fds reopened after a
                                                    connection is established */
         gf_lock_t              rec_lock;
@@ -96,10 +100,6 @@ typedef struct clnt_conf {
         char                   disconnect_err_logged; /* flag used to prevent
                                                          excessive disconnect
                                                          logging */
-
-        char                   need_different_port; /* flag used to change the
-                                                       portmap path in case of
-                                                       'tcp,rdma' on server */
         gf_boolean_t           lk_heal;
         uint16_t               lk_version; /* this variable is used to distinguish
                                               client-server transaction while
@@ -120,12 +120,19 @@ typedef struct clnt_conf {
 						*/
         gf_boolean_t           filter_o_direct; /* if set, filter O_DIRECT from
                                                    the flags list of open() */
-        gf_boolean_t           send_gids; /* let the server resolve gids */
         /* set volume is the op which results in creating/re-using
          * the conn-id and is called once per connection, this remembers
          * how manytimes set_volume is called
          */
         uint64_t               setvol_count;
+
+        gf_boolean_t           send_gids; /* let the server resolve gids */
+
+        int                     event_threads; /* # of event threads
+                                                * configured */
+
+        gf_boolean_t           destroy; /* if enabled implies fini was called
+                                         * on @this xlator instance */
 } clnt_conf_t;
 
 typedef struct _client_fd_ctx {
@@ -223,7 +230,8 @@ int client_submit_request (xlator_t *this, void *req,
                            struct iovec *rsp_payload, int rsp_count,
                            struct iobref *rsp_iobref, xdrproc_t xdrproc);
 
-int unserialize_rsp_dirent (struct gfs3_readdir_rsp *rsp, gf_dirent_t *entries);
+int unserialize_rsp_dirent (xlator_t *this, struct gfs3_readdir_rsp *rsp,
+                            gf_dirent_t *entries);
 int unserialize_rsp_direntp (xlator_t *this, fd_t *fd,
                              struct gfs3_readdirp_rsp *rsp, gf_dirent_t *entries);
 
@@ -260,4 +268,6 @@ int client_fd_fop_prepare_local (call_frame_t *frame, fd_t *fd,
                                  int64_t remote_fd);
 gf_boolean_t
 __is_fd_reopen_in_progress (clnt_fd_ctx_t *fdctx);
+int
+client_notify_dispatch (xlator_t *this, int32_t event, void *data, ...);
 #endif /* !_CLIENT_H */
