@@ -16,6 +16,7 @@
 #include "server.h"
 #include "server-helpers.h"
 #include "gidcache.h"
+#include "server-messages.h"
 
 #include <fnmatch.h>
 #include <pwd.h>
@@ -42,25 +43,27 @@ gid_resolve (server_conf_t *conf, call_stack_t *root)
 
         ret = getpwuid_r (root->uid, &mypw, mystrs, sizeof(mystrs), &result);
         if (ret != 0) {
-                gf_log("gid-cache", GF_LOG_ERROR, "getpwuid_r(%u) failed",
-                       root->uid);
+                gf_msg ("gid-cache", GF_LOG_ERROR, errno,
+                        PS_MSG_GET_UID_FAILED, "getpwuid_r(%u) failed",
+                        root->uid);
                 return -1;
         }
 
         if (!result) {
-                gf_log ("gid-cache", GF_LOG_ERROR, "getpwuid_r(%u) found "
-                        "nothing", root->uid);
+                gf_msg ("gid-cache", GF_LOG_ERROR, 0, PS_MSG_UID_NOT_FOUND,
+                        "getpwuid_r(%u) found nothing", root->uid);
                 return -1;
         }
 
-        gf_log ("gid-cache", GF_LOG_TRACE, "mapped %u => %s", root->uid,
-                result->pw_name);
+        gf_msg_trace ("gid-cache", 0, "mapped %u => %s", root->uid,
+                      result->pw_name);
 
         ngroups = GF_MAX_AUX_GROUPS;
         ret = getgrouplist (result->pw_name, root->gid, mygroups, &ngroups);
         if (ret == -1) {
-                gf_log ("gid-cache", GF_LOG_ERROR, "could not map %s to group "
-                        "list (%d gids)", result->pw_name, root->ngrps);
+                gf_msg ("gid-cache", GF_LOG_ERROR, 0, PS_MSG_MAPPING_ERROR,
+                        "could not map %s to group list (%d gids)",
+                        result->pw_name, root->ngrps);
                 return -1;
         }
         root->ngrps = (uint16_t) ngroups;
@@ -142,6 +145,7 @@ server_decode_groups (call_frame_t *frame, rpcsvc_request_t *req)
         if (frame->root->ngrps == 0)
                 return 0;
 
+	/* ngrps cannot be bigger than USHRT_MAX(65535) */
         if (frame->root->ngrps > GF_MAX_AUX_GROUPS)
                 return -1;
 
@@ -285,12 +289,14 @@ do_fd_cleanup (xlator_t *this, client_t* client, fdentry_t *fdentries, int fd_co
                         ret = inode_path (fd->inode, NULL, &path);
 
                         if (ret > 0) {
-                                gf_log (this->name, GF_LOG_INFO,
+                                gf_msg (this->name, GF_LOG_INFO, 0,
+                                        PS_MSG_FD_CLEANUP,
                                         "fd cleanup on %s", path);
                                 GF_FREE (path);
                         }  else {
 
-                                gf_log (this->name, GF_LOG_INFO,
+                                gf_msg (this->name, GF_LOG_INFO, 0,
+                                        PS_MSG_FD_CLEANUP,
                                         "fd cleanup on inode with gfid %s",
                                         uuid_utoa (fd->inode->gfid));
                         }
@@ -333,7 +339,9 @@ server_connection_cleanup (xlator_t *this, client_t *client,
         serv_ctx = server_ctx_get (client, client->this);
 
         if (serv_ctx == NULL) {
-                gf_log (this->name, GF_LOG_INFO, "server_ctx_get() failed");
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        PS_MSG_SERVER_CTX_GET_FAILED, "server_ctx_get() "
+                        "failed");
                 goto out;
         }
 
@@ -355,7 +363,8 @@ server_connection_cleanup (xlator_t *this, client_t *client,
         if (fdentries != NULL)
                 ret = do_fd_cleanup (this, client, fdentries, fd_count);
         else
-                gf_log (this->name, GF_LOG_INFO, "no fdentries to clean");
+                gf_msg (this->name, GF_LOG_INFO, 0, PS_MSG_FDENTRY_NULL,
+                        "no fdentries to clean");
 
         if (cd_ret || ret)
                 ret = -1;
@@ -410,7 +419,11 @@ get_frame_from_request (rpcsvc_request_t *req)
 {
         call_frame_t  *frame = NULL;
         client_t      *client = NULL;
-        server_conf_t *conf = NULL;
+        client_t      *tmp_client = NULL;
+        xlator_t  *this = NULL;
+        server_conf_t *priv = NULL;
+        clienttable_t *clienttable = NULL;
+        unsigned int   i           = 0;
 
         GF_VALIDATE_OR_GOTO ("server", req, out);
 
@@ -424,6 +437,59 @@ get_frame_from_request (rpcsvc_request_t *req)
 
         frame->root->unique   = req->xid;
 
+        client = req->trans->xl_private;
+        this = req->trans->xl;
+        priv = this->private;
+        clienttable = this->ctx->clienttable;
+
+        for (i = 0; i < clienttable->max_clients; i++) {
+                tmp_client = clienttable->cliententries[i].client;
+                if (client == tmp_client) {
+                        /* for non trusted clients username and password
+                           would not have been set. So for non trusted clients
+                           (i.e clients not from the same machine as the brick,
+                           and clients from outside the storage pool)
+                           do the root-squashing.
+                           TODO: If any client within the storage pool (i.e
+                           mounting within a machine from the pool but using
+                           other machine's ip/hostname from the same pool)
+                           is present treat it as a trusted client
+                        */
+                        if (!client->auth.username && req->pid != NFS_PID)
+                                RPC_AUTH_ROOT_SQUASH (req);
+
+                        /* Problem: If we just check whether the client is
+                           trusted client and do not do root squashing for
+                           them, then for smb clients and UFO clients root
+                           squashing will never happen as they use the fuse
+                           mounts done within the trusted pool (i.e they are
+                           trusted clients).
+                           Solution: To fix it, do root squashing for trusted
+                           clients also. If one wants to have a client within
+                           the storage pool for which root-squashing does not
+                           happen, then the client has to be mounted with
+                           --no-root-squash option. But for defrag client and
+                           gsyncd client do not do root-squashing.
+                        */
+                        if (client->auth.username &&
+                            req->pid != GF_CLIENT_PID_NO_ROOT_SQUASH &&
+                            req->pid != GF_CLIENT_PID_GSYNCD &&
+                            req->pid != GF_CLIENT_PID_DEFRAG &&
+                            req->pid != GF_CLIENT_PID_AFR_SELF_HEALD &&
+                            req->pid != GF_CLIENT_PID_QUOTA_MOUNT)
+                                RPC_AUTH_ROOT_SQUASH (req);
+
+                        /* For nfs clients the server processes will be running
+                           within the trusted storage pool machines. So if we
+                           do not do root-squashing for nfs servers, thinking
+                           that its a trusted client, then root-squashing wont
+                           work for nfs clients.
+                        */
+                        if (req->pid == NFS_PID)
+                                RPC_AUTH_ROOT_SQUASH (req);
+                }
+        }
+
         frame->root->uid      = req->uid;
         frame->root->gid      = req->gid;
         frame->root->pid      = req->pid;
@@ -431,8 +497,7 @@ get_frame_from_request (rpcsvc_request_t *req)
         frame->root->client   = client;
         frame->root->lk_owner = req->lk_owner;
 
-        conf = frame->this->private;
-        if (conf->server_manage_gids)
+        if (priv->server_manage_gids)
             server_resolve_groups (frame, req);
         else
             server_decode_groups (frame, req);
@@ -464,9 +529,9 @@ server_build_config (xlator_t *this, server_conf_t *conf)
         if (data) {
                 ret = gf_string2boolean(data->data, &conf->verify_volfile);
                 if (ret != 0) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "wrong value for 'verify-volfile-checksum', "
-                                "Neglecting option");
+                        gf_msg (this->name, GF_LOG_WARNING, EINVAL,
+                                PS_MSG_INVALID_ENTRY, "wrong value for '"
+                                "verify-volfile-checksum', Neglecting option");
                 }
         }
 
@@ -474,9 +539,9 @@ server_build_config (xlator_t *this, server_conf_t *conf)
         if (data) {
                 ret = gf_string2boolean (data->data, &conf->trace);
                 if (ret != 0) {
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "'trace' takes on only boolean values. "
-                                "Neglecting option");
+                        gf_msg (this->name, GF_LOG_WARNING, EINVAL,
+                                PS_MSG_INVALID_ENTRY, "'trace' takes on only "
+                                "boolean values. Neglecting option");
                 }
         }
 
@@ -484,9 +549,8 @@ server_build_config (xlator_t *this, server_conf_t *conf)
         ret = dict_get_int32 (this->options, "limits.transaction-size",
                               &conf->rpc_conf.max_block_size);
         if (ret < 0) {
-                gf_log (this->name, GF_LOG_TRACE,
-                        "defaulting limits.transaction-size to %d",
-                        DEFAULT_BLOCK_SIZE);
+                gf_msg_trace (this->name, 0, "defaulting limits.transaction-"
+                              "size to %d", DEFAULT_BLOCK_SIZE);
                 conf->rpc_conf.max_block_size = DEFAULT_BLOCK_SIZE;
         }
 
@@ -496,16 +560,17 @@ server_build_config (xlator_t *this, server_conf_t *conf)
                    or directory specified is non standard */
                 ret = stat (data->data, &buf);
                 if ((ret != 0) || !S_ISDIR (buf.st_mode)) {
-                        gf_log (this->name, GF_LOG_ERROR,
-                                "Directory '%s' doesn't exist, exiting.",
-                                data->data);
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                PS_MSG_DIR_NOT_FOUND, "Directory '%s' doesn't "
+                                "exist, exiting.", data->data);
                         ret = -1;
                         goto out;
                 }
                 /* Make sure that conf-dir doesn't contain ".." in path */
                 if ((gf_strstr (data->data, "/", "..")) == -1) {
                         ret = -1;
-                        gf_log (this->name, GF_LOG_ERROR,
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                PS_MSG_CONF_DIR_INVALID,
                                 "%s: invalid conf_dir", data->data);
                         goto out;
                 }
@@ -702,9 +767,8 @@ server_print_reply (call_frame_t *frame, int op_ret, int op_errno)
         if (state->fd)
                 snprintf (fdstr, 32, " fd=%p", state->fd);
 
-        gf_log (this->name, GF_LOG_INFO,
-                "%s%s => (%d, %d)%s",
-                op, caller, op_ret, op_errno, fdstr);
+        gf_msg (this->name, GF_LOG_INFO, op_errno, PS_MSG_SERVER_MSG,
+                "%s%s => (%d, %d)%s", op, caller, op_ret, op_errno, fdstr);
 out:
         return;
 }
@@ -765,9 +829,8 @@ server_print_request (call_frame_t *frame)
                 break;
         }
 
-        gf_log (this->name, GF_LOG_INFO,
-                "%s%s%s%s%s%s%s",
-                op, caller,
+        gf_msg (this->name, GF_LOG_INFO, 0, PS_MSG_SERVER_MSG,
+                "%s%s%s%s%s%s%s", op, caller,
                 resolve_vars, loc_vars, resolve2_vars, loc2_vars, other_vars);
 out:
         return;
@@ -801,10 +864,10 @@ serialize_rsp_direntp (gf_dirent_t *entries, gfs3_readdirp_rsp *rsp)
                 /* if 'dict' is present, pack it */
                 if (entry->dict) {
                         trav->dict.dict_len = dict_serialized_length (entry->dict);
-                        if (trav->dict.dict_len < 0) {
-                                gf_log (THIS->name, GF_LOG_ERROR,
-                                        "failed to get serialized length "
-                                        "of reply dict");
+                        if (trav->dict.dict_len > UINT_MAX) {
+                                gf_msg (THIS->name, GF_LOG_ERROR, EINVAL,
+                                        PS_MSG_INVALID_ENTRY, "failed to get "
+                                        "serialized length of reply dict");
                                 errno = EINVAL;
                                 trav->dict.dict_len = 0;
                                 goto out;
@@ -820,7 +883,8 @@ serialize_rsp_direntp (gf_dirent_t *entries, gfs3_readdirp_rsp *rsp)
 
                         ret = dict_serialize (entry->dict, trav->dict.dict_val);
                         if (ret < 0) {
-                                gf_log (THIS->name, GF_LOG_ERROR,
+                                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                        PS_MSG_DICT_SERIALIZE_FAIL,
                                         "failed to serialize reply dict");
                                 errno = -ret;
                                 trav->dict.dict_len = 0;
@@ -932,7 +996,8 @@ gf_server_check_getxattr_cmd (call_frame_t *frame, const char *key)
                 pthread_mutex_lock (&conf->mutex);
                 {
                         list_for_each_entry (xprt, &conf->xprt_list, list) {
-                                gf_log ("mount-point-list", GF_LOG_INFO,
+                                gf_msg ("mount-point-list", GF_LOG_INFO, 0,
+                                        PS_MSG_MOUNT_PT_FAIL,
                                         "%s", xprt->peerinfo.identifier);
                         }
                 }
@@ -964,7 +1029,7 @@ gf_server_check_setxattr_cmd (call_frame_t *frame, dict_t *dict)
                         total_read  += xprt->total_bytes_read;
                         total_write += xprt->total_bytes_write;
                 }
-                gf_log ("stats", GF_LOG_INFO,
+                gf_msg ("stats", GF_LOG_INFO, 0, PS_MSG_RW_STAT,
                         "total-read %"PRIu64", total-write %"PRIu64,
                         total_read, total_write);
         }
@@ -981,7 +1046,7 @@ server_cancel_grace_timer (xlator_t *this, client_t *client)
         gf_boolean_t   cancelled = _gf_false;
 
         if (!this || !client) {
-                gf_log (THIS->name, GF_LOG_ERROR,
+                gf_msg (THIS->name, GF_LOG_ERROR, EINVAL, PS_MSG_INVALID_ENTRY,
                         "Invalid arguments to cancel connection timer");
                 return cancelled;
         }
@@ -989,7 +1054,9 @@ server_cancel_grace_timer (xlator_t *this, client_t *client)
         serv_ctx = server_ctx_get (client, client->this);
 
         if (serv_ctx == NULL) {
-                gf_log (this->name, GF_LOG_INFO, "server_ctx_get() failed");
+                gf_msg (this->name, GF_LOG_INFO, 0,
+                        PS_MSG_SERVER_CTX_GET_FAILED,
+                        "server_ctx_get() failed");
                 goto out;
         }
 
@@ -1049,3 +1116,107 @@ server_ctx_get (client_t *client, xlator_t *xlator)
 out:
         return ctx;
 }
+
+int
+auth_set_username_passwd (dict_t *input_params, dict_t *config_params,
+                          client_t *client)
+{
+        int      ret           = 0;
+        data_t  *allow_user    = NULL;
+        data_t  *passwd_data   = NULL;
+        char    *username      = NULL;
+        char    *password      = NULL;
+        char    *brick_name    = NULL;
+        char    *searchstr     = NULL;
+        char    *username_str  = NULL;
+        char    *tmp           = NULL;
+        char    *username_cpy  = NULL;
+
+        ret = dict_get_str (input_params, "username", &username);
+        if (ret) {
+                gf_msg_debug ("auth/login", 0, "username not found, returning "
+                              "DONT-CARE");
+                /* For non trusted clients username and password
+                   will not be there. So dont reject the client.
+                */
+                ret = 0;
+                goto out;
+        }
+
+        ret = dict_get_str (input_params, "password", &password);
+        if (ret) {
+                gf_msg ("auth/login", GF_LOG_WARNING, 0,
+                        PS_MSG_DICT_GET_FAILED,
+                        "password not found, returning DONT-CARE");
+                goto out;
+        }
+
+        ret = dict_get_str (input_params, "remote-subvolume", &brick_name);
+        if (ret) {
+                gf_msg ("auth/login", GF_LOG_ERROR, 0, PS_MSG_DICT_GET_FAILED,
+                        "remote-subvolume not specified");
+                ret = -1;
+                goto out;
+        }
+
+        ret = gf_asprintf (&searchstr, "auth.login.%s.allow", brick_name);
+        if (-1 == ret) {
+                ret = 0;
+                goto out;
+        }
+
+        allow_user = dict_get (config_params, searchstr);
+        GF_FREE (searchstr);
+
+        if (allow_user) {
+                username_cpy = gf_strdup (allow_user->data);
+                if (!username_cpy)
+                        goto out;
+
+                username_str = strtok_r (username_cpy, " ,", &tmp);
+
+                while (username_str) {
+                        if (!fnmatch (username_str, username, 0)) {
+                                ret = gf_asprintf (&searchstr,
+                                                   "auth.login.%s.password",
+                                                   username);
+                                if (-1 == ret)
+                                        goto out;
+
+                                passwd_data = dict_get (config_params,
+                                                        searchstr);
+                                GF_FREE (searchstr);
+
+                                if (!passwd_data) {
+                                        gf_msg ("auth/login", GF_LOG_ERROR, 0,
+                                                PS_MSG_LOGIN_ERROR, "wrong "
+                                                "username/password "
+                                                "combination");
+                                        ret = -1;
+                                        goto out;
+                                }
+
+                                ret = !((strcmp (data_to_str (passwd_data),
+                                                    password))?0: -1);
+                                if (!ret) {
+                                        client->auth.username =
+                                                gf_strdup (username);
+                                        client->auth.passwd =
+                                                gf_strdup (password);
+                                }
+                                if (ret == -1)
+                                        gf_msg ("auth/login", GF_LOG_ERROR, 0,
+                                                PS_MSG_LOGIN_ERROR, "wrong "
+                                                "password for user %s",
+                                                username);
+                                break;
+                        }
+                        username_str = strtok_r (NULL, " ,", &tmp);
+                }
+        }
+
+out:
+        GF_FREE (username_cpy);
+
+        return ret;
+}
