@@ -107,7 +107,8 @@ gfs_serialize_reply (rpcsvc_request_t *req, void *arg, struct iovec *outmsg,
                 xdr_size = xdr_sizeof (xdrproc, arg);
                 iob = iobuf_get2 (req->svc->ctx->iobuf_pool, xdr_size);
                 if (!iob) {
-                        gf_log_callingfn (THIS->name, GF_LOG_ERROR,
+                        gf_msg_callingfn (THIS->name, GF_LOG_ERROR, ENOMEM,
+                                          PS_MSG_NO_MEMORY,
                                           "Failed to get iobuf");
                         goto ret;
                 };
@@ -125,7 +126,9 @@ gfs_serialize_reply (rpcsvc_request_t *req, void *arg, struct iovec *outmsg,
                         /* Failed to Encode 'GlusterFS' msg in RPC is not exactly
                            failure of RPC return values.. client should get
                            notified about this, so there are no missing frames */
-                        gf_log_callingfn ("", GF_LOG_ERROR, "Failed to encode message");
+                        gf_msg_callingfn ("", GF_LOG_ERROR, 0,
+                                          PS_MSG_ENCODE_MSG_FAILED,
+                                          "Failed to encode message");
                         req->rpc_err = GARBAGE_ARGS;
                         retlen = 0;
                 }
@@ -197,12 +200,15 @@ server_submit_reply (call_frame_t *frame, rpcsvc_request_t *req, void *arg,
          */
         iobuf_unref (iob);
         if (ret == -1) {
-                gf_log_callingfn ("", GF_LOG_ERROR, "Reply submission failed");
+                gf_msg_callingfn ("", GF_LOG_ERROR, 0,
+                                  PS_MSG_REPLY_SUBMIT_FAILED,
+                                  "Reply submission failed");
                 if (frame && client && !lk_heal) {
                         server_connection_cleanup (frame->this, client,
                                                   INTERNAL_LOCKS | POSIX_LOCKS);
                 } else {
-                        gf_log_callingfn ("", GF_LOG_ERROR,
+                        gf_msg_callingfn ("", GF_LOG_ERROR, 0,
+                                          PS_MSG_REPLY_SUBMIT_FAILED,
                                           "Reply submission failed");
                         /* TODO: Failure of open(dir), create, inodelk, entrylk
                            or lk fops send failure must be handled specially. */
@@ -488,7 +494,8 @@ server_rpc_notify (rpcsvc_t *rpc, void *xl, rpcsvc_event_t event,
         server_ctx_t        *serv_ctx   = NULL;
 
         if (!xl || !data) {
-                gf_log_callingfn ("server", GF_LOG_WARNING,
+                gf_msg_callingfn ("server", GF_LOG_WARNING, 0,
+                                  PS_MSG_RPC_NOTIFY_ERROR,
                                   "Calling rpc_notify without initializing");
                 goto out;
         }
@@ -718,6 +725,7 @@ reconfigure (xlator_t *this, dict_t *options)
         server_conf_t            *conf =NULL;
         rpcsvc_t                 *rpc_conf;
         rpcsvc_listener_t        *listeners;
+        rpc_transport_t          *xprt = NULL;
         int                       inode_lru_limit;
         gf_boolean_t              trace;
         data_t                   *data;
@@ -729,7 +737,8 @@ reconfigure (xlator_t *this, dict_t *options)
         conf = this->private;
 
         if (!conf) {
-                gf_log_callingfn (this->name, GF_LOG_DEBUG, "conf == null!!!");
+                gf_msg_callingfn (this->name, GF_LOG_DEBUG, EINVAL,
+                                  PS_MSG_INVALID_ENTRY, "conf == null!!!");
                 goto out;
         }
 
@@ -785,6 +794,7 @@ reconfigure (xlator_t *this, dict_t *options)
                 /* logging already done in validate_auth_options function. */
                 goto out;
         }
+
         dict_foreach (this->options, _delete_auth_opt, this->options);
         dict_foreach (options, _copy_auth_opt, this->options);
 
@@ -812,8 +822,41 @@ reconfigure (xlator_t *this, dict_t *options)
                 goto out;
         }
 
-        (void) rpcsvc_set_allow_insecure (rpc_conf, options);
-        (void) rpcsvc_set_root_squash (rpc_conf, options);
+        ret = rpcsvc_auth_reconf (rpc_conf, options);
+        if (ret == -1) {
+                gf_log (GF_RPCSVC, GF_LOG_ERROR,
+                                "Failed to reconfigure authentication");
+                goto out;
+        }
+
+        GF_OPTION_RECONF ("dynamic-auth", conf->dync_auth, options,
+                        bool, out);
+
+        if (conf->dync_auth) {
+                pthread_mutex_lock (&conf->mutex);
+                {
+                        list_for_each_entry (xprt, &conf->xprt_list, list) {
+                                /* check for client authorization */
+                                ret = gf_authenticate (xprt->clnt_options,
+                                                options, conf->auth_modules);
+                                if (ret == AUTH_ACCEPT) {
+                                        gf_msg (this->name, GF_LOG_TRACE, 0,
+                                               PS_MSG_CLIENT_ACCEPTED,
+                                               "authorized client, hence we "
+                                               "continue with this connection");
+                                } else {
+                                        gf_msg (this->name, GF_LOG_INFO,
+                                                EACCES,
+                                                PS_MSG_AUTHENTICATE_ERROR,
+                                                "unauthorized client, hence "
+                                                "terminating the connection %s",
+                                                xprt->peerinfo.identifier);
+                                        rpc_transport_disconnect(xprt);
+                                }
+                        }
+                }
+                pthread_mutex_unlock (&conf->mutex);
+        }
 
         ret = rpcsvc_set_outstanding_rpc_limit (rpc_conf, options,
                                          RPCSVC_DEFAULT_OUTSTANDING_RPC_LIMIT);
@@ -964,6 +1007,12 @@ init (xlator_t *this)
                         "Failed to initialize group cache.");
                 goto out;
         }
+        ret = dict_get_str_boolean (this->options, "dynamic-auth",
+                        _gf_true);
+        if (ret == -1)
+                conf->dync_auth = _gf_true;
+        else
+                conf->dync_auth = ret;
 
         /* RPC related */
         conf->rpc = rpcsvc_init (this, this->ctx, this->options, 0);
@@ -1158,7 +1207,8 @@ server_process_event_upcall (xlator_t *this, void *data)
                 xdrproc = (xdrproc_t)xdr_gfs3_cbk_cache_invalidation_req;
                 break;
         default:
-                gf_log (this->name, GF_LOG_WARNING,
+                gf_msg (this->name, GF_LOG_WARNING, EINVAL,
+                        PS_MSG_INVALID_ENTRY,
                         "Received invalid upcall event(%d)",
                         upcall_data->event_type);
                 goto out;
@@ -1211,7 +1261,8 @@ notify (xlator_t *this, int32_t event, void *data, ...)
 
                 ret = server_process_event_upcall (this, data);
                 if (ret) {
-                        gf_log (this->name, GF_LOG_ERROR,
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                PS_MSG_SERVER_EVENT_UPCALL_FAILED,
                                 "server_process_event_upcall failed");
                         goto out;
                 }
@@ -1368,7 +1419,6 @@ struct volume_options options[] = {
                          "requests from a client. 0 means no limit (can "
                          "potentially run out of memory)"
         },
-
         { .key   = {"manage-gids"},
           .type  = GF_OPTION_TYPE_BOOL,
           .default_value = "off",
@@ -1389,6 +1439,13 @@ struct volume_options options[] = {
                          " responses faster, depending on available processing"
                          " power. Range 1-32 threads."
         },
-
+        { .key   = {"dynamic-auth"},
+          .type  = GF_OPTION_TYPE_BOOL,
+          .default_value = "on",
+          .description   = "When 'on' perform dynamic authentication of volume "
+                           "options in order to allow/terminate client "
+                           "transport connection immediately in response to "
+                           "*.allow | *.reject volume set options."
+        },
         { .key   = {NULL} },
 };
