@@ -32,11 +32,14 @@
 
 #include "bd.h"
 #include "bd-aio.h"
+#include "bd-mem-types.h"
 #include "defaults.h"
 #include "glusterfs3-xdr.h"
 #include "run.h"
 #include "protocol-common.h"
 #include "checksum.h"
+#include "syscall.h"
+#include "lvm-defaults.h"
 
 /*
  * Call back function for setxattr and removexattr.
@@ -213,7 +216,7 @@ bd_forget (xlator_t *this, inode_t *inode)
         ret = bd_inode_ctx_get (inode, this, &bdatt);
         if (!ret) {
                 inode_ctx_del (inode, this, &ctx);
-                FREE (bdatt);
+                GF_FREE (bdatt);
         }
         return 0;
 }
@@ -236,7 +239,7 @@ bd_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
                                      entry->d_stat.ia_gfid, &type, &size)) {
                         entry->d_stat.ia_size = size;
                         entry->d_stat.ia_blocks = size / 512;
-                        FREE (type);
+                        GF_FREE (type);
                 }
         }
 
@@ -653,7 +656,7 @@ bd_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
                 goto posix;
 
         uuid_utoa_r (fd->inode->gfid, gfid);
-        asprintf (&devpath, "/dev/%s/%s", priv->vg, gfid);
+        gf_asprintf (&devpath, "/dev/%s/%s", priv->vg, gfid);
         BD_VALIDATE_MEM_ALLOC (devpath, ret, out);
 
         _fd = open (devpath, flags | O_LARGEFILE, 0);
@@ -688,9 +691,10 @@ posix:
 out:
         BD_STACK_UNWIND (open, frame, -1, ret, fd, NULL);
 
-        FREE (devpath);
+        GF_FREE (devpath);
         if (ret) {
-                close (_fd);
+                if (_fd >= 0)
+                        close (_fd);
                 GF_FREE (bd_fd);
         }
 
@@ -718,9 +722,8 @@ bd_do_fsync (int fd, int datasync)
 {
         int   op_errno = 0;
 
-#ifdef HAVE_FDATASYNC
         if (datasync) {
-                if (fdatasync (fd)) {
+                if (sys_fdatasync (fd)) {
                         op_errno = errno;
                         gf_log (THIS->name, GF_LOG_ERROR,
                                 "fdatasync on fd=%d failed: %s",
@@ -728,9 +731,9 @@ bd_do_fsync (int fd, int datasync)
                 }
 
         } else
-#endif
+
         {
-                if (fsync (fd)) {
+                if (sys_fsync (fd)) {
                         op_errno = errno;
                         gf_log (THIS->name, GF_LOG_ERROR,
                                 "fsync on fd=%d failed: %s",
@@ -793,7 +796,7 @@ bd_fsync (call_frame_t *frame, xlator_t *this,
                 local->bdatt->type = gf_strdup (bdatt->type);
                 memcpy (&local->bdatt->iatt, &bdatt->iatt, sizeof (struct iatt));
                 bd_update_amtime (&local->bdatt->iatt, valid);
-                uuid_copy (local->loc.gfid, fd->inode->gfid);
+                gf_uuid_copy (local->loc.gfid, fd->inode->gfid);
                 STACK_WIND (frame, bd_fsync_setattr_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->setattr, &local->loc,
                             &local->bdatt->iatt,
@@ -850,7 +853,7 @@ bd_flush (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
         BD_VALIDATE_MEM_ALLOC (local, op_errno, out);
 
         local->fd = fd_ref (fd);
-        uuid_copy (loc.gfid, bdatt->iatt.ia_gfid);
+        gf_uuid_copy (loc.gfid, bdatt->iatt.ia_gfid);
 
         /* Update the a|mtime during flush */
         STACK_WIND (frame, bd_flush_setattr_cbk, FIRST_CHILD (this),
@@ -1213,7 +1216,7 @@ bd_offload_dest_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 goto out;
         }
 
-        local->bdatt = CALLOC (1, sizeof (bd_attr_t));
+        local->bdatt = GF_CALLOC (1, sizeof (bd_attr_t), gf_bd_attr);
         BD_VALIDATE_MEM_ALLOC (local->bdatt, op_errno, out);
 
         STACK_WIND (frame, bd_offload_getx_cbk, FIRST_CHILD(this),
@@ -1266,7 +1269,7 @@ bd_do_merge(call_frame_t *frame, xlator_t *this)
                 op_errno = EINVAL;
                 goto out;
         }
-        uuid_copy (local->loc.pargfid, parent->gfid);
+        gf_uuid_copy (local->loc.pargfid, parent->gfid);
 
         p = strrchr (local->loc.path, '/');
         if (p)
@@ -1303,7 +1306,7 @@ bd_offload (call_frame_t *frame, xlator_t *this, loc_t *loc,
         local->dict = dict_new ();
         BD_VALIDATE_MEM_ALLOC (local->dict, op_errno, out);
 
-        local->dloc = CALLOC (1, sizeof (loc_t));
+        local->dloc = GF_CALLOC (1, sizeof (loc_t), gf_bd_loc_t);
         BD_VALIDATE_MEM_ALLOC (local->dloc, op_errno, out);
 
         strncpy (param, local->data->data, local->data->len);
@@ -1324,7 +1327,7 @@ bd_offload (call_frame_t *frame, xlator_t *this, loc_t *loc,
                 goto out;
         }
 
-        uuid_parse (gfid, local->dloc->gfid);
+        gf_uuid_parse (gfid, local->dloc->gfid);
         local->offload = offload;
 
         STACK_WIND (frame, bd_offload_dest_lookup_cbk, FIRST_CHILD (this),
@@ -1443,7 +1446,6 @@ bd_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *dict,
 
         bd_inode_ctx_get (fd->inode, this, &bdatt);
 
-        data =  dict_get (dict, BD_XATTR);
         if ((data = dict_get (dict, BD_XATTR)))
                 cl_type = BD_OF_NONE;
         else if ((data = dict_get (dict, BD_CLONE)))
@@ -1591,7 +1593,8 @@ revert_xattr:
         /* revert setxattr */
         op_ret = dict_get_str (local->dict, BD_XATTR, &bd);
         GF_FREE (bd);
-        gf_asprintf (&bd, "%s:%ld", bdatt->type, bdatt->iatt.ia_size);
+	if (bdatt)
+		gf_asprintf (&bd, "%s:%ld", bdatt->type, bdatt->iatt.ia_size);
 
         if (local->fd)
                 STACK_WIND (frame, bd_trunc_setxattr_setx_cbk,
@@ -1790,10 +1793,11 @@ __bd_pwritev (int fd, struct iovec *vector, int count, off_t offset,
 
         retval = pwritev (fd, vector, count, offset);
         if (retval == -1) {
+                int64_t off = offset;
                 gf_log (THIS->name, GF_LOG_WARNING,
-                        "base %p, length %ld, offset %ld, message %s",
+                        "base %p, length %zd, offset %" PRId64 ", message %s",
                         vector[index].iov_base, vector[index].iov_len,
-                        internal_offset, strerror (errno));
+                        off, strerror (errno));
                 retval = -errno;
                 goto err;
         }
@@ -1923,7 +1927,7 @@ bd_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
 
         memcpy (postbuf, &bdatt->iatt, sizeof (struct iatt));
 out:
-        FREE (valid);
+        GF_FREE (valid);
         BD_STACK_UNWIND (setattr, frame, op_ret, op_errno, prebuf,
                          postbuf, xdata);
         return 0;
@@ -1948,7 +1952,7 @@ bd_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc, struct iatt *stbuf,
         local = bd_local_init (frame, this);
         BD_VALIDATE_MEM_ALLOC (local, op_errno, out);
 
-        ck_valid = CALLOC (1, sizeof (valid));
+        ck_valid = GF_CALLOC (1, sizeof (valid), gf_bd_int32_t);
         BD_VALIDATE_MEM_ALLOC (ck_valid, op_errno, out);
 
         local->inode = inode_ref (loc->inode);
@@ -2268,6 +2272,7 @@ mem_acct_init (xlator_t *this)
         if (ret != 0)
                 gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
                        "failed");
+
         return ret;
 }
 
