@@ -80,6 +80,7 @@ static char* list_xattr_ignore_xattrs[] = {
         GFID_XATTR_KEY,
         NULL
 };
+
 gf_boolean_t
 posix_special_xattr (char **pattern, char *key)
 {
@@ -120,29 +121,77 @@ posix_xattr_ignorable (char *key)
         return _is_in_array (posix_ignore_xattrs, key);
 }
 
+static gf_boolean_t
+posix_is_valid_namespace (char *key)
+{
+        static char *xattr_namespaces[] = {"trusted.", "security.", "system.",
+                                           "user.", NULL };
+        int i = 0;
+
+        for (i = 0; xattr_namespaces[i]; i++) {
+                if (strncmp (key, xattr_namespaces[i],
+                             strlen (xattr_namespaces[i])) == 0)
+                        return _gf_true;
+        }
+
+        return _gf_false;
+}
+
 static int
 _posix_xattr_get_set_from_backend (posix_xattr_filler_t *filler, char *key)
 {
         ssize_t  xattr_size = -1;
         int      ret        = 0;
-        char    *value      = NULL;
+        char     *value     = NULL;
+        char     val_buf[256] = {0};
+        gf_boolean_t have_val   = _gf_false;
 
+        if (!posix_is_valid_namespace (key)) {
+                ret = -1;
+                goto out;
+        }
+
+        /* Most of the gluster internal xattrs don't exceed 256 bytes. So try
+         * getxattr with ~256 bytes. If it gives ERANGE then go the old way
+         * of getxattr with NULL buf to find the length and then getxattr with
+         * allocated buf to fill the data. This way we reduce lot of getxattrs.
+         */
         if (filler->real_path)
-                xattr_size = sys_lgetxattr (filler->real_path, key, NULL, 0);
+                xattr_size = sys_lgetxattr (filler->real_path, key, val_buf,
+                                            sizeof (val_buf) - 1);
         else
+                xattr_size = sys_fgetxattr (filler->fdnum, key, val_buf,
+                                            sizeof (val_buf) - 1);
+
+        if (xattr_size >= 0) {
+                have_val = _gf_true;
+        } else if (xattr_size == -1 && errno != ERANGE) {
+                ret = -1;
+                goto out;
+        }
+
+        if (have_val) {
+                /*No need to do getxattr*/
+        } else if (filler->real_path) {
+                xattr_size = sys_lgetxattr (filler->real_path, key, NULL, 0);
+        } else {
                 xattr_size = sys_fgetxattr (filler->fdnum, key, NULL, 0);
+        }
 
         if (xattr_size != -1) {
                 value = GF_CALLOC (1, xattr_size + 1, gf_posix_mt_char);
                 if (!value)
                         goto out;
 
-                if (filler->real_path)
+                if (have_val) {
+                        memcpy (value, val_buf, xattr_size);
+                } else if (filler->real_path) {
                         xattr_size = sys_lgetxattr (filler->real_path, key,
                                                     value, xattr_size);
-                else
+                } else {
                         xattr_size = sys_fgetxattr (filler->fdnum, key, value,
                                                     xattr_size);
+                }
                 if (xattr_size == -1) {
                         if (filler->real_path)
                                 gf_msg (filler->this->name, GF_LOG_WARNING, 0,
@@ -518,8 +567,6 @@ posix_fdstat (xlator_t *this, int fd, struct iatt *stbuf_p)
         iatt_from_stat (&stbuf, &fstatbuf);
 
         ret = posix_fill_gfid_fd (this, fd, &stbuf);
-        if (ret)
-                gf_log_callingfn (this->name, GF_LOG_DEBUG, "failed to get gfid");
 
         posix_fill_ino_from_gfid (this, &stbuf);
 
@@ -1072,7 +1119,7 @@ void posix_dump_buffer (xlator_t *this, const char *real_path, const char *key,
 
 int
 posix_handle_pair (xlator_t *this, const char *real_path,
-                   char *key, data_t *value, int flags)
+                   char *key, data_t *value, int flags, struct iatt *stbuf)
 {
         int sys_ret = -1;
         int ret     = 0;
@@ -1084,7 +1131,12 @@ posix_handle_pair (xlator_t *this, const char *real_path,
                 ret = posix_set_file_contents (this, real_path, key, value,
                                                flags);
         } else if (GF_POSIX_ACL_REQUEST (key)) {
+                if (stbuf && IS_DHT_LINKFILE_MODE (stbuf))
+                        goto out;
                 ret = posix_pacl_set (real_path, key, value->data);
+        } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR, strlen(key))
+                   && stbuf && IS_DHT_LINKFILE_MODE (stbuf)) {
+                goto out;
         } else {
                 sys_ret = sys_lsetxattr (real_path, key, value->data,
                                          value->len, flags);
@@ -1135,7 +1187,7 @@ out:
 
 int
 posix_fhandle_pair (xlator_t *this, int fd,
-                    char *key, data_t *value, int flags)
+                    char *key, data_t *value, int flags, struct iatt *stbuf)
 {
         int sys_ret = -1;
         int ret     = 0;
@@ -1143,6 +1195,9 @@ posix_fhandle_pair (xlator_t *this, int fd,
         if (XATTR_IS_PATHINFO (key)) {
                 ret = -EACCES;
                 goto out;
+        } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR, strlen(key))
+                   && stbuf && IS_DHT_LINKFILE_MODE (stbuf)) {
+                goto out;
         }
 
         sys_ret = sys_fsetxattr (fd, key, value->data,
@@ -1525,7 +1580,7 @@ _handle_entry_create_keyvalue_pair (dict_t *d, char *k, data_t *v,
         }
 
         ret = posix_handle_pair (filler->this, filler->real_path, k, v,
-                                 XATTR_CREATE);
+                                 XATTR_CREATE, filler->stbuf);
         if (ret < 0) {
                 errno = -ret;
                 return -1;
@@ -1546,6 +1601,7 @@ posix_entry_create_xattr_set (xlator_t *this, const char *path,
 
         filler.this = this;
         filler.real_path = path;
+        filler.stbuf = NULL;
 
         ret = dict_foreach (dict, _handle_entry_create_keyvalue_pair, &filler);
 
