@@ -15,12 +15,32 @@
 
 #include "posix-acl.h"
 #include "posix-acl-xattr.h"
+#include "posix-acl-mem-types.h"
 
 
 #define UINT64(ptr) ((uint64_t)((long)(ptr)))
 #define PTR(num) ((void *)((long)(num)))
 
 
+int32_t
+mem_acct_init (xlator_t *this)
+{
+        int     ret = -1;
+
+        if (!this)
+                return ret;
+
+        ret = xlator_mem_acct_init (this, gf_posix_acl_mt_end + 1);
+
+        if (ret != 0) {
+                gf_log(this->name, GF_LOG_ERROR, "Memory accounting init"
+                       "failed");
+                return ret;
+        }
+
+        return ret;
+}
+
 static uid_t
 r00t ()
 {
@@ -277,7 +297,7 @@ posix_acl_ctx_get (inode_t *inode, xlator_t *this)
         if ((ret == 0) && (int_ctx))
                 return PTR(int_ctx);
 
-        ctx = CALLOC (1, sizeof (*ctx));
+        ctx = GF_CALLOC (1, sizeof (*ctx), gf_posix_acl_mt_ctx_t);
         if (!ctx)
                 return NULL;
 
@@ -286,6 +306,26 @@ posix_acl_ctx_get (inode_t *inode, xlator_t *this)
         return ctx;
 }
 
+int
+__posix_acl_set_specific (inode_t *inode, xlator_t *this,
+                          gf_boolean_t is_access, struct posix_acl *acl)
+{
+        int                    ret = 0;
+        struct posix_acl_ctx  *ctx = NULL;
+
+        ctx = posix_acl_ctx_get (inode, this);
+        if (!ctx) {
+                ret = -1;
+                goto out;
+        }
+
+        if (is_access)
+                ctx->acl_access = acl;
+        else
+                ctx->acl_default = acl;
+out:
+        return ret;
+}
 
 int
 __posix_acl_set (inode_t *inode, xlator_t *this, struct posix_acl *acl_access,
@@ -333,7 +373,8 @@ posix_acl_new (xlator_t *this, int entrycnt)
         struct posix_acl *acl = NULL;
         struct posix_ace *ace = NULL;
 
-        acl = CALLOC (1, sizeof (*acl) + (entrycnt * sizeof (*ace)));
+        acl = GF_CALLOC (1, sizeof (*acl) + (entrycnt * sizeof (*ace)),
+                         gf_posix_acl_mt_posix_ace_t);
         if (!acl)
                 return NULL;
 
@@ -348,7 +389,7 @@ posix_acl_new (xlator_t *this, int entrycnt)
 void
 posix_acl_destroy (xlator_t *this, struct posix_acl *acl)
 {
-        FREE (acl);
+       GF_FREE (acl);
 
         return;
 }
@@ -405,6 +446,36 @@ posix_acl_unref (xlator_t *this, struct posix_acl *acl)
                 posix_acl_destroy (this, acl);
 }
 
+int
+posix_acl_set_specific (inode_t *inode, xlator_t *this, struct posix_acl *acl,
+                        gf_boolean_t is_access)
+{
+        int                     ret = 0;
+        int                     oldret = 0;
+        struct posix_acl       *old_acl = NULL;
+        struct posix_acl_conf  *conf = NULL;
+
+        conf = this->private;
+
+        LOCK (&conf->acl_lock);
+        {
+                if (is_access)
+                        oldret = __posix_acl_get (inode, this, &old_acl, NULL);
+                else
+                        oldret = __posix_acl_get (inode, this, NULL, &old_acl);
+                if (acl)
+                        acl->refcnt++;
+                ret = __posix_acl_set_specific (inode, this, is_access, acl);
+        }
+        UNLOCK (&conf->acl_lock);
+
+        if (oldret == 0) {
+                if (old_acl)
+                        posix_acl_unref (this, old_acl);
+        }
+
+        return ret;
+}
 
 int
 posix_acl_set (inode_t *inode, xlator_t *this, struct posix_acl *acl_access,
@@ -476,7 +547,6 @@ unlock:
         return ret;
 }
 
-
 mode_t
 posix_acl_inherit_mode (struct posix_acl *acl, mode_t modein)
 {
@@ -577,7 +647,7 @@ posix_acl_inherit (xlator_t *this, loc_t *loc, dict_t *params, mode_t mode,
         ctx->perm = retmode;
 
         size_access = posix_acl_to_xattr (this, acl_access, NULL, 0);
-        xattr_access = CALLOC (1, size_access);
+        xattr_access = GF_CALLOC (1, size_access, gf_posix_acl_mt_char);
         if (!xattr_access) {
                 gf_log (this->name, GF_LOG_ERROR, "out of memory");
                 ret = -1;
@@ -600,7 +670,7 @@ posix_acl_inherit (xlator_t *this, loc_t *loc, dict_t *params, mode_t mode,
         acl_default = posix_acl_ref (this, par_default);
 
         size_default = posix_acl_to_xattr (this, acl_default, NULL, 0);
-        xattr_default = CALLOC (1, size_default);
+        xattr_default = GF_CALLOC (1, size_default, gf_posix_acl_mt_char);
         if (!xattr_default) {
                 gf_log (this->name, GF_LOG_ERROR, "out of memory");
                 ret = -1;
@@ -924,18 +994,29 @@ int
 posix_acl_truncate (call_frame_t *frame, xlator_t *this, loc_t *loc, off_t off,
                     dict_t *xdata)
 {
+        struct posix_acl_ctx *ctx = NULL;
+
         if (acl_permits (frame, loc->inode, POSIX_ACL_WRITE))
                 goto green;
-        else
-                goto red;
+        /* NFS does a truncate through SETATTR, the owner does not need write
+         * permissions for this. Group permissions and root are checked above.
+         */
+        else if (frame->root->pid == NFS_PID) {
+                ctx = posix_acl_ctx_get (loc->inode, frame->this);
+
+                if (ctx && frame_is_user (frame, ctx->uid))
+                        goto green;
+        }
+
+        /* fail by default */
+        STACK_UNWIND_STRICT (truncate, frame, -1, EACCES, NULL, NULL, NULL);
+        return 0;
+
 green:
         STACK_WIND (frame, posix_acl_truncate_cbk,
                     FIRST_CHILD(this), FIRST_CHILD(this)->fops->truncate,
                     loc, off, xdata);
         return 0;
-red:
-        STACK_UNWIND_STRICT (truncate, frame, -1, EACCES, NULL, NULL, NULL);
-        return 0;
 }
 
 
@@ -1851,11 +1932,72 @@ posix_acl_setxattr_update (xlator_t *this, inode_t *inode, dict_t *xattr)
         return ret;
 }
 
+/* *
+ * Posix acl can be set using other xattr such as GF_POSIX_ACL_ACCESS/
+ * GF_POSIX_ACL_DEFAULT  which requires to update the context of
+ * access-control translator
+ */
+int
+handling_other_acl_related_xattr (xlator_t *this, inode_t *inode, dict_t *xattr)
+{
+        struct posix_acl       *acl      = NULL;
+        struct posix_acl_ctx   *ctx      = NULL;
+        data_t                 *data     = NULL;
+        int                     ret      = 0;
 
+        data = dict_get (xattr, POSIX_ACL_ACCESS_XATTR);
+        if (data) {
+
+
+                acl = posix_acl_from_xattr (this, data->data, data->len);
+                if (!acl) {
+                        ret = -1;
+                        goto out;
+                }
+
+                ret = posix_acl_set_specific (inode, this, acl, _gf_true);
+                if (ret)
+                        goto out;
+
+                ctx = posix_acl_ctx_get (inode, this);
+                if (!ctx) {
+                        ret = -1;
+                        goto out;
+                }
+
+                posix_acl_access_set_mode (acl, ctx);
+                posix_acl_unref (this, acl);
+                acl = NULL;
+        }
+
+        data = dict_get (xattr, POSIX_ACL_DEFAULT_XATTR);
+        if (data) {
+                acl = posix_acl_from_xattr (this, data->data, data->len);
+                if (!acl) {
+                        ret = -1;
+                        goto out;
+                }
+
+                ret = posix_acl_set_specific (inode, this, acl, _gf_false);
+        }
+
+out:
+        if (acl)
+                posix_acl_unref (this, acl);
+
+        return ret;
+}
 int
 posix_acl_setxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         int op_ret, int op_errno, dict_t *xdata)
 {
+
+        /*
+         * Update the context of posix_acl_translator, if any of
+         * POSIX_ACL_*_XATTR set in the call back
+         */
+        handling_other_acl_related_xattr (this, (inode_t *)cookie, xdata);
+
         STACK_UNWIND_STRICT (setxattr, frame, op_ret, op_errno, xdata);
 
         return 0;
@@ -1875,9 +2017,13 @@ posix_acl_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
 
         posix_acl_setxattr_update (this, loc->inode, xattr);
 
-        STACK_WIND (frame, posix_acl_setxattr_cbk,
-                    FIRST_CHILD(this), FIRST_CHILD(this)->fops->setxattr,
-                    loc, xattr, flags, xdata);
+        /*
+         * inode is required in call back function to update the context
+         * this translator
+         */
+        STACK_WIND_COOKIE (frame, posix_acl_setxattr_cbk, loc->inode,
+                           FIRST_CHILD(this), FIRST_CHILD(this)->fops->setxattr,
+                           loc, xattr, flags, xdata);
         return 0;
 red:
         STACK_UNWIND_STRICT (setxattr, frame, -1, op_errno, xdata);
@@ -1941,11 +2087,13 @@ posix_acl_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
                 goto green;
         else
                 goto red;
+
 green:
         STACK_WIND (frame, posix_acl_getxattr_cbk,
                     FIRST_CHILD(this), FIRST_CHILD(this)->fops->getxattr,
                     loc, name, xdata);
         return 0;
+
 red:
         STACK_UNWIND_STRICT (getxattr, frame, -1, EACCES, NULL, xdata);
 
@@ -2050,7 +2198,7 @@ posix_acl_forget (xlator_t *this, inode_t *inode)
         if (ctx->acl_default)
                 posix_acl_unref (this, ctx->acl_default);
 
-        FREE (ctx);
+        GF_FREE (ctx);
 out:
         return 0;
 }
@@ -2078,7 +2226,7 @@ init (xlator_t *this)
         struct posix_acl        *minacl = NULL;
         struct posix_ace        *minace = NULL;
 
-        conf = CALLOC (1, sizeof (*conf));
+        conf = GF_CALLOC (1, sizeof (*conf), gf_posix_acl_mt_conf_t);
         if (!conf) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "out of memory");
